@@ -9,6 +9,12 @@
 #include "analyser/analyser.h"
 #include "lib/fastbuf.h"
 
+static void
+an_test_init(struct an_hook *h)
+{
+  h->need_mask = AN_NEED_TEXT | AN_NEED_METAS;
+}
+
 static int
 an_test_need(struct an_hook *h UNUSED, struct an_iface *ai)
 {
@@ -27,7 +33,7 @@ an_test_analyse(struct an_hook *h UNUSED, struct an_iface *ai)
 
 struct analyser an_test = {
   .name = "test",
+  .init = an_test_init,
   .need = an_test_need,
-  .analyse = an_test_analyse,
-  .need_mask = AN_NEED_TEXT | AN_NEED_METAS
+  .analyse = an_test_analyse
 };
