@@ -11,7 +11,7 @@
 #include "charset/unicat.h"
 #include "charset/U-ligatures.h"
 
-const word *
+const u16 *
 Uexpand_lig(uns x)
 {
   return _U_lig_hash[x % LIG_HASH_SIZE];
