@@ -1,19 +1,26 @@
 #!/usr/bin/perl
 # Find the call tree of memory allocations (using cscope)
 # (c) 2004 Martin Mares <mj@ucw.cz>
+# (c) 2006 Robert Spalek <robert@ucw.cz>
 
 # The set of allocator calls we want to trace
 my @queue = (
-	"xmalloc", "xmalloc_zero", "sh_xmalloc", "xfree", "free",
+	"xmalloc", "xmalloc_zero", "sh_xmalloc", "xfree", "free", "sh_xfree",
 	"mmap_file", "munmap_file",
 	"mp_new", "mp_delete",
-	"resolve_fingerprints"
+	"alloc_read_ary", "write_free_ary",
+	"attrs_map", "attrs_unmap",
+	"resolve_fastbuf",
+	"bb_grow",
+	#"partmap_open", "partmap_close",
+	#"notes_part_map", "notes_part_unmap", "notes_skel_part_map", "notes_skel_part_unmap",
+	#"attrs_part_map", "attrs_part_unmap",
 );
 my %known = map { $_ => 2 } @queue;
 
 # Preprocess the source and remove gcc attributes, because they confuse cscope
 my $src = join(" ", @ARGV);
-`gcc -C -E -P -I. $src | sed 's/__attribute__ *((.*))//g' >allocs.tmp`;
+`gcc -C -E -P -I. -Iobj $src | sed 's/__attribute__ *((.*))//g' >allocs.tmp`;
 $? && die;
 $src = "allocs.tmp";
 unlink "cscope.out";
@@ -27,7 +34,7 @@ close X;
 # Search the call graph backwards
 while (@queue) {
 	my $f = shift @queue;
-	my @r = `cscope -I. -L -3 $f $src`;
+	my @r = `cscope -I. -Iobj -L -3 $f $src`;
 	$? && die "cscope failed";
 	foreach my $l (@r) {
 		my ($ffile, $ffunc, $fline, $fctxt) = ($l =~ /^(\S+)\s+(\S+)\s+(\S+)\s*(.*)/) or die;
