@@ -23,7 +23,7 @@ byte *lng_parse(struct query *q, enum custom_op op, byte *val, uns intval)
   return NULL;
 }
 
-int custom_match(struct query *q, struct ref_context *c, struct card_attr *ca, int *QQ)
+int custom_match(struct query *q, struct ref_context *c UNUSED, struct card_attr *ca, int *QQ)
 {
   q->lng_value = CA_GET_FILE_LANG(ca);
   if (q->lng_only >= 0 && (int)q->lng_value != q->lng_only)
