@@ -1,6 +1,6 @@
 /* Customization file for the matcher-test config */
 
-#include "bare/lib/custom.h"
+#include "customs/bare/lib/custom.h"
 
 #define CUSTOM_STAT_VARS u32 lng_defined_count;
 #define CUSTOM_INIT_STATS(q,s) s->lng_defined_count=0;
