@@ -51,7 +51,7 @@ fv_replace(struct filter_args *args, struct filter_value *ret, struct filter_val
 	ret->undef = 1;
 	if (arg[0].undef || arg[1].undef || arg[2].undef)
 		return;
-	res = rx_subst(arg[1].v.r->regex, arg[2].v.s, arg[0].v.s, buf, MAX_REGEX_BUF);
+	res = rx_subst(args->filter->lookup[arg[1].v.i].regex->regex, arg[2].v.s, arg[0].v.s, buf, MAX_REGEX_BUF);
 	if (res < 0)
 	{
 		log(L_ERROR, "Cannot perform fv_replace, buffer of length %d is too short for %s -> %s", MAX_REGEX_BUF, arg[0].v.s, arg[2].v.s);
@@ -100,12 +100,31 @@ fv_standardize_domain(struct filter_args *args UNUSED, struct filter_value *ret,
 	strcpy(ret->v.s+1, domain);
 }
 
+static void
+fv_cut_at(struct filter_args *args UNUSED, struct filter_value *ret, struct filter_value arg[MAX_FUNC_ARGS])
+{
+	if (arg[0].undef || arg[1].undef)
+	{
+		ret->undef = 1;
+		return;
+	}
+	byte *p = strrchr(arg[0].v.s, *arg[1].v.s);
+	if (p)
+	{
+		ret->v.s = mp_strdup(args->pool, p);
+		ret->undef = 0;
+	}
+	else
+		ret->undef = 1;
+}
+
 struct filter_function filter_builtin_func[] = {
 	{ "toint",	1, {F_ET_STRING},				F_ET_INT,	fv_str2int },
 	{ "replace",	4, {F_ET_STRING, F_ET_REGEXP, F_ET_STRING, F_ET_STRING},
 									F_ET_STRING,	fv_replace },
 	{ "has_repeated_component",	1,	{F_ET_STRING},		F_ET_INT,	fv_has_repeated_component },
 	{ "standardize_domain",		1,	{F_ET_STRING},		F_ET_STRING,	fv_standardize_domain },
+	{ "cut_at",	2, {F_ET_STRING, F_ET_STRING},			F_ET_STRING,	fv_cut_at },
 	CUSTOM_FILTER_BUILTINS
 	{ NULL,		0, {},						0,		NULL }
 };
@@ -155,6 +174,11 @@ struct filter_variable filter_builtin_vars[] = {
   { "image_size",	F_LVC_RAW,	F_ET_INT,	1 },
   { "image_aspect_ratio",F_LVC_RAW,	F_ET_INT,	1 },
   { "image_colors",	F_LVC_RAW,	F_ET_INT,	1 },
+  { "audio_length",	F_LVC_RAW,	F_ET_INT,	1 },
+  { "audio_bitrate",	F_LVC_RAW,	F_ET_INT,	1 },
+  { "audio_srate",	F_LVC_RAW,	F_ET_INT,	1 },
+  { "audio_channels",	F_LVC_RAW,	F_ET_INT,	1 },
+  
   /* Area attributes */
   { "area",		F_LVC_RAW,	F_ET_INT,	0 },
   /* Extended attributes, all attributes not listed explicitly are of type
