@@ -70,7 +70,7 @@ dump_variable_name(struct fastbuf *b, struct filter_lvalue *lval)
 }
 
 static void
-dump_expression(struct fastbuf *b, struct filter_expr *e)
+dump_expression(struct fastbuf *b, struct filter *f, struct filter_expr *e)
 {
 	int i;
 	switch (e->cat)
@@ -81,14 +81,17 @@ dump_expression(struct fastbuf *b, struct filter_expr *e)
 				case F_ET_INT: bprintf(b, "%d", e->o.i); break;
 				case F_ET_STRING: bprintf(b, "\"%s\"", e->o.s); break;
 				case F_ET_REGEXP:
-					if (e->o.r->precompiled)
+				{
+					struct filter_regex_value *rv = f->lookup[e->o.i].regex;
+					if (rv->precompiled)
 					{
 						bputc(b, '~');
-						if (e->o.r->icase)
+						if (rv->icase)
 							bputc(b, '~');
 					}
-					bprintf(b, "\"%s\"", e->o.r->source);
+					bprintf(b, "\"%s\"", rv->source);
 					break;
+				}
 				default: ASSERT(0);
 			}
 			break;
@@ -97,12 +100,12 @@ dump_expression(struct fastbuf *b, struct filter_expr *e)
 			break;
 		case F_EC_UNOP:
 			bputc(b, e->o.un.op);
-			dump_expression(b, e->o.un.r);
+			dump_expression(b, f, e->o.un.r);
 			break;
 		case F_EC_BINOP:
 			if (e->o.bin.op != INTERVAL)
 				bputs(b, " (");
-			dump_expression(b, e->o.bin.l);
+			dump_expression(b, f, e->o.bin.l);
 			if (e->o.bin.op != INTERVAL)
 			{
 				bputc(b, ' ');
@@ -111,7 +114,7 @@ dump_expression(struct fastbuf *b, struct filter_expr *e)
 			}
 			else
 				bputs(b, " .. ");
-			dump_expression(b, e->o.bin.r);
+			dump_expression(b, f, e->o.bin.r);
 			if (e->o.bin.op != INTERVAL)
 				bputs(b, ") ");
 			break;
@@ -120,7 +123,7 @@ dump_expression(struct fastbuf *b, struct filter_expr *e)
 			bputc(b, '(');
 			for (i=0; i<e->o.func.func->args; i++)
 			{
-				dump_expression(b, e->o.func.a[i]);
+				dump_expression(b, f, e->o.func.a[i]);
 				if (i < e->o.func.func->args-1)
 					bputs(b, ", ");
 			}
@@ -132,7 +135,7 @@ dump_expression(struct fastbuf *b, struct filter_expr *e)
 }
 
 static void
-dump_condition(struct fastbuf *b, struct filter_cond *c)
+dump_condition(struct fastbuf *b, struct filter *f, struct filter_cond *c)
 {
 	byte *msg = NULL;
 	switch (c->cat)
@@ -150,7 +153,7 @@ dump_condition(struct fastbuf *b, struct filter_cond *c)
 			if (c->o.expr.l)
 			{
 				bputs(b, " (");
-				dump_expression(b, c->o.expr.l);
+				dump_expression(b, f, c->o.expr.l);
 			}
 			bputc(b, ' ');
 			if (!c->o.expr.neg)
@@ -183,27 +186,27 @@ dump_condition(struct fastbuf *b, struct filter_cond *c)
 			if (c->o.expr.icase)
 				bputc(b, msg[strlen(msg) - 1]);
 			bputc(b, ' ');
-			dump_expression(b, c->o.expr.r);
+			dump_expression(b, f, c->o.expr.r);
 			if (c->o.expr.l)
 				bputs(b, ") ");
 			break;
 		case F_CC_DEFCOND:
 			bputs(b, "defined(");
-			dump_condition(b, c->o.neg);
+			dump_condition(b, f, c->o.neg);
 			bputs(b, ") ");
 			break;
 		case F_CC_DEFEXPR:
 			bputs(b, "defined(");
-			dump_expression(b, c->o.defexpr);
+			dump_expression(b, f, c->o.defexpr);
 			bputs(b, ") ");
 			break;
 		case F_CC_UNOP:
 			bputc(b, '!');
-			dump_condition(b, c->o.neg);
+			dump_condition(b, f, c->o.neg);
 			break;
 		case F_CC_BINOP:
 			bputs(b, " (");
-			dump_condition(b, c->o.bin.l);
+			dump_condition(b, f, c->o.bin.l);
 			switch (c->o.bin.op)
 			{
 				case AND: bputs(b, " && "); break;
@@ -212,7 +215,7 @@ dump_condition(struct fastbuf *b, struct filter_cond *c)
 				case NE: bputs(b, " != "); break;
 				default: ASSERT(0);
 			}
-			dump_condition(b, c->o.bin.r);
+			dump_condition(b, f, c->o.bin.r);
 			bputs(b, ") ");
 			break;
 		default:
@@ -221,11 +224,11 @@ dump_condition(struct fastbuf *b, struct filter_cond *c)
 }
 
 static void
-dump_hash_chain(struct fastbuf *b, struct filter_hash_record *hr, byte *op, uns level)
+dump_hash_chain(struct fastbuf *b, struct filter *f, struct filter_hash_record *hr, byte *op, uns level)
 {
 	if (!hr)
 		return;
-	dump_hash_chain(b, hr->next, op, level);
+	dump_hash_chain(b, f, hr->next, op, level);
 	/* The list is printed in the reverse order so that a dumped and reread
 	 * filter stays the same.  */
 	filter_dump_spaces(b, level);
@@ -234,11 +237,11 @@ dump_hash_chain(struct fastbuf *b, struct filter_hash_record *hr, byte *op, uns
 	bputs(b, " \"");
 	bputs(b, hr->string);
 	bputs(b, "\":\n");
-	filter_dump_commands(b, hr->cmd, level+1);
+	filter_dump_commands(b, f, hr->cmd, level+1);
 }
 
 static void
-dump_switch_hash(struct fastbuf *b, struct filter_hash_table *ht, byte *op, uns level)
+dump_switch_hash(struct fastbuf *b, struct filter *f, struct filter_hash_table *ht, byte *op, uns level)
 {
 	uns i;
 	if (!ht)
@@ -246,11 +249,11 @@ dump_switch_hash(struct fastbuf *b, struct filter_hash_table *ht, byte *op, uns
 	filter_dump_spaces(b, level);
 	bprintf(b, "# hash-table for the operator %s\n", op);
 	for (i=0; i<ht->size; i++)
-		dump_hash_chain(b, ht->h[i], op, level);
+		dump_hash_chain(b, f, ht->h[i], op, level);
 }
 
 static void
-dump_switch_trie(struct fastbuf *b, struct filter_trie_table *trie, byte *op, uns level)
+dump_switch_trie(struct fastbuf *b, struct filter *f, struct filter_trie_table *trie, byte *op, uns level)
 {
 	if (!trie)
 		return;
@@ -265,18 +268,18 @@ dump_switch_trie(struct fastbuf *b, struct filter_trie_table *trie, byte *op, un
 		bputs(b, " \"");
 		bputs(b, cas->cond->o.expr.r->o.s);
 		bputs(b, "\":\n");
-		filter_dump_commands(b, cas->positive, level+1);
+		filter_dump_commands(b, f, cas->positive, level+1);
 	}
 	if (filter_trace > 0)
 	{
 		bputs(b, "/*\n");
-		filter_trie_dump(b, trie);
+		filter_trie_dump(b, f, trie);
 		bputs(b, "*/\n");
 	}
 }
 
 static void
-dump_switch_cases(struct fastbuf *b, struct filter_case *fc, uns level)
+dump_switch_cases(struct fastbuf *b, struct filter *f, struct filter_case *fc, uns level)
 {
 	if (fc)
 	{
@@ -287,14 +290,14 @@ dump_switch_cases(struct fastbuf *b, struct filter_case *fc, uns level)
 	{
 		filter_dump_spaces(b, level);
 		bputs(b, "case ");
-		dump_condition(b, fc->cond);
+		dump_condition(b, f, fc->cond);
 		bputs(b, ":\n");
-		filter_dump_commands(b, fc->positive, level+1);
+		filter_dump_commands(b, f, fc->positive, level+1);
 	}
 }
 
 void
-filter_dump_commands(struct fastbuf *b, struct filter_cmd *cmd, uns level)
+filter_dump_commands(struct fastbuf *b, struct filter *f, struct filter_cmd *cmd, uns level)
 {
 	for (; cmd; cmd=cmd->next)
 	{
@@ -315,7 +318,7 @@ filter_dump_commands(struct fastbuf *b, struct filter_cmd *cmd, uns level)
 				}
 				bputs(b, msg);
 				bputc(b, ' ');
-				dump_expression(b, cmd->c.print.expr);
+				dump_expression(b, f, cmd->c.print.expr);
 				bputs(b, ";\n");
 				break;
 			case ACCEPT:
@@ -328,7 +331,7 @@ filter_dump_commands(struct fastbuf *b, struct filter_cmd *cmd, uns level)
 				if (cmd->c.print.expr)
 				{
 					bputc(b, ' ');
-					dump_expression(b, cmd->c.print.expr);
+					dump_expression(b, f, cmd->c.print.expr);
 				}
 				bputs(b, ";\n");
 				break;
@@ -340,7 +343,7 @@ filter_dump_commands(struct fastbuf *b, struct filter_cmd *cmd, uns level)
 					bputs(b, msg);
 				dump_variable_name(b, cmd->c.set.lv);
 				bputs(b, " = ");
-				dump_expression(b, cmd->c.set.expr);
+				dump_expression(b, f, cmd->c.set.expr);
 				bputs(b, ";\n");
 				break;
 			case DELETE:
@@ -350,22 +353,22 @@ filter_dump_commands(struct fastbuf *b, struct filter_cmd *cmd, uns level)
 				break;
 			case IF:
 				bputs(b, "if ");
-				dump_condition(b, cmd->c.cond.cond);
+				dump_condition(b, f, cmd->c.cond.cond);
 				bputs(b, " {\n");
-				filter_dump_commands(b, cmd->c.cond.positive, level+1);
+				filter_dump_commands(b, f, cmd->c.cond.positive, level+1);
 				filter_dump_spaces(b, level);
 				bputc(b, '}');
 				if (cmd->c.cond.negative)
 				{
 					bputs(b, " else {\n");
-					filter_dump_commands(b, cmd->c.cond.negative, level+1);
+					filter_dump_commands(b, f, cmd->c.cond.negative, level+1);
 					filter_dump_spaces(b, level);
 					bputc(b, '}');
 				}
 				if (cmd->c.cond.undefined)
 				{
 					bputs(b, " undef {\n");
-					filter_dump_commands(b, cmd->c.cond.undefined, level+1);
+					filter_dump_commands(b, f, cmd->c.cond.undefined, level+1);
 					filter_dump_spaces(b, level);
 					bputc(b, '}');
 				}
@@ -373,36 +376,36 @@ filter_dump_commands(struct fastbuf *b, struct filter_cmd *cmd, uns level)
 				break;
 			case SWITCH:
 				bputs(b, "switch ");
-				dump_expression(b, cmd->c.swit.expr);
+				dump_expression(b, f, cmd->c.swit.expr);
 				bputs(b, " {\n");
-				dump_switch_hash(b, cmd->c.swit.cmp, "==", level+1);
-				dump_switch_hash(b, cmd->c.swit.icmp, "===", level+1);
-				dump_switch_trie(b, cmd->c.swit.pat, "=*", level+1);
-				dump_switch_trie(b, cmd->c.swit.ipat, "=**", level+1);
+				dump_switch_hash(b, f, cmd->c.swit.cmp, "==", level+1);
+				dump_switch_hash(b, f, cmd->c.swit.icmp, "===", level+1);
+				dump_switch_trie(b, f, f->lookup[cmd->c.swit.pat].trie, "=*", level+1);
+				dump_switch_trie(b, f, f->lookup[cmd->c.swit.ipat].trie, "=**", level+1);
 				if (cmd->c.swit.expr->type == F_ET_STRING)
 				{
-					filter_s_tree_dump(b, cmd->c.swit.bins, level+1);
-					filter_is_tree_dump(b, cmd->c.swit.binis, level+1);
+					filter_s_tree_dump(b, f, cmd->c.swit.bins, level+1);
+					filter_is_tree_dump(b, f, cmd->c.swit.binis, level+1);
 				}
 				else
 				{
-					filter_ud_tree_dump(b, cmd->c.swit.binud, level+1);
-					filter_d_tree_dump(b, cmd->c.swit.bind, level+1);
+					filter_ud_tree_dump(b, f, cmd->c.swit.binud, level+1);
+					filter_d_tree_dump(b, f, cmd->c.swit.bind, level+1);
 				}
-				dump_switch_cases(b, cmd->c.swit.cases, level+1);
+				dump_switch_cases(b, f, cmd->c.swit.cases, level+1);
 				filter_dump_spaces(b, level);
 				bputc(b, '}');
 				if (cmd->c.swit.negative)
 				{
 					bputs(b, " else {\n");
-					filter_dump_commands(b, cmd->c.swit.negative, level+1);
+					filter_dump_commands(b, f, cmd->c.swit.negative, level+1);
 					filter_dump_spaces(b, level);
 					bputc(b, '}');
 				}
 				if (cmd->c.swit.undefined)
 				{
 					bputs(b, " undef {\n");
-					filter_dump_commands(b, cmd->c.swit.undefined, level+1);
+					filter_dump_commands(b, f, cmd->c.swit.undefined, level+1);
 					filter_dump_spaces(b, level);
 					bputc(b, '}');
 				}
@@ -422,5 +425,5 @@ filter_dump(struct fastbuf *b, struct filter *f)
 	bputs(b, "\n# Context variables\n");
 	dump_declarations(b, f->deleted_decl);
 	bputs(b, "\n# Program\n");
-	filter_dump_commands(b, f->body, 0);
+	filter_dump_commands(b, f, f->body, 0);
 }
