@@ -74,7 +74,11 @@ enum out_type {
   VAR(title, T_STR, B_INDEXER) \
   VAR(image_size, T_INT, B_INDEXER) \
   VAR(image_aspect_ratio, T_INT, B_INDEXER) \
-  VAR(image_colors, T_INT, B_INDEXER)
+  VAR(image_colors, T_INT, B_INDEXER) \
+  VAR(audio_length, T_INT, B_INDEXER) \
+  VAR(audio_bitrare, T_INT, B_INDEXER) \
+  VAR(audio_srate, T_INT, B_INDEXER) \
+  VAR(audio_channels, T_INT, B_INDEXER)
 
 struct args {
 #define VAR(name, type, class) type name;
