@@ -2,7 +2,7 @@
  *	Sherlock Filter Engine
  *
  *	(c) 1999--2000 Martin Mares <mj@ucw.cz>
- *	(c) 2001--2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2001--2007 Robert Spalek <robert@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
@@ -30,14 +30,19 @@ filter_load(byte *name, struct filter_variable *var, struct filter_binding *bind
 
 	f = xmalloc(sizeof(struct filter));
 	f->pool = mp_new(4096);
-	f->name_head.next = NULL;
-	f->name_head.name[0] = 0;
+	f->name_head = mp_alloc(f->pool, sizeof(struct filter_lex_name) + 1);
+	f->name_head->next = NULL;
+	f->name_head->name[0] = 0;
 	f->var = var;
 	f->bind = bind;
 	f->func = func;
 	f->user_vars = 0;
 	f->decl = f->deleted_decl = NULL;
 	f->body = NULL;
+	f->lookup_count = 0;
+	f->lookup_limit = 32;
+	f->lookup = xmalloc(f->lookup_limit * sizeof(*f->lookup));
+	bzero(f->lookup, sizeof(*f->lookup));
 
 	filter_current = f;
 	filter_lex_init(name);
@@ -55,9 +60,76 @@ filter_load(byte *name, struct filter_variable *var, struct filter_binding *bind
 	return f;
 }
 
+struct filter *
+filter_clone(struct filter *filter)
+{
+	struct filter *f;
+
+	ASSERT(filter);
+	f = xmalloc(sizeof(*f));
+	memcpy(f, filter, sizeof(*f));
+	f->pool = mp_new(4096);
+	f->lookup = xmalloc((f->lookup_count + 1) * sizeof(*f->lookup));
+	memcpy(f->lookup, filter->lookup, (f->lookup_count + 1) * sizeof(*f->lookup));
+	for (uns i = 1; i <= f->lookup_count; i++)
+	{
+		struct filter_lookup *l = f->lookup + i;
+		switch (l->type)
+		{
+			case F_LT_REGEX:
+			{
+				l->regex = mp_memdup(f->pool, l->regex, sizeof(*l->regex));
+				if (!(l->regex->regex = rx_compile(l->regex->source, l->regex->icase)))
+					die("Cannot clone regular expression");
+				break;
+			}
+			case F_LT_TRIE:
+			{
+				l->trie = mp_memdup(f->pool, l->trie, sizeof(*l->trie));
+				l->trie->results = mp_memdup(f->pool, l->trie->results, l->trie->cmds * sizeof(struct filter_trie_result));
+				init_list(&l->trie->tests);
+				init_list(&l->trie->incremented_tests);
+				init_list(&l->trie->passed_tests);
+				for (uns i = 0; i < l->trie->cmds; i++)
+				{
+		  			if (l->trie->results[i].init_passed == 2)
+						add_tail(&l->trie->passed_tests, &l->trie->results[i].n);
+					else
+		  				add_tail(&l->trie->tests, &l->trie->results[i].n);
+				}
+				break;
+			}
+			default:
+				ASSERT(0);
+		}
+	}
+	return f;
+}
+
+int
+filter_lookup_new(struct filter *filter, enum filter_lookup_type type, void *value)
+{
+	if (filter->lookup_count + 1 == filter->lookup_limit)
+	{
+		filter->lookup_limit *= 2;
+		filter->lookup = xrealloc(filter->lookup, filter->lookup_limit * sizeof(*filter->lookup));
+	}
+	struct filter_lookup *l = filter->lookup + ++filter->lookup_count;
+	l->type = type;
+	l->value = value;
+	return filter->lookup_count; 
+}
+
+int
+filter_lookup_new_no_null(struct filter *filter, enum filter_lookup_type type, void *value)
+{
+	return value ? filter_lookup_new(filter, type, value) : 0;
+}
+
 void
 filter_delete(struct filter *f)
 {
+	// memory leak (regex)
 	if (!f)
 		return;
 	mp_delete(f->pool);
@@ -67,16 +139,10 @@ filter_delete(struct filter *f)
 struct filter_args *
 filter_intr_new(struct filter *f)
 {
-	struct filter_args *a = xmalloc(sizeof(struct filter_args));
+	struct filter_args *a = xmalloc_zero(sizeof(struct filter_args));
 	a->filter = f;
 	if (a->filter->user_vars)
 		a->user_var = xmalloc_zero(a->filter->user_vars*sizeof(union filter_raw_value));
-	else
-		a->user_var = NULL;
-	a->pool = NULL;
-	a->msg = NULL;
-	a->raw = NULL;
-	a->attr = NULL;
 	return a;
 }
 
@@ -217,13 +283,16 @@ filter_set_lvalue(struct filter_lvalue *dest, struct filter_args *args, struct f
 			}
 			break;
 		case F_LVC_CONF:
-			cf_journal_block(dest->v.cfg->ptr, sizeof(void*));
-			if (v->type == F_ET_INT)
-				* (int*) (dest->v.cfg->ptr) = v->v.i;
-			else if (v->type == F_ET_STRING)
-				* (byte**) (dest->v.cfg->ptr) = v->v.s;
-			else
-				ASSERT(0);
+			if (args->config_changes_mode)
+			{
+				cf_journal_block(dest->v.cfg->ptr, sizeof(void*));
+				if (v->type == F_ET_INT)
+					* (int*) (dest->v.cfg->ptr) = v->v.i;
+				else if (v->type == F_ET_STRING)
+					* (byte**) (dest->v.cfg->ptr) = v->v.s;
+				else
+					ASSERT(0);
+			}
 			break;
 		case F_LVC_USER:
 			args->user_var[dest->v.decl->nr] = v->v;
@@ -301,7 +370,7 @@ filter_eval_expr(struct filter_value *dest, struct filter_args *args, struct fil
 			else if (expr->type == F_ET_STRING)
 				dest->v.s = expr->o.s;
 			else if (expr->type == F_ET_REGEXP)
-				dest->v.r = expr->o.r;
+				dest->v.i = expr->o.i;
 			else
 				ASSERT(0);
 			break;
@@ -366,6 +435,12 @@ filter_eval_expr(struct filter_value *dest, struct filter_args *args, struct fil
 					dest->v.i = i;
 					break;
 				}
+				case '&':
+					dest->v.i = l.v.i & r.v.i;
+					break;
+				case '|':
+					dest->v.i = l.v.i | r.v.i;
+					break;
 				case '.':
 				{
 					int len = printable_length(&l) + printable_length(&r) + 1;
@@ -482,7 +557,7 @@ log_not(int x)
 }
 
 int
-filter_eval_cond(struct filter_args *args, struct filter_cond *cond)
+filter_eval_cond(struct filter_args *args, struct filter_cond *cond, struct filter_expr *partial_expr)
 {
 	if (cond->undef)
 		return UNDEFINED;
@@ -494,14 +569,14 @@ filter_eval_cond(struct filter_args *args, struct filter_cond *cond)
 		{
 			struct filter_value l, r;
 			int res;
-			filter_eval_expr(&l, args, cond->o.expr.l);
+			filter_eval_expr(&l, args, cond->o.expr.l ? : partial_expr);
 			filter_eval_expr(&r, args, cond->o.expr.r);
 			if (cond->o.expr.op == EREG)
 			{
 				if (l.undef)
 					return UNDEFINED;
 				/* ignoring case already handled when compiled */
-				res = rx_match(r.v.r->regex, l.v.s) ? TRUE : FALSE;
+				res = rx_match(args->filter->lookup[r.v.i].regex->regex, l.v.s) ? TRUE : FALSE;
 			}
 			else if (cond->o.expr.op == EIN)
 			{
@@ -542,7 +617,7 @@ filter_eval_cond(struct filter_args *args, struct filter_cond *cond)
 		case F_CC_DEFCOND:
 		{
 			int r;
-			r = filter_eval_cond(args, cond->o.neg);
+			r = filter_eval_cond(args, cond->o.neg, NULL);
 			return r == UNDEFINED ? FALSE : TRUE;
 		}
 		case F_CC_DEFEXPR:
@@ -554,14 +629,14 @@ filter_eval_cond(struct filter_args *args, struct filter_cond *cond)
 		case F_CC_UNOP:
 		{
 			int r;
-			r = filter_eval_cond(args, cond->o.neg);	/* ! */
+			r = filter_eval_cond(args, cond->o.neg, NULL);	/* ! */
 			return log_not(r);
 		}
 		case F_CC_BINOP:
 		{
 			int l, r;
-			l = filter_eval_cond(args, cond->o.bin.l);
-			r = filter_eval_cond(args, cond->o.bin.r);
+			l = filter_eval_cond(args, cond->o.bin.l, NULL);
+			r = filter_eval_cond(args, cond->o.bin.r, NULL);
 			switch (cond->o.bin.op)
 			{
 				case AND:
@@ -661,7 +736,7 @@ filter_eval_cmd(struct filter_args *args, struct filter_cmd *cmd)
 			case IF:
 			{
 				int cond, res;
-				cond = filter_eval_cond(args, cmd->c.cond.cond);
+				cond = filter_eval_cond(args, cmd->c.cond.cond, NULL);
 				switch (cond)
 				{
 					case TRUE:
@@ -697,9 +772,9 @@ filter_eval_cmd(struct filter_args *args, struct filter_cmd *cmd)
 					goto eval_cmd;
 				if (cmd->c.swit.icmp && filter_ht_find(cmd->c.swit.icmp, a.v.s, &run_cmd))
 					goto eval_cmd;
-				if (cmd->c.swit.pat && (run_cmd = filter_trie_search(cmd->c.swit.pat, a.v.s)) )
+				if (cmd->c.swit.pat && (run_cmd = filter_trie_search(args->filter->lookup[cmd->c.swit.pat].trie, a.v.s)) )
 					goto eval_cmd;
-				if (cmd->c.swit.ipat && (run_cmd = filter_trie_search(cmd->c.swit.ipat, a.v.s)) )
+				if (cmd->c.swit.ipat && (run_cmd = filter_trie_search(args->filter->lookup[cmd->c.swit.ipat].trie, a.v.s)) )
 					goto eval_cmd;
 				if (cmd->c.swit.expr->type == F_ET_STRING)
 				{
@@ -717,9 +792,7 @@ filter_eval_cmd(struct filter_args *args, struct filter_cmd *cmd)
 				}
 				for (cas=cmd->c.swit.cases; cas; cas=cas->next)
 				{
-					cas->cond->o.expr.l = cmd->c.swit.expr;
-					res = filter_eval_cond(args, cas->cond);
-					cas->cond->o.expr.l = NULL;
+					res = filter_eval_cond(args, cas->cond, cmd->c.swit.expr);
 					if (res == TRUE)
 					{
 						run_cmd = cas->positive;
@@ -773,6 +846,8 @@ filter_intr_undo_init(struct filter_args *a)
 {
 	/* All the changes will be commited, because the undo pool is forgotten
 	 * and reset to the initial state.  */
+	a->saved_pool = cf_pool;
+	cf_pool = a->pool; /* faster than allocation of a new pool */
 	a->oldj = cf_journal_new_transaction(0);
 }
 
@@ -781,6 +856,7 @@ filter_intr_undo(struct filter_args *a)
 {
 	cf_journal_rollback_transaction(0, a->oldj);
 	a->oldj = NULL;
+	cf_pool = a->saved_pool;
 }
 
 int
@@ -790,11 +866,11 @@ filter_intr_run(struct filter_args *a)
 
 	a->msg = NULL;
 	filter_user_var_init(a);
-	if (!a->want_config_changes)
-		filter_intr_undo_init(a);
+	if (a->config_changes_mode == 2)
+	                   filter_intr_undo_init(a);
 	res = filter_eval_cmd(a, a->filter->body);
-	if (!a->want_config_changes)
-		filter_intr_undo(a);
+	if (a->config_changes_mode == 2)
+	                  filter_intr_undo(a);
 	if (res != REJECT && res != ACCEPT)
 	{
 		log(L_ERROR, "filter: Neither ACCEPT nor REJECT issued, defaulting to REJECT.");
