@@ -2,7 +2,7 @@
  *	Sherlock Filter Engine
  *
  *	(c) 1999--2000 Martin Mares <mj@ucw.cz>
- *	(c) 2001--2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2001--2007 Robert Spalek <robert@ucw.cz>
  */
 
 struct fastbuf;
@@ -15,6 +15,7 @@ enum filter_lvalue_cat { F_LVC_RAW, F_LVC_ATTR, F_LVC_CONF, F_LVC_USER};
 enum filter_expr_type { F_ET_INT, F_ET_STRING, F_ET_REGEXP, F_ET_UNKNOWN};
 enum filter_expr_cat { F_EC_CONST, F_EC_LVALUE, F_EC_UNOP, F_EC_BINOP, F_EC_FUNC};
 enum filter_cond_cat { F_CC_CONST, F_CC_EXPR, F_CC_DEFCOND, F_CC_DEFEXPR, F_CC_UNOP, F_CC_BINOP};
+enum filter_lookup_type { F_LT_REGEX, F_LT_TRIE };
 
 /*
  * Values of the variables may be undefined by the context (not bound) or by
@@ -81,7 +82,6 @@ struct filter_expr {
 	union {
 		int i;
 		byte *s;
-		struct filter_regex_value *r;
 		union filter_raw_value2 raw;
 		struct filter_lvalue *lv;
 		struct {
@@ -155,7 +155,7 @@ struct filter_cmd {
 			struct filter_case *cases;
 			struct filter_cmd *negative, *undefined;
 			struct filter_hash_table *cmp, *icmp;
-			struct filter_trie_table *pat, *ipat;
+			int pat, ipat;
 			union {
 				struct {
 					struct filter_s_tree *bins;
@@ -228,15 +228,26 @@ struct filter_declaration {
 	struct filter_variable var;
 };
 
+struct filter_lookup {
+	enum filter_lookup_type type;
+	union {
+		void *value;
+		struct filter_regex_value *regex;
+		struct filter_trie_table *trie;
+	};
+};
+
 struct filter {
 	struct mempool *pool;
-	struct filter_lex_name name_head;
+	struct filter_lex_name *name_head;
 	struct filter_variable *var;
 	struct filter_binding *bind;
 	struct filter_function *func;
 	int user_vars;			/* the number of user-defined variables */
 	struct filter_declaration *decl, *deleted_decl;
 	struct filter_cmd *body;
+	uns lookup_count, lookup_limit;
+	struct filter_lookup *lookup;
 };
 
 /*
@@ -244,7 +255,7 @@ struct filter {
  * raw variable structure and pointer to the argument list must be set here.
  *
  * If you want to let the filter modify your program configuration, you have
- * to set want_config_changes and call filter_intr_undo() appropriately to
+ * to set config_changes_mode and call filter_intr_undo() appropriately to
  * restrict the scope of the config changes.
  */
 
@@ -255,8 +266,9 @@ struct filter_args {
 	byte *msg;				/* return message or NULL */
 	void *raw;
 	struct odes *attr;
-	int want_config_changes;
+	int config_changes_mode;		/* 0 - disabled, 1 - leave changes, 2 - reset changes automatically */
 	struct cf_journal_item *oldj;		/* journalling of cf_item changes */
+	struct mempool *saved_pool;		/* saved cf_pool for want_config_changes disabled */
 };
 
 /* parse.y */
@@ -281,7 +293,10 @@ int yylex(void);
 extern struct filter *filter_current;
 
 struct filter *filter_load(byte *, struct filter_variable *, struct filter_binding *, struct filter_function *);
+struct filter *filter_clone(struct filter *);
 void filter_delete(struct filter *);
+int filter_lookup_new(struct filter *filter, enum filter_lookup_type type, void *value);
+int filter_lookup_new_no_null(struct filter *filter, enum filter_lookup_type type, void *value);
 
 struct filter_args *filter_intr_new(struct filter *);
 void filter_intr_delete(struct filter_args *);
@@ -291,7 +306,7 @@ void filter_intr_undo(struct filter_args *);
 
 /* Exported for pruning: */
 void filter_eval_expr(struct filter_value *dest, struct filter_args *args, struct filter_expr *expr);
-int filter_eval_cond(struct filter_args *args, struct filter_cond *cond);
+int filter_eval_cond(struct filter_args *args, struct filter_cond *cond, struct filter_expr *partial_expr);
 
 /* prune.c */
 
@@ -355,7 +370,7 @@ struct filter_trie_table
 
 struct filter_trie_table *filter_trie_new(struct mempool *mp, struct filter_cmd *case_cmd, uns icase);
 struct filter_cmd *filter_trie_search(struct filter_trie_table *trie, byte *string);
-void filter_trie_dump(struct fastbuf *b, struct filter_trie_table *trie);
+void filter_trie_dump(struct fastbuf *b, struct filter *f, struct filter_trie_table *trie);
 
 /* trees.c */
 
@@ -367,15 +382,15 @@ struct filter_cmd *filter_s_tree_search(struct filter_s_tree *tree, byte *val);
 struct filter_cmd *filter_is_tree_search(struct filter_is_tree *tree, byte *val);
 struct filter_cmd *filter_ud_tree_search(struct filter_ud_tree *tree, uns val);
 struct filter_cmd *filter_d_tree_search(struct filter_d_tree *tree, int val);
-void filter_s_tree_dump(struct fastbuf *fb, struct filter_s_tree *tree, uns level);
-void filter_is_tree_dump(struct fastbuf *fb, struct filter_is_tree *tree, uns level);
-void filter_ud_tree_dump(struct fastbuf *fb, struct filter_ud_tree *tree, uns level);
-void filter_d_tree_dump(struct fastbuf *fb, struct filter_d_tree *tree, uns level);
+void filter_s_tree_dump(struct fastbuf *fb, struct filter *f, struct filter_s_tree *tree, uns level);
+void filter_is_tree_dump(struct fastbuf *fb, struct filter *f, struct filter_is_tree *tree, uns level);
+void filter_ud_tree_dump(struct fastbuf *fb, struct filter *f, struct filter_ud_tree *tree, uns level);
+void filter_d_tree_dump(struct fastbuf *fb, struct filter *f, struct filter_d_tree *tree, uns level);
 
 /* dumper.c */
 
 void filter_dump_spaces(struct fastbuf *b, uns level);
-void filter_dump_commands(struct fastbuf *b, struct filter_cmd *cmd, uns level);
+void filter_dump_commands(struct fastbuf *b, struct filter *f, struct filter_cmd *cmd, uns level);
 void filter_dump(struct fastbuf *b, struct filter *f);
 
 /* fconfig.c */
