@@ -91,6 +91,17 @@ prune_expr(struct filter_expr *r)
 					if (HAS_E_VALUE(r->o.bin.l, 1))
 						val.v.i = 1, replace = 1;
 					break;
+				case '&':
+					if (HAS_E_VALUE(r->o.bin.l, 0)
+					|| HAS_E_VALUE(r->o.bin.r, 0))
+						val.v.i = 0, replace = 1;
+					break;
+				case '|':
+					if (HAS_E_VALUE(r->o.bin.l, 0))
+						return r->o.bin.r;
+					if (HAS_E_VALUE(r->o.bin.r, 0))
+						return r->o.bin.l;
+					break;
 			}
 			break;
 		case F_EC_FUNC:
@@ -132,7 +143,7 @@ prune_cond(struct filter_cond *c)
 			{
 				if (c->o.expr.l->cat == F_EC_CONST
 				&& c->o.expr.r->cat == F_EC_CONST)
-					res = filter_eval_cond(&prune_args, c);
+					res = filter_eval_cond(&prune_args, c, NULL);
 				else if (c->o.expr.l->undef || c->o.expr.r->undef)
 					res = UNDEFINED;
 			}
@@ -141,7 +152,7 @@ prune_cond(struct filter_cond *c)
 			c->o.defexpr = prune_expr(c->o.defexpr);
 			if (c->o.defexpr->undef
 			|| c->o.defexpr->cat == F_EC_CONST)
-				res = filter_eval_cond(&prune_args, c);
+				res = filter_eval_cond(&prune_args, c, NULL);
 			/* otherwise the condition can/cannot be defined
 			 * depending on the values of the operands */
 			break;
@@ -150,13 +161,13 @@ prune_cond(struct filter_cond *c)
 			c->o.neg = prune_cond(c->o.neg);
 			if (c->o.neg->undef
 			|| c->o.neg->cat == F_CC_CONST)
-				res = filter_eval_cond(&prune_args, c);
+				res = filter_eval_cond(&prune_args, c, NULL);
 			break;
 		case F_CC_BINOP:
 			c->o.bin.l = prune_cond(c->o.bin.l);
 			c->o.bin.r = prune_cond(c->o.bin.r);
 			if (c->o.bin.l->cat == F_CC_CONST && c->o.bin.r->cat == F_CC_CONST)
-				res = filter_eval_cond(&prune_args, c);
+				res = filter_eval_cond(&prune_args, c, NULL);
 #define	HAS_C_VALUE(cond,val)	(cond->cat == F_CC_CONST && cond->o.i == val)
 			else switch (c->o.bin.op)
 			{
@@ -324,9 +335,7 @@ start:
 							&& cas->cond->o.expr.r->o.bin.l->cat == F_EC_CONST
 							&& cas->cond->o.expr.r->o.bin.r->cat == F_EC_CONST))
 						{
-							cas->cond->o.expr.l = cmd->c.swit.expr;
-							int res = filter_eval_cond(&prune_args, cas->cond);
-							cas->cond->o.expr.l = NULL;
+							int res = filter_eval_cond(&prune_args, cas->cond, cmd->c.swit.expr);
 							if (res == TRUE)
 							{
 								replace_by = cas->positive;
@@ -367,38 +376,31 @@ start:
 }
 
 /* Returns the NEW last case in this chain.  */
-static struct filter_case *
-prune_switch_case(struct filter_case **Case)
+static void
+prune_switch_case(struct filter_case **Case, uns remove_empty)
 {
-	struct filter_case *c, *new_last;
-start:
-	c = *Case;
-	if (!c)
-		return NULL;
-	c->cond = prune_cond(c->cond);
-	ASSERT(c->undef == c->cond->undef);
-	prune_command(&c->positive);
-	if (c->undef || !c->positive)
+	struct filter_case *c, **p = Case, *l = NULL;
+	for (c = *Case; c; c = c->next)
 	{
-		/* This deletes the CASE if the command is empty, hence other
-		 * matches can take part in.  If the cases are overlapping, the
-		 * variable can match another case.  Tell me if you think it
-		 * matters.  */
-		*Case = c->next;
-		goto start;
-	}
-	new_last = prune_switch_case(&c->next);
-	if (new_last)
-	{
-		ASSERT(c->next);
-		c->last = new_last;
-		return new_last;
-	}
-	else
-	{
-		c->last = c;
-		return c;
+		c->cond = prune_cond(c->cond);
+		ASSERT(c->undef == c->cond->undef);
+		prune_command(&c->positive);
+		if (!remove_empty || !(c->undef || !c->positive))
+		{
+			*p = l = c;
+			p = &c->next;
+		}
+		else
+		{
+			/* This deletes the CASE if the command is empty, hence other
+			 * matches can take part in.  If the cases are overlapping, the
+			 * variable can match another case.  Tell me if you think it
+			 * matters.  */
+		}
 	}
+	*p = NULL;
+	for (c = *Case; c; c = c->next)
+		c->last = l;
 }
 
 /* Returns whether the whole SWITCH command should be deleted.  */
@@ -413,7 +415,7 @@ prune_switch_command(struct filter_cmd *cmd)
 	ASSERT(!cmd->c.swit.binis);
 	prune_command(&cmd->c.swit.negative);
 	prune_command(&cmd->c.swit.undefined);
-	prune_switch_case(&cmd->c.swit.cases);
+	prune_switch_case(&cmd->c.swit.cases, !cmd->c.swit.negative);
 	if (!cmd->c.swit.cases
 	&& !cmd->c.swit.negative
 	&& !cmd->c.swit.undefined)
@@ -475,8 +477,8 @@ possibly_create_lookup_tables(struct filter *f, struct filter_cmd *cmd)
 		cmd->c.swit.cmp = build_hash_table(f, cmd, 0, cmps[1]);
 	if (cmps[2] >= filter_hash_limit)
 		cmd->c.swit.icmp = build_hash_table(f, cmd, 1, cmps[2]);
-	cmd->c.swit.pat = filter_trie_new(f->pool, cmd, 0);
-	cmd->c.swit.ipat = filter_trie_new(f->pool, cmd, 1);
+	cmd->c.swit.pat = filter_lookup_new_no_null(f, F_LT_TRIE, filter_trie_new(f->pool, cmd, 0));
+	cmd->c.swit.ipat = filter_lookup_new_no_null(f, F_LT_TRIE, filter_trie_new(f->pool, cmd, 1));
 	if (cmd->c.swit.expr->type == F_ET_STRING)
 	{
 		cmd->c.swit.bins = filter_s_tree_new(f->pool, cmd);
