@@ -186,7 +186,7 @@ NAME_(dump1)(struct fastbuf *fb, int dir, PTR_DECL v1, PTR_DECL v2)
 }
 
 void
-NAME_(tree_dump)(struct fastbuf *fb, struct NAME_(tree) *tree, uns level)
+NAME_(tree_dump)(struct fastbuf *fb, struct filter *f, struct NAME_(tree) *tree, uns level)
 {
 	PTR_DECL v1 = NAME_(full_int)[0];
 	PTR_DECL v2;
@@ -210,14 +210,14 @@ NAME_(tree_dump)(struct fastbuf *fb, struct NAME_(tree) *tree, uns level)
 			v2 = n->STRUCT_ATTR;
 		filter_dump_spaces(fb, level);
 		NAME_(dump1)(fb, n->dir, v1, v2);
-		filter_dump_commands(fb, n->cmd, level+1);
+		filter_dump_commands(fb, f, n->cmd, level+1);
 	}
 	TREE_END_FOR;
 	if (last_dir == -2)
 	{
 		filter_dump_spaces(fb, level);
 		NAME_(dump1)(fb, last_dir, v1, v1);
-		filter_dump_commands(fb, last_cmd, level+1);
+		filter_dump_commands(fb, f, last_cmd, level+1);
 	}
 	if (filter_trace > 0)
 	{
