@@ -414,7 +414,7 @@ dump_trie(struct fastbuf *b, struct filter_trie_record *r, uns level, int value)
 }
 
 void
-filter_trie_dump(struct fastbuf *b, struct filter_trie_table *trie)
+filter_trie_dump(struct fastbuf *b, struct filter *f UNUSED, struct filter_trie_table *trie)
 {
 	bprintf(b, "Trie: icase=%d commands=%d\n", trie->icase, trie->cmds);
 	for (uns i=0; i<trie->cmds; i++)
