@@ -178,6 +178,15 @@ void custom_create_attrs(struct odes *odes, struct card_attr *ca);
  *				is called whenever a link from a normal card `referer' to an image card
  *				`img', possibly after traversing a chain of redirects, is encountered.
  *				Again, this can be useful for propagating values of custom attributes.
+ *	CUSTOM_MKLEX(struct odes *odes, void (*map)(byte *start, *end))
+ *				is called in the mklex and allows the customization to
+ *				make its own additional lexicon mappings.
+ *	CUSTOM_CHEW_PREPROC(struct odes *odes, struct card_attr *ca)
+ *				is called in the chewer before all other manipulations with an object.
+ *	CUSTOM_CHEW_ATTRS(struct odes *odes, struct card_attr *ca)
+ *				is called in the chewer before `ca' is written.
+ *				This is useful if you need to make updates to this structure
+ *				in the second phase of indexation.
  */
 
 /*
@@ -246,4 +255,7 @@ void custom_create_attrs(struct odes *odes, struct card_attr *ca);
  *  Custom initialization (optional):
  *
  *	CUSTOM_INIT()		run after sherlockd initializes itself and before it calls any other CUSTOM_xxx macros.
+ *	CUSTOM_FETCH(struct odes *o)
+ *				called in the indexer for each fetched object and allows
+ *				the user to modify some of its attributes.
  */
