@@ -8,7 +8,6 @@
 #include "sherlock/sherlock.h"
 #include "lib/clists.h"
 #include "lib/fastbuf.h"
-#include "lib/conf.h"
 #include "lib/chartype.h"
 #include "lib/unicode.h"
 #include "lib/ff-utf8.h"
@@ -16,8 +15,9 @@
 #include "lang/lang.h"
 #include "gather/gather.h"
 #include "charset/charconv.h"
-#include "sherlock/conf-parse.h"
+#include "sherlock/conf.h"
 
+#include <stdio.h>
 #include <string.h>
 
 /* Configuration */
@@ -580,7 +580,7 @@ convert_stream_charset(uns id)
       src = fbmem_clone_read(gthis->contents);
       dest = fbmem_create(16384);
       int c;
-      while ((c = bget_utf8(src)) != EOF)
+      while ((c = bget_utf8(src)) >= 0)
 	bput_utf8(dest, c);
       bclose(src);
       bclose(gthis->contents);
