@@ -572,7 +572,6 @@ vaticinate(byte **type)		/* Check the existing Content-Type or find the first ma
 	{
 		int results[CBMODE_END][CBRES_END];	/* number of results[CBMODE_*][CBRESULT_*] */
 		struct content_type_record *ctr;
-		int i;
 
 		memset(results, 0, sizeof(results));
 		for (ctr=ct_find(*type); ctr; ctr=ct_find_next(ctr))
@@ -587,13 +586,13 @@ vaticinate(byte **type)		/* Check the existing Content-Type or find the first ma
 			results[CBMODE_EXTENSION][CBRES_TRUE]++;
 			results[CBMODE_EXTENSION][CBRES_FALSE]--;
 		}
-		for (i=CBMODE_WILDCARD; i<=CBMODE_EXTENSION; i++)
-			if (i != CBMODE_IS_ASCII)
-				for (int j=0; j<CBRES_END; j++)
-				{
-					results[CBMODE_SUFFIX][j] += results[i][j];
-					results[i][j] = 0;
-				}
+		for (int j=0; j<CBRES_END; j++)
+		{
+			results[CBMODE_SUFFIX][j] += results[CBMODE_WILDCARD][j];
+			results[CBMODE_SUFFIX][j] += results[CBMODE_EXTENSION][j];
+			results[CBMODE_WILDCARD][j] = 0;
+			results[CBMODE_EXTENSION][j] = 0;
+		}
 		type_m = "[confirmed] ";
 		if (results[CBMODE_SUFFIX][CBRES_FALSE]
 		&& !results[CBMODE_SUFFIX][CBRES_TRUE])
