@@ -13,11 +13,13 @@
 #include "lib/url.h"
 #include "lib/db.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/hashfunc.h"
 #include "lib/mempool.h"
 #include "sherlock/object.h"
 #include "gather/daemon/gatherd.h"
 
+#include <stdio.h>
 #include <time.h>
 #include <setjmp.h>
 #include <unistd.h>
@@ -534,7 +536,7 @@ shake_md5(oid_t new, uns type, byte *buck, uns len)
   struct fastbuf fb;
   fbbuf_init_read(&fb, buck, len, 0);
   mp_flush(pool);
-  struct odes *o = obj_read_bucket(read_buf, pool, type, len, &fb, NULL);
+  struct odes *o = obj_read_bucket(read_buf, pool, type, len, &fb, NULL, 1);
   if (unlikely(!o))
     {
       log(L_ERROR, "Cannot parse object %08x, deleting", new);
@@ -601,7 +603,7 @@ shake(void)
   TRACE("Shaking down buckets");
   read_buf = buck2obj_alloc();
   pool = mp_new(1<<16);
-  obuck_shakedown(shake_kibitz);
+  obuck_shakedown(&bucket_file, shake_kibitz);
   mp_delete(pool);
   buck2obj_free(read_buf);
 
@@ -634,7 +636,7 @@ quick_delete(void)
       if (id->new == OID_UNDEFINED)
 	{
 	  DBG("Deleting %08x", id->oid);
-	  obuck_delete(id->oid);
+	  obuck_delete(&bucket_file, id->oid);
 	  count++;
 	}
       else
@@ -654,7 +656,7 @@ quick_delete(void)
 
 struct qs_key {
   u32 weight;
-  addr_int_t host;
+  uintptr_t host;
   uns len;
 };
 
@@ -905,7 +907,7 @@ chew_urls(void)
 	  else
 	    w = 0;
 	  bputl(qlist, w);
-	  bputa(qlist, (addr_int_t) h);
+	  bputa(qlist, (uintptr_t) h);
 	  bputw(qlist, strlen(rfdata.url_s.rest));
 	  bputs(qlist, rfdata.url_s.rest);
 	  bputl(qlist, (ur.oid == OID_UNDEFINED) ? rfdata.section : ~0U);
@@ -1021,7 +1023,7 @@ main(int argc, char **argv)
   gather_lock(0);
   queue_init(1);
   refs_init(1);
-  obuck_init(1);
+  bucket_open(1);
   stamp_read();
 
   ids_init();
@@ -1053,7 +1055,7 @@ main(int argc, char **argv)
     }
 
   stamp_write();
-  obuck_cleanup();
+  bucket_close();
   queue_cleanup();
   gather_unlock();
   return 0;
