@@ -73,17 +73,17 @@ static int
 dynamic_filter(struct mempool *pool, byte *urlrest, oid_t oid)
 {
   byte rest[MAX_URL_SIZE];
-  struct obuck_header buck;
+  struct obuck_context ctx;
   int accept = 1;
 
   if (oid == OID_UNDEFINED || oid >= OID_FIRST_ERROR)
     return 1;
   if (url_enescape(urlrest, rest))
     die("dynamic_filter: error escaping %s", urlrest);
-  buck.oid = oid;
-  obuck_find_by_oid(&buck);
-  struct fastbuf *b = obuck_fetch();
-  struct odes *obj = obj_read_bucket(read_buf, pool, buck.type, buck.length, b, NULL);
+  ctx.hdr.oid = oid;
+  obuck_find_by_oid(&bucket_file, &ctx, 0);
+  struct fastbuf *b = obuck_fetch(&bucket_file, &ctx);
+  struct odes *obj = obj_read_bucket(read_buf, pool, ctx.hdr.type, ctx.hdr.length, b, NULL, 1);
   for (struct oattr *o = obj_find_attr(obj, 'r'); o; o=o->same)
     if (!strncmp(rest, o->val, str_len(o->val)))
       {
@@ -157,10 +157,10 @@ gather_send_result(struct thread *t)
   if (gthis->error_code <= 1)		/* Downloaded successfully or redirected */
     {
       u32 out_format = (compress_level > 0) ? BUCKET_TYPE_V33_LIZARD : (compress_level < 0) ? BUCKET_TYPE_V30 : BUCKET_TYPE_V33;
-      struct fastbuf *b = obuck_create();
+      struct fastbuf *b = obuck_create(&bucket_file);
       out_format = gobj_write(b, out_format, GWF_DUMP_BODY | (dump_full_objects ? GWF_DUMP_SOURCE : 0));
       struct obuck_header buck;
-      obuck_create_end(b, out_format, &buck);
+      obuck_create_end(&bucket_file, b, out_format, &buck);
       obj_set_attr_num(gthis->aa, 'O', buck.oid);
     }
 
@@ -172,7 +172,7 @@ gather_send_result(struct thread *t)
        *  When changing this condition, check there as well!
        */
       TTRACE("Deleting old object %08x in subthread", t->refreshing);
-      obuck_delete(t->refreshing);
+      obuck_delete(&bucket_file, t->refreshing);
     }
 
   /* Send description to gatherd master */
@@ -204,11 +204,11 @@ gather_document(struct thread *t)
   if (t->refreshing != OID_UNDEFINED)
     {
       /* Read the old object we're trying to refresh */
-      struct obuck_header oh;
-      oh.oid = t->refreshing;
-      obuck_find_by_oid(&oh);
-      struct fastbuf *b = obuck_fetch();
-      gthis->refreshing = obj_read_bucket(read_buf, gthis->pool, oh.type, oh.length, b, NULL);
+      struct obuck_context ctx;
+      ctx.hdr.oid = t->refreshing;
+      obuck_find_by_oid(&bucket_file, &ctx, 0);
+      struct fastbuf *b = obuck_fetch(&bucket_file, &ctx);
+      gthis->refreshing = obj_read_bucket(read_buf, gthis->pool, ctx.hdr.type, ctx.hdr.length, b, NULL, 1);
       bclose(b);
       if (t->ur.http_last_mod)
 	obj_set_attr_num(gthis->refreshing, 'L', t->ur.http_last_mod);
@@ -403,7 +403,7 @@ gather_finish(struct thread *t, int err)
 	      thread_err > 1 && thread_err < 2000)
 	    {
 	      TTRACE("Deleting old object %08x in main thread", t->refreshing);
-	      obuck_delete(t->refreshing);
+	      obuck_delete(&bucket_file, t->refreshing);
 	    }
 	  log_new = "+";
 	  if (md5 = obj_find_aval(o, 'C'))
@@ -439,7 +439,7 @@ gather_finish(struct thread *t, int err)
       sync_counter = 0;
       queue_sync();
       db_sync();
-      obuck_sync();
+      obuck_sync(&bucket_file);
     }
   return oid;
 }
@@ -631,7 +631,7 @@ handle_event(struct thread *t)
   struct fastbuf *fb = fbmem_clone_read(t->reply);
   bclose(t->reply);
   t->reply = fb;
-  t->obj = obj_read_bucket(read_buf, t->pool, BUCKET_TYPE_V33, bfilesize(t->reply), t->reply, NULL);
+  t->obj = obj_read_bucket(read_buf, t->pool, BUCKET_TYPE_V33, bfilesize(t->reply), t->reply, NULL, 1);
   if (unlikely(!t->obj))
     die("Cannot parse the reply of process %d", t->pid);
 
@@ -651,14 +651,14 @@ hard_shutdown(void)
    */
   if (!allow_hard_shutdown)
     return;
-  obuck_lock_write();
+  obuck_lock_write(&bucket_file);
   WALK_LIST(t, busy_threads)
     {
       if (!k++)
 	log(L_INFO, "Killing subprocesses");
       kill(t->pid, SIGTERM);
     }
-  obuck_unlock();
+  obuck_unlock(&bucket_file);
 }
 
 static void
@@ -765,7 +765,7 @@ main(int argc, char **argv)
   urldb_open();
   md5db_open();
   refs_init(0);
-  obuck_init(1);
+  bucket_open(1);
 
   read_buf = buck2obj_alloc();
   init_threads();
@@ -773,7 +773,7 @@ main(int argc, char **argv)
   loop();
   buck2obj_free(read_buf);
 
-  obuck_cleanup();
+  bucket_close();
   md5db_close();
   urldb_close();
   queue_cleanup();
