@@ -123,7 +123,7 @@ struct qhost {				/* Representation of queued host ([*] marks persistent items)
   u32 qkey;				/* [*] Queueing key (e.g., an IP address) */
   uns qpriority;			/* [*] Priority of the first item in the queue */
   u32 sequence;				/* Sequence number for secondary round-robin ordering */
-  word port;
+  u16 port;
   byte protocol;
   byte flags;
   byte state;
