@@ -399,7 +399,7 @@ del_obj_single(byte *u)
   else
     {
       if (ur.oid != OID_UNDEFINED && ur.oid < OID_FIRST_ERROR)
-	obuck_delete(ur.oid);
+	obuck_delete(&bucket_file, ur.oid);
       if (!urldb_delete(u))
 	log(L_ERROR, "%s: Delete failed, don't ask me why", u);
     }
@@ -410,9 +410,9 @@ delete_objects(byte *u)
 {
   gather_lock(0);
   urldb_open();
-  obuck_init(1);
+  bucket_open(1);
   for_each_url(u, del_obj_single, "Deleting");
-  obuck_cleanup();
+  bucket_close();
   urldb_close();
   gather_unlock();
 }
@@ -467,8 +467,8 @@ static void
 insert_urls(byte *arg)
 {
   gather_lock(0);
-  obuck_init(1);
-  obuck_cleanup();
+  bucket_open(1);
+  bucket_close();
   queue_init(1);
   urldb_open();
   refs_init(1);
@@ -483,7 +483,7 @@ insert_urls(byte *arg)
 struct hs {
   struct hs *next;
   uns okays, errs, queued, requeued;
-  word port;
+  u16 port;
   byte protocol;
   byte hostname[1];
 };
@@ -622,7 +622,7 @@ requeue_url(byte *url)
 	u.oid = OID_UNDEFINED;
       if (requeue_delete_orig && u.oid != OID_UNDEFINED)
 	{
-	  obuck_delete(u.oid);
+	  obuck_delete(&bucket_file, u.oid);
 	  u.oid = OID_UNDEFINED;
 	}
       u.retries = 0;
@@ -643,9 +643,9 @@ requeue(uns delete_orig, byte *arg)
   queue_init(1);
   urldb_open();
   refs_init(1);
-  obuck_init(1);
+  bucket_open(1);
   for_each_url(arg, requeue_url, "Requeueing");
-  obuck_cleanup();
+  bucket_close();
   urldb_close();
   queue_cleanup();
   gather_unlock();
@@ -701,10 +701,10 @@ reb_fetch_key(struct fastbuf *f, struct reb_key *k)
       byte *url, *x;
 
     restart:
-      if (!(fb = obuck_slurp_pool(&bhdr, OBUCK_OID_ANY)))
+      if (!(fb = obuck_slurp_pool(&bucket_file, &bhdr, OBUCK_OID_ANY)))
 	return 0;
       mp_flush(reb_pool);
-      obj = obj_read_bucket(read_buf, reb_pool, bhdr.type, bhdr.length, fb, NULL);
+      obj = obj_read_bucket(read_buf, reb_pool, bhdr.type, bhdr.length, fb, NULL, 1);
       if (unlikely(!obj))
 	{
 	  log(L_ERROR, "Cannot parse bucket %08x", bhdr.oid);
@@ -789,13 +789,13 @@ synchronize_urldb(void)
   gather_lock(0);
   gather_note_state("reconstructing");
 
-  obuck_init(0);
+  bucket_open(0);
   reb_pool = mp_new(65536);
   read_buf = buck2obj_alloc();
   reb_counter = 0;
   log(L_INFO, "Scanning buckets");
   f = reb_sort(NULL);
-  obuck_cleanup();
+  bucket_close();
   log(L_INFO, "Found %d URL's", reb_counter);
 
   log(L_INFO, "Updating URLdb");
