@@ -10,6 +10,7 @@
 #include "gather/daemon/gatherd.h"
 #include "lib/url.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/mempool.h"
 #include "lib/pagecache.h"
 #include "lib/lfs.h"
@@ -17,6 +18,7 @@
 #include "lib/hashfunc.h"
 #include "lib/heap.h"
 
+#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -752,7 +754,7 @@ load_host_list(void)
 
   DBG("Loading host list");
   host_file = f = bopen(host_file_name, O_RDWR | O_CREAT, 4096);
-  if (bgetc(f) == EOF)
+  if (bgetc(f) < 0)
     {
       log(L_INFO, "Host database doesn't exist, creating new one");
       free_list = new_host(0, "", 0);
@@ -768,7 +770,7 @@ load_host_list(void)
 	  byte name[MAX_URL_SIZE];
 
 	  proto = bgetc(f);
-	  if (proto == EOF)
+	  if (proto < 0)
 	    break;
 	  port = bgetw(f);
 	  if (!bgets(f, name, MAX_URL_SIZE))
