@@ -7,10 +7,9 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
 #include "lib/chartype.h"
 #include "sherlock/index.h"
-#include "sherlock/conf-parse.h"
+#include "sherlock/conf.h"
 #include "lib/unicode.h"
 #include "lib/ff-utf8.h"
 #include "lib/clists.h"
@@ -19,6 +18,7 @@
 #include "gather/gather.h"
 #include "gather/format/html/html.h"
 
+#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -214,6 +214,15 @@ html_add_ref(int type, byte *url, uns nofollow)
   return html_add_ref_full(type, url, NULL, NULL, nofollow);
 }
 
+static int
+contains_frames(void)
+{
+  CLIST_FOR_EACH(struct gobj_ref *, r, gthis->ref_list)
+    if (r->type == 'F')
+      return 1;
+  return 0;
+}
+
 /* Word gathering */
 
 static int force_class;
@@ -411,12 +420,14 @@ add_obj_ref(byte *base_url, byte *ref, byte *ctype)
   html_add_ref_full('A', ref, ctype, base, 0);
 }
 
+#ifndef CONFIG_IMAGES
 struct gobj_ref *
 image_add_ref(byte *src, byte *width UNUSED, byte *height UNUSED, uns nofollow UNUSED)
 {
   struct gobj_ref *r = html_add_ref_full('I', src, NULL, NULL, 1);
   return r;
 }
+#endif
 
 /* Processing of META tags */
 
@@ -552,7 +563,6 @@ chew_meta(byte *name, byte *text)
     if (!strcasecmp(name, um->meta))
       {
 	MTRACE("User-defined META -> attribute %c", um->attr);
-	ASSERT(strlen(text) < MAX_ATTR_SIZE);
 	obj_add_attr(gthis->aa, um->attr, text);
 	return;
       }
@@ -936,6 +946,12 @@ look_for_robot_comment(void)
   ctrl[cnt] = 0;
   DBG("<!-- %s ... -->", ctrl);
 
+#ifdef CUSTOM_HTML_COMMENT
+  { CUSTOM_HTML_COMMENT(ctrl); }
+  if (!robot_comments)
+    return;
+#endif
+
   p = ctrl;
   if (*p == '/')
     {
@@ -970,7 +986,9 @@ parse_sgml_tag(void)
 	  c = get_char();
 	  if (c != '-')
 	    continue;
+#ifndef CUSTOM_HTML_COMMENT	  
 	  if (robot_comments)
+#endif
 	    look_for_robot_comment();
 	  switch (comment_mode)
 	    {
@@ -1251,6 +1269,9 @@ html_parse(byte **args UNUSED)
   /* Guess charset and convert to UTF-8 */
   convert_charset(found_meta_charset);
 
+  /* Run raw HTML analyser */
+  gather_raw_analyse();
+
   /* Parse the document */
   head_mode = 0;
   attrs_seen = 0;
@@ -1261,7 +1282,7 @@ html_parse(byte **args UNUSED)
   chew_html();
   bclose(html_in);
 
-  if (fake_redirect && btell(gthis->text) < refresh_threshold)
+  if (fake_redirect && btell(gthis->text) < refresh_threshold && !contains_frames())
     {
       /* Convert the whole page to a redirect if asked for. However, we keep
        * at least the meta-data for the indexer to use when the destination
@@ -1281,5 +1302,10 @@ html_parse(byte **args UNUSED)
   if (local_robot_accum)
     add_robot_note("Robot control comments tweaked", local_robot_accum, local_robot_accum);
 
+#ifdef CONFIG_IMAGES
+  /* Image references need finishing */
+  image_filter_refs();
+#endif
+
   return 1;
 }
