@@ -10,6 +10,9 @@ P(deflate)
 P(compress)
 P(robots)
 P(external)
+#ifdef CONFIG_IMAGES
+  P(image)
+#endif
 #ifdef CONFIG_PDF
   P(pdf)
 #endif
