@@ -11,6 +11,8 @@
 #include "gather/format/pdf/pdf.h"
 #include "gather/format/pdf/glyphs.h"
 
+#include <stdio.h>
+
 static struct font *fonttop;
 
 static s32
@@ -28,7 +30,7 @@ so2i(OBJECT obj, byte lmin, byte lmax, byte *olen) {
 }
 
 static struct unichar tempbuf[TEMPBUFLEN];
-word *unistr[UNICODESTRINGCNT];
+u16 *unistr[UNICODESTRINGCNT];
 int unistrp;
 
 static struct fencoding *
@@ -126,7 +128,7 @@ uni_parse(void) {
      * Parse the input stream and store encoding table in tempbuf.
      */
     lcode=0;
-    while(in_check_nschar(SP_ANYWCOMM)!=EOF) {
+    while(in_check_nschar(SP_ANYWCOMM)>=0) {
 	if(!cur)
 	    cur=start=mp_alloc_zero(page_pool,sizeof(ARRAY_ENTRY));
 	else
