@@ -11,6 +11,7 @@
 #include "gather/gather.h"
 #include "gather/format/pdf/pdf.h"
 
+#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -46,9 +47,9 @@ int
 ingetc(struct fastbuf *in) {
     int c;
     c=bgetc(in);
-    if(c==EOF && in==pdf_in)
+    if(c<0 && in==pdf_in)
 	gerror(2200,"(ingetc) Unexpected EOF");
-    if(c==EOF && in==pdf_stream_in)
+    if(c<0 && in==pdf_stream_in)
 	TRACE(10001,"End of stream");
     return c;
 }
@@ -61,8 +62,8 @@ ingetc(struct fastbuf *in) {
 
 int
 in_check_nschar(int spacetype) {
-    if(skip_space(spacetype)==EOF)
-	return EOF;
+    if(skip_space(spacetype)<0)
+	return -1;
     return in_check_char();
 }
 
@@ -134,18 +135,18 @@ in_get_line(char * buf) {
 
     do {
 	c=in_get_char();
-	if(c!=EOF && buf)
+	if(c>=0 && buf)
 	    buf[i]=c;
 	i++;
-    } while (c!=EOF && c!=10 && c!=13 && i<MAX_BUFF_SIZE);
-    if(c==EOF)
-	return EOF;
+    } while (c>=0 && c!=10 && c!=13 && i<MAX_BUFF_SIZE);
+    if(c<0)
+	return -1;
     if(buf)
 	buf[i-1]=0;
     if(c==13) {
 	c=in_get_char();
-	if(c==EOF)
-	    return EOF;
+	if(c<0)
+	    return -1;
 	if(c!=10)
 	    in_unget_char();
 	else
@@ -177,17 +178,17 @@ skip_space(uns type) {
     int c;
     int i=0;
 
-    while(is_space(c=in_get_char(),type) && c!=EOF) {
+    while(is_space(c=in_get_char(),type) && c>=0) {
 	TRACE(10001,"(skip_space) char %d",c);
 	if((type & SP_COMMENT) && c=='%') {
-	    if(in_get_line(NULL)==EOF)
+	    if(in_get_line(NULL)<0)
 		break;
 	}
 	i++;
     }
     TRACE(10001,"(skip_space) end char %d",c);
-    if(c==EOF)
-	return EOF;
+    if(c<0)
+	return -1;
     in_unget_char();
     return (i>0);
 }
@@ -199,9 +200,9 @@ get_tok(void) {
     byte *b=stringbuf, *e=stringbuf+STRINGBUFLEN;
 
     obj.type=OT_UTOK;
-    while(!is_space(c=in_get_char(),SP_ANYWDELIM) && c!=EOF && b<e)
+    while(!is_space(c=in_get_char(),SP_ANYWDELIM) && c>=0 && b<e)
 	*b++=c;
-    if(c!=EOF)
+    if(c>=0)
 	in_unget_char();
     *b=0;
     c=b-stringbuf+1;
@@ -245,7 +246,7 @@ get_int_real(void) {
 	gerror(2200,"(get_int_real) Wrong int/real number (start)");
     while((is_numchar(c=in_get_char()) || c=='.') && (i+=(c=='.'))<2 && b<e)
 	*b++=c;
-    if(c!=EOF)
+    if(c>=0)
 	in_unget_char();
     *b=0;
     if(i>=1) {
@@ -286,18 +287,18 @@ get_hexastring(void) {
 
     obj.type=OT_STRING;
     do {
-	if(skip_space(SP_ANYSPACE)==EOF)
+	if(skip_space(SP_ANYSPACE)<0)
 	    break;
 	c=in_get_char();
-	if(c=='>' || c==EOF)
+	if(c=='>' || c<0)
 	    break;
-	if(skip_space(SP_ANYSPACE)==EOF)
+	if(skip_space(SP_ANYSPACE)<0)
 	    break;
 	d=in_get_char();
-	*b++=(d=='>' || d==EOF) ? hexbyte(c,'0') : hexbyte(c,d);
+	*b++=(d=='>' || d<0) ? hexbyte(c,'0') : hexbyte(c,d);
 	if(b>=e)
 	    gerror(2203,"(get_hexastring) stringbuf overflow at %d",in_tell());
-    } while(d!='>' && d!=EOF);
+    } while(d!='>' && d>=0);
 
     obj.value.str.i=mp_alloc(page_pool,b-stringbuf);
     obj.value.str.l=b-stringbuf;
@@ -320,7 +321,7 @@ get_string(void) {
 	if(b>=e)
 	    gerror(2203,"(get_string) stringbuf overflow at %d",in_tell());
 	c=in_get_char();
-	if(c==EOF)
+	if(c<0)
 	    break;
 	*b=c;
 	switch(c) {
@@ -336,7 +337,7 @@ get_string(void) {
 	    case '\\':
 		c=in_get_char();
 		switch(c) { /* table of special chars */
-		    case EOF:
+		    case -1:
 			parenlev=0;
 			break;
 		    case '\r':
@@ -366,7 +367,7 @@ get_string(void) {
 			    c=in_get_char();
 			    i++;
 			}
-			if(c==EOF) {
+			if(c<0) {
 			    parenlev=0;	/* break from for cycle */
 			    break;
 			}
@@ -399,7 +400,7 @@ get_name(void) {
 	if(c=='#') {
 	    c=in_get_char();
 	    d=in_get_char();
-	    if(d==EOF) {
+	    if(d<0) {
 		b--;
 		break;
 	    }
@@ -414,7 +415,7 @@ get_name(void) {
 		gerror(2200,"(get_name) control character found in object name");
 	}
     }
-    if(c!=EOF)
+    if(c>=0)
 	in_unget_char();
     *b=0;
     obj.value.s=mp_alloc(page_pool,b-stringbuf+1);
@@ -462,7 +463,7 @@ get_obj(void) {
     c=in_check_nschar(SP_ANYWCOMM);
     TRACE(500,"get_obj: filepos=%d; TYPECHAR=%c",in_tell(),c);
     switch(c) {
-	case EOF:
+	case -1:
 	    gerror(2200,"(get_obj) Unexpected EOF");
 	    break;
 	case '0':
