@@ -461,7 +461,7 @@ cs_gettexts(struct resources *res) {
     gstack[1]=gstack[0];	/* initializing gs current state */
     gscur=gstack+1;
 
-    while(in_check_nschar(SP_ANYWCOMM)!=EOF) {
+    while(in_check_nschar(SP_ANYWCOMM) >= 0) {
 	if(!cur) {
 	    cur=start=mp_alloc_zero(page_pool,sizeof(ARRAY_ENTRY));
 	} else {
@@ -570,12 +570,12 @@ cs_gettexts(struct resources *res) {
 		     * exception from Content Stream syntax
 		     */
 		    do {
-			if(skip_space(SP_ANYWCOMM)==EOF)
+			if(skip_space(SP_ANYWCOMM) < 0)
 			    break;
 			obj=get_obj();
 		    } while(obj.type!=OT_UTOK || memcmp(obj.value.s,"ID",3));
 		    do {
-			if(skip_space(SP_ANYWDELIM)==EOF)
+			if(skip_space(SP_ANYWDELIM) < 0)
 			    break;
 			obj=get_tok();
 		    } while(memcmp(obj.value.s,"EI",3));
