@@ -71,7 +71,7 @@ extern uns pdf_warnings;
 
 typedef struct {
     s32 position;
-    word generation;
+    u16 generation;
 } OBJ_START;
 
 /* OBJECT types */
@@ -165,8 +165,8 @@ struct gsname {
  */
 
 struct unichar {
-    word fcode;
-    word unicode;
+    u16 fcode;
+    u16 unicode;
 };
 
 struct fencoding {
@@ -291,5 +291,5 @@ in_unirange(int u) {
     return ((u>=0x20 && u<0xe000) || (u>=0xf900 && u<=0xffff));
 }
 
-extern word *unistr[UNICODESTRINGCNT];
+extern u16 *unistr[UNICODESTRINGCNT];
 extern int unistrp;
