@@ -10,6 +10,7 @@
 #include "gather/gather.h"
 #include "gather/format/pdf/pdf.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <setjmp.h>
@@ -159,7 +160,7 @@ main(int argc, char **argv)
       set_input_method(pdf_stream_in);
       if (cmd == CMD_DUMP_STREAM)
 	{
-	  while (in_check_nschar(SP_ANYWCOMM)!=EOF)
+	  while (in_check_nschar(SP_ANYWCOMM) >= 0)
 	    {
 	      OBJECT o = get_obj();
 	      printobj(o);
