@@ -23,7 +23,7 @@ struct fchain {
     byte	    *data;
     byte	    *end;
     struct fchain   *prev;
-    addr_int_t	    priv;
+    uintptr_t	    priv;
 };
 
 static struct fchain *sf_top;
@@ -83,7 +83,7 @@ sf_get_nschar(int sptype, struct fchain *g) {
 		return c;
 	}
     }
-    return EOF;
+    return -1;
 }
 
 static inline void
@@ -143,7 +143,7 @@ sf_inflate(struct fchain *cur) {
     bufshift(cur,cur->end-cur->data);
     if (!d_stream) {
 	d_stream = mp_alloc_zero(sf_pool, sizeof(*d_stream));
-	cur->priv = (addr_int_t) d_stream;
+	cur->priv = (uintptr_t) d_stream;
 	err=inflateInit(d_stream);
 	if(err!=Z_OK)
 	    gerror(2223,"(sf_inflate) InflateInit error: %d",err);
@@ -190,7 +190,7 @@ sf_a85(struct fchain *cur) {
 	out=0;
 	for(n=0;n<5;n++) {
 	    c=sf_get_nschar(SP_ANYSPACE,cur->prev);
-	    if(c==EOF)
+	    if(c<0)
 		gerror(2223,"(sf_a85) Input error: unexpected EOD");
 	    if(c=='z' && n==0) {
 		cur->end[0]=cur->end[1]=cur->end[2]=cur->end[3]=0;
@@ -198,7 +198,7 @@ sf_a85(struct fchain *cur) {
 		break;
 	    }
 	    if(c=='~')
-		if(sf_get_nschar(SP_ANYSPACE,cur->prev)=='>' && sf_get_nschar(SP_ANYSPACE,cur->prev)==EOF) {
+		if(sf_get_nschar(SP_ANYSPACE,cur->prev)=='>' && sf_get_nschar(SP_ANYSPACE,cur->prev)<0) {
 		    cur->priv=1;
 		    return 1;
 		}
@@ -251,7 +251,7 @@ sf_rc4(struct fchain *cur) {
 
     TRACE(100, "(sf_rc4) Buffer %d-%d, avail_in: %d",(uns)(cur->data-cur->buf),(uns)(cur->end-cur->buf),avail_in);
     if(!avail_in) {
-	cur->priv=(addr_int_t) NULL;
+	cur->priv=(uintptr_t) NULL;
 	return (cur->end <= cur->data);
     }
     return 0;
@@ -339,7 +339,7 @@ check_i_stream(s32 n) {
 	sf_top=tf;
 	tf->end=tf->data=tf->buf;
 	rc4_state_stream = rc4_state;
-	tf->priv=(addr_int_t) &rc4_state_stream;
+	tf->priv=(uintptr_t) &rc4_state_stream;
 	tf->func=sf_rc4;
 	TRACE(200,"(check_i_stream) sff=RC4DECRYPT");
 	encrypted&=~1;		/* streams are encrypted _outside_, no RC4 decryption inside */
