@@ -30,7 +30,7 @@ flush_word(struct fastbuf *out, u16 *w, int len)
 }
 
 int
-text_parse(void)
+text_parse(byte **args UNUSED)
 {
   struct fastbuf *in, *out;
   u16 w[MAX_WORD_LEN];
