@@ -84,22 +84,15 @@ typedef unsigned long 	cmp_code_int;
 
 #define MAXCODE(n)	(1L << (n))
 
-#ifdef CPU_BIG_ENDIAN
-#  define input(b,o,c,n,m){ \
-	(c) = (*(long *)(&(b)[(o)>>3])>>((o)&0x7))&(m); \
-		(o) += (n); \
-}
-#else
 #  define input(b,o,c,n,m){ \
 	char_type *p = &(b)[(o)>>3]; \
 		(c) = ((((long)(p[0]))|((long)(p[1])<<8)| \
 			((long)(p[2])<<16))>>((o)&0x7))&(m); \
 			(o) += (n); \
 }
-#endif
 
 #define tab_prefixof(i) tab_prefix[i]
-#define clear_tab_prefixof()	bzero(tab_prefix, 256);
+#define clear_tab_prefixof()	bzero(tab_prefix, 256 * sizeof(*tab_prefix));
 
 #define de_stack        ((char_type *)(&d_buf[DIST_BUFSIZE-1]))
 #define tab_suffixof(i) tab_suffix[i]
@@ -124,7 +117,7 @@ compress_parse(byte **args UNUSED)
 
 	uch inbuf[INBUFSIZ+INBUF_EXTRA];	/* input buffer */
 	uch outbuf[OUTBUFSIZ+OUTBUF_EXTRA];	/* output buffer */
-	ush d_buf[DIST_BUFSIZE];		/* buffer for distances, see trees.c */
+	uch d_buf[DIST_BUFSIZE];		/* buffer for distances, see trees.c */
 	uch window[WSIZE];			/* Sliding window and suffix table (unlzw) */
 #define tab_suffix window
 #define tab_prefix prev				/* hash link (see deflate.c) */
