@@ -7,6 +7,7 @@
 
 #include "sherlock/sherlock.h"
 #include "gather/gather.h"
+#include "lib/ff-binary.h"
 
 #include <string.h>
 #include <zlib.h>
@@ -51,12 +52,12 @@ check_gzip_header(struct fastbuf *in)
 		len = bgetc(in);
 		len += bgetc(in) << 8;
 		/* len is garbage if EOF but the loop below will quit anyway */
-		while (len-- != 0 && (c = bgetc(in)) != EOF) ;
+		while (len-- != 0 && (c = bgetc(in)) >= 0) ;
 	}
 	if ((flags & ORIG_NAME) != 0) { /* skip the original file name */
 		byte url[MAX_URL_SIZE];
 		uns i = 0;
-		while ((c = bgetc(in)) != 0 && c != EOF)
+		while ((c = bgetc(in)) != 0 && c >= 0)
 			if (i<MAX_URL_SIZE)
 				url[i++] = c;
 		url[i] = 0;
@@ -65,12 +66,12 @@ check_gzip_header(struct fastbuf *in)
 		TRACE("Changing filename after ungziping to %s", url);
 	}
 	if ((flags & COMMENT) != 0) {   /* skip the .gz file comment */
-		while ((c = bgetc(in)) != 0 && c != EOF) ;
+		while ((c = bgetc(in)) != 0 && c >= 0) ;
 	}
 	if ((flags & HEAD_CRC) != 0) {  /* skip the header crc */
 		bgetw(in);
 	}
-	return c == EOF ? Z_DATA_ERROR : Z_OK;
+	return c < 0 ? Z_DATA_ERROR : Z_OK;
 }
 
 #define	ERR(nr, txt)	do { err=nr; msg=txt; goto bye; } while(0)
@@ -153,7 +154,7 @@ incomplete_stream:
 			crc1 += bgetc(in) << 16;
 			crc1 += bgetc(in) << 24;
 			for (i=0; i<4; i++)
-				if (bgetc(in) == EOF)
+				if (bgetc(in) < 0)
 					goto incomplete_stream;
 			if (crc1 != crc)
 				ERR(2501, "Gzip CRC error");
