@@ -64,6 +64,7 @@ struct gobject {
   byte *etag;				/* HTTP entity tag */
   byte *if_different_etag;		/* Download only if etag differs */
   unsigned int orig_size;		/* Size of original file */
+  int expected_size;			/* Expected size of original file if truncated */
   byte truncated;
   unsigned int conv_count;		/* Conversion counter */
   byte MD5[MD5_SIZE];			/* MD5 sum of document contents */
@@ -135,6 +136,7 @@ void gather_filter_undo(void);
 /* ganalyse.c */
 
 void gather_init_analyser(void);
+void gather_raw_analyse(void);
 int gather_analysis_needed(void);
 void gather_analyse(void);
 
