@@ -13,6 +13,7 @@
 #include "sherlock/bucket.h"
 #include "gather/gather.h"
 
+#include <stdio.h>
 #include <setjmp.h>
 #include <unistd.h>
 #include <string.h>
@@ -88,10 +89,10 @@ do_url(byte *url)
     }
   if (gthis->error_code <= 1)
     {
-      struct fastbuf *b = obuck_create();
+      struct fastbuf *b = obuck_create(&bucket_file);
       uns out_format = gobj_write(b, BUCKET_TYPE_V33_LIZARD, GWF_DUMP_BODY);
       struct obuck_header buck;
-      obuck_create_end(b, out_format, &buck);
+      obuck_create_end(&bucket_file, b, out_format, &buck);
       log(L_INFO, "%s: %04d %s [%x]", gthis->url, gthis->error_code, gthis->error_msg, buck.oid);
     }
   else
@@ -141,7 +142,7 @@ main(int argc, char **argv)
 
   gatherer_init();
   set_base_url();
-  obuck_init(1);
+  bucket_open(1);
   if (optind == argc)
     {
       byte x[MAX_URL_SIZE];
@@ -151,6 +152,6 @@ main(int argc, char **argv)
     }
   else while (optind < argc)
     do_one(argv[optind++]);
-  obuck_cleanup();
+  bucket_close();
   return 0;
 }
