@@ -44,7 +44,7 @@ gather_init_filter(void)
     return;
   gf_filter = filter_load(gather_filter_name, filter_builtin_vars, gf_bindings, NULL);
   gf_filter_args = filter_intr_new(gf_filter);
-  gf_filter_args->want_config_changes = 1;
+  gf_filter_args->config_changes_mode = 1;
 }
 
 void
