@@ -485,12 +485,14 @@ recv_header(FILE *f)
   byte buf[1024], buf2[1024];
   byte *k, *l;
   uns maxlin = 0;
+  uns warned = 0;
 
   buf2[0] = 0;
   l = NULL;
   firsthdr = NULL;
   for(;;)
     {
+next:
       if (maxlin++ > max_hdr_lines)
 	gerror(2113, "Maximal number of header lines (%d) exceeded", max_hdr_lines);
       if (!fgets(buf, sizeof(buf), f))
@@ -501,6 +503,13 @@ recv_header(FILE *f)
       if (!k)
 	gerror(2110, "Header line too long");
       *k = 0;
+      for (k = buf; *k; k++)
+	if ((*k < 0x20 || *k >= 0x80) && *k != '\t')
+	  {
+	    if (!warned++)
+	      log(L_WARN_R, "Found non-ASCII character(s) in HTTP header");
+	    goto next;
+	  }
       if (response_code < 0)
 	{
 	  parse_resphdr(buf);
@@ -663,6 +672,8 @@ recv_body_normal(FILE *f)
       if ((uns)sum > max_obj_size)
 	{
 	  gobj_truncate();
+	  if (expected_length >= sum)
+	    gthis->expected_size = expected_length;
 	  return;
 	}
     }
@@ -774,6 +785,7 @@ download_url(byte *host, uns port, byte *rest)
   parse_hdr();
   alarm(body_timeout);
   recv_body(f);
+  fclose(f);
 }
 
 void
