@@ -1,7 +1,7 @@
 /*
  *	Sherlock Gatherer: Protocol Multiplexer
  *
- *	(c) 2001--2004 Martin Mares <mj@ucw.cz>
+ *	(c) 2001--2006 Martin Mares <mj@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
@@ -17,16 +17,18 @@ static downloader_t downloaders[URL_PROTO_MAX] = {
 };
 
 void
-download(void)
+gather_download(void)
 {
   if (!downloaders[gthis->url_s.protoid])
     gerror(2100, "Unknown protocol");
-  gather_filter();
+  gather_filter(0);
   if (gthis->refreshing)
     {
       byte *last_dwn = obj_find_aval(gthis->refreshing, 'D');
       byte *last_mod = obj_find_aval(gthis->refreshing, 'L');
-      if (last_dwn && gthis->start_time - (sh_time_t)atol(last_dwn) <= max_refresh_age)
+      if (last_dwn &&
+	  gthis->start_time - (sh_time_t)atol(last_dwn) <= max_refresh_age &&
+	  !gather_analysis_needed())
 	{
 	  if (last_mod && min_ims_delay >= 0)
 	    gthis->if_modified_since_time = atol(last_mod) + min_ims_delay;
