@@ -6,12 +6,11 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
 #include "lib/unicode.h"
 #include "lib/slists.h"
 #include "charset/unicat.h"
 #include "indexer/alphabet.h"
-#include "sherlock/conf-parse.h"
+#include "sherlock/conf.h"
 
 #include <string.h>
 
