@@ -2,7 +2,7 @@
  *	Sherlock Indexer -- Processing of Frames, Backlinks and Image Referers
  *
  *	(c) 2002--2006 Martin Mares <mj@ucw.cz>
- *	(c) 2003--2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2003--2006 Robert Spalek <robert@ucw.cz>
  *
  *	This module performs six tasks:
  *
@@ -35,12 +35,15 @@
 #include "lib/conf.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/bitarray.h"
 #include "lib/math.h"
 #include "sherlock/object.h"
 #include "indexer/indexer.h"
 #include "indexer/merges.h"
+#include "indexer/graph.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 
@@ -73,22 +76,18 @@ flatten_merges(void)
 static void
 analyse_redirs(void)
 {
-  s32 v;
-  uns d, loops, cnt;
-  u32 w, t;
+  uns loops, cnt;
+  u32 w, v, d, t;
 
   loops = cnt = 0;
-  while ((v = bgetl(graph)) >= 0)
+  while (bget_graph_hdr(graph, &w, &d))
     {
-      d = bgetw(graph);
       while (d--)
 	{
-	  w = bgetl(graph);
-	  t = w & ETYPE_MASK;
-	  w &= ~ETYPE_MASK;
-	  if (w >= FIRST_ID_NEW)
-	    continue;
-	  ASSERT((uns)v < card_count && w < card_count);
+	  v = bgetl(graph);
+	  t = v & ETYPE_IMAGE;
+	  v &= ~ETYPE_MASK;
+	  ASSERT(v < card_count && (uns)w < card_count);
 	  /* Found edge v->w of type t */
 	  switch (t)
 	    {
@@ -234,17 +233,14 @@ analyse_links(void)
   for (v=0; v<card_count; v++)
     best[v] = ~0U;
 
-  while ((v = bgetl(graph)) != ~0U)
+  while (bget_graph_hdr(graph, &w, &d))
     {
-      d = bgetw(graph);
       while (d--)
 	{
-	  w = bgetl(graph);
-	  if ((w & ETYPE_MASK) != ETYPE_NORMAL)
-	    continue;
-	  w &= ~ETYPE_MASK;
-	  if (w >= FIRST_ID_NEW)
+	  v = bgetl(graph);
+	  if ((v & ETYPE_IMAGE) != ETYPE_NORMAL)
 	    continue;
+	  v &= ~ETYPE_MASK;
 	  v = merges_follow(v);
 	  w = merges_follow(w);
 	  if (v == w)
@@ -305,19 +301,16 @@ static void
 propagate_image_attrs(void)
 {
   uns cnt=0;
-  u32 v, w, t;
+  u32 v, w, d, t;
 
   attrs_map(1);
-  while ((v = bgetl(graph)) != ~0U)
+  while (bget_graph_hdr(graph, &w, &d))
     {
-      uns d = bgetw(graph);
       while (d--)
 	{
-	  w = bgetl(graph);
-	  t = w & ETYPE_MASK;
-	  w &= ~ETYPE_MASK;
-	  if (w >= FIRST_ID_NEW)
-	    continue;
+	  v = bgetl(graph);
+	  t = v & ETYPE_IMAGE;
+	  v &= ~ETYPE_MASK;
 	  ASSERT(v < card_count && w < card_count);
 	  if (t == ETYPE_NORMAL || ETYPE_IMAGE)
 	    {
@@ -423,7 +416,7 @@ write_backlinks(void)
   bgets(url_list, buf, sizeof(buf));
   while ((v = bgetl(label_graph)) != ~0U)
     {
-      switch (v & ETYPE_MASK)
+      switch (v & ETYPE_IMAGE)
 	{
 	case ETYPE_NORMAL:
 	  type = 'z'; break;
@@ -516,7 +509,7 @@ main(int argc, char **argv)
   if (phases & 1)
   {
     /* Scan the graph, analyse redirects and extract frame and image edges */
-    graph = index_bopen(fn_link_graph, O_RDONLY);
+    graph = index_bopen(fn_graph_obj, O_RDONLY);
     frame_graph = index_bopen(frame_path, O_RDWR | O_CREAT | O_TRUNC);
     image_graph = index_bopen(image_path, O_WRONLY | O_CREAT | O_TRUNC);
     log(L_INFO, "Analysing redirects");
@@ -534,7 +527,7 @@ main(int argc, char **argv)
   if (phases & 2)
   {
     /* Analyse superframes */
-    graph = index_bopen(fn_link_graph, O_RDONLY);
+    graph = index_bopen(fn_graph_obj, O_RDONLY);
     label_graph = bopen_tmp(indexer_fb_size);
     frame_graph = index_bopen(frame_path, O_RDONLY);
     analyse_superframes();
@@ -596,7 +589,7 @@ main(int argc, char **argv)
   if (phases & 2)
     {
       /* Propagate image attributes */
-      graph = index_bopen(fn_link_graph, O_RDONLY);
+      graph = index_bopen(fn_graph_obj, O_RDONLY);
       propagate_image_attrs();
       set_unlinked_image_attrs();
       bclose(graph);
