@@ -3,19 +3,20 @@
  *
  *	(c) 2001--2006 Martin Mares <mj@ucw.cz>
  *	(c) 2003--2006 Robert Spalek <robert@ucw.cz>
+ *	(c) 2006--2007 Pavel Charvat <pchar@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
 #include "lib/lists.h"
 #include "lib/simple-lists.h"
-#include "lib/conf.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/mempool.h"
 #include "lib/url.h"
 #include "sherlock/object.h"
 #include "sherlock/attrset.h"
-#include "sherlock/conf-parse.h"
+#include "sherlock/conf.h"
 #include "lib/hashfunc.h"
 #include "sherlock/tagged-text.h"
 #include "lib/unicode.h"
@@ -26,6 +27,7 @@
 #include "indexer/lexicon.h"
 #include "indexer/params.h"
 
+#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -128,8 +130,8 @@ struct out_index {
   byte *log_prefix;
 };
 
-#define CARD_ID_MASK 0x3fffffff
-#define INDEX_ID_SHIFT 30
+#define CARD_ID_MASK 0x1fffffff
+#define INDEX_ID_SHIFT 29
 
 static inline uns id_to_card(uns id) { return id & CARD_ID_MASK; }
 static inline uns id_to_idx(uns id) { return id >> INDEX_ID_SHIFT; }
@@ -140,11 +142,12 @@ static uns num_out_indices;
 
 static struct out_index *current_out;
 static uns card_id;
-static uns card_cnt;
 
 static void
 out_init(void)
 {
+  if (!fn_directory)
+    die("Indexer.Directory not set");
   if (!clist_empty(&subindices))
     {
       num_out_indices = 0;
@@ -181,7 +184,7 @@ out_find(struct card_attr *attr, struct card_note *note)
 #else
   ft = 0;
 #endif
-  id = get_subindexing_id(card_cnt, note);
+  id = get_subindexing_id(fetch_id, note);
 
   for (uns i=0; i<num_out_indices; i++)
     if ((out_index[i].type_mask & (1 << ft)) &&
@@ -400,7 +403,7 @@ static void
 string_close(struct out_index *out)
 {
   bclose(out->string_index);
-  log(L_INFO, "%sGenerated %Ld strings in %d runs; %Ld strings dropped", out->log_prefix, out->string_cnt, string_runs, out->string_dropped_cnt);
+  log(L_INFO, "%sGenerated %lld strings in %d runs; %lld strings dropped", out->log_prefix, (long long) out->string_cnt, string_runs, (long long) out->string_dropped_cnt);
 }
 
 static void
@@ -545,7 +548,7 @@ static void
 preproc_card(struct odes *o)
 {
   byte *w = doc_buf;
-  byte *stop = doc_buf + doc_buf_size - MAX_ATTR_SIZE;
+  byte *stop = doc_buf + doc_buf_size;
 
   byte *version = obj_find_aval(o, 'v');
   if (!version)
@@ -589,7 +592,7 @@ preproc_card(struct odes *o)
       r = oa->val;
       uns c;
 
-      if (w >= stop)
+      if (w >= stop - strlen(r) - 1)
 	{
 	  trim_counter++;
 	  break;
@@ -781,7 +784,7 @@ penalize_card(struct fastbuf *cards_mem, struct card_attr *attr, struct card_not
     bput_attr_format(cards_mem, '.', "Penalized by %d: giant class", giant_penalty);
     wt -= giant_penalty;
   }
-  if (!is_image && !is_in_catalog)
+  if (!is_image && !is_in_catalog && !(note->flags & CARD_NOTE_AUDIO))
   {
     if (no_contents_penalty && note->useful_size < no_contents_threshold)
     {
@@ -1004,7 +1007,7 @@ word_flush(int final)
   nr_used_words = 0;
 
   if (final)
-    ITRACE("Words used %Ld entries, that is %Ld bytes", (long long)word_entries, (long long)(word_entries * sizeof(lentry)));
+    ITRACE("Words used %lld entries, that is %lld bytes", (long long)word_entries, (long long)(word_entries * sizeof(lentry)));
 }
 
 static inline int
@@ -1052,7 +1055,7 @@ word_add(struct verbum *v, uns type, int pos)
 }
 
 static enum word_class
-lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp)
+lm_lookup(enum word_class orig_class, u16 *uni, uns ulen, word_id_t *idp, void *user UNUSED)
 {
   struct verbum *v;
 
@@ -1064,7 +1067,7 @@ lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp)
 }
 
 static void
-lm_got_word(uns pos, uns cat, word_id_t w)
+lm_got_word(uns pos, uns cat, word_id_t w, void *user UNUSED)
 {
   if (meta_static_part)
     {
@@ -1088,14 +1091,14 @@ lm_got_word(uns pos, uns cat, word_id_t w)
 
 #ifdef CONFIG_CONTEXTS
 static inline void
-lm_got_complex(uns pos, uns cat, word_id_t root, word_id_t w, uns dir)
+lm_got_complex(uns pos, uns cat, word_id_t root, word_id_t w, uns dir, void *user UNUSED)
 {
   struct verbum *v = context_words[root->context_class] + w->context_class + (dir ? lex_context_slots : 0);
-  lm_got_word(pos, cat, v);
+  lm_got_word(pos, cat, v, user);
 }
 #else
 static inline void
-lm_got_complex(uns pos UNUSED, uns cat UNUSED, word_id_t root UNUSED, word_id_t w UNUSED, uns dir UNUSED)
+lm_got_complex(uns pos UNUSED, uns cat UNUSED, word_id_t root UNUSED, word_id_t w UNUSED, uns dir UNUSED, void *user UNUSED)
 {
 }
 #endif
@@ -1109,6 +1112,7 @@ struct meta_info {
 };
 static struct meta_info *meta_first[16], *meta_last[16];
 static struct fastbuf *analyse_meta_fb;
+static struct mempool *meta_pool;
 
 static inline void
 word_meta_preprocess(struct oattr *av, struct mempool *pool)
@@ -1155,13 +1159,13 @@ word_meta(struct odes *o, struct card_note *note)
 {
   uns permit_types = (note->flags & CARD_NOTE_GIANT) ? ~giant_ban_meta : ~0U;
   bzero(meta_first, sizeof(meta_first));
-  struct meta_info *mi = alloca(sizeof(*mi));
+  mp_flush(meta_pool);
+  struct meta_info *mi = mp_alloc_fast(meta_pool, sizeof(*mi));
 
   if (analyse_meta_fb)
     fbgrow_reset(analyse_meta_fb);
 
-  /* These must be macros, not functions, because we want to use alloca(). */
-#define ADD_META(a) if (word_meta_add(a->val, mi, permit_types)) mi=alloca(sizeof(*mi))
+#define ADD_META(a) if (word_meta_add(a->val, mi, permit_types)) mi=mp_alloc_fast(meta_pool, sizeof(*mi))
 #define DO_METAS(x) for (struct oattr *a=obj_find_attr(x, 'M'); a; a=a->same) ADD_META(a)
 #define DO_CAT(y) for (struct oattr *c=obj_find_attr(y, 'c' + OBJ_ATTR_SON); c; c=c->same) DO_METAS(c->son)
 
@@ -1194,7 +1198,7 @@ word_meta(struct odes *o, struct card_note *note)
   for (uns type=0; type<16; type++)
     if (meta_first[type])
       {
-	lm_doc_start();
+	lm_doc_start(NULL);
 	for (struct meta_info *m=meta_first[type]; m; m=m->next)
 	  {
 	    meta_static_part = m->static_part;
@@ -1211,7 +1215,7 @@ word_meta(struct odes *o, struct card_note *note)
 static void
 word_card(struct odes *o, struct card_note *note)
 {
-  lm_doc_start();
+  lm_doc_start(NULL);
   meta_static_part = 0;
   lm_map_text(doc_buf, doc_buf + doc_length);
   word_meta(o, note);
@@ -1226,6 +1230,7 @@ word_init(void)
   lm_init();
   lex_load();
   word_pool = mp_new(sizeof(lentry) * LENT_BITE);
+  meta_pool = mp_new(4096);
   lentry_limit = word_buf_size / (sizeof(lentry) + sizeof(struct lentry *));
   uw_init(&used_words_buf);
   uw_grow(&used_words_buf, lentry_limit);
@@ -1243,7 +1248,7 @@ static void
 word_close(struct out_index *out)
 {
   bclose(out->word_index);
-  log(L_INFO, "%sGenerated %Ld word refs in %d runs", out->log_prefix, out->word_cnt, word_runs);
+  log(L_INFO, "%sGenerated %lld word refs in %d runs", out->log_prefix, (long long) out->word_cnt, word_runs);
 }
 
 /*
@@ -1412,9 +1417,12 @@ card_card(struct odes *o, struct card_attr *attr, struct card_note *note)
   /* Dump reftexts */
   bput_oattr_nocheck(cards_mem, obj_find_attr(o, 'x' + OBJ_ATTR_SON));
 
-  /* Then dump all other attributes */
+  /* Then dump all other attributes (subobjects first because of the "CAVEAT" above) */
   for (struct oattr *at=o->attrs; at; at=at->next)
-    if (attr_set_match(&card_attr_set, at))
+    if (at->attr >= OBJ_ATTR_SON && attr_set_match(&card_attr_set, at))
+      bput_oattr_nocheck(cards_mem, at);
+  for (struct oattr *at=o->attrs; at; at=at->next)
+    if (at->attr < OBJ_ATTR_SON && attr_set_match(&card_attr_set, at))
       bput_oattr_nocheck(cards_mem, at);
 
   /* Perform final penalization and dump notes on evolution of the weight */
@@ -1489,6 +1497,8 @@ prints_card(struct odes *o)
  */
 
 static struct fastbuf analyse_text_fb;
+static struct an_context an_context;
+static struct mempool *an_pool;
 
 #define GBUF_TYPE	struct odes *
 #define GBUF_PREFIX(x)	ob_##x
@@ -1498,8 +1508,10 @@ static ob_t url_block_buf;
 static void
 analyse_init(void)
 {
-  analyser_init(AN_HOOK_CHEWER, AN_NEED_TEXT | AN_NEED_METAS | AN_NEED_ALL_URLS);
-  if (an_hook_need_mask & AN_NEED_METAS)
+  an_pool = mp_new(0x2000);
+  analyser_init_hook(AN_HOOK_CHEWER);
+  analyser_init(&an_context, AN_HOOK_CHEWER, AN_NEED_TEXT | AN_NEED_METAS | AN_NEED_ALL_URLS, NULL);
+  if (an_context.need_mask & AN_NEED_METAS)
     analyse_meta_fb = fbgrow_create(4096);
   ob_init(&url_block_buf);
 }
@@ -1508,7 +1520,9 @@ static void
 analyse_end(void)
 {
   bclose(analyse_meta_fb);
-  analyser_log_stats();
+  analyser_log_stats(&an_context);
+  analyser_cleanup(&an_context);
+  mp_delete(an_pool);
 }
 
 static void
@@ -1517,10 +1531,11 @@ analyse_card(struct odes *o)
   struct oattr *ua = obj_find_attr(o, 'U' + OBJ_ATTR_SON);
   struct an_iface ai = {
     .obj = o,
-    .url_block = ua->son
+    .url_block = ua->son,
+    .pool = an_pool
   };
 
-  if (an_hook_need_mask & AN_NEED_ALL_URLS)
+  if (an_context.need_mask & AN_NEED_ALL_URLS)
     {
       uns i = 0;
       for (; ua; ua=ua->same)
@@ -1538,7 +1553,7 @@ analyse_card(struct odes *o)
       ai.all_urls = url_block_buf.ptr;
     }
 
-  uns need = analyser_need(&ai);
+  uns need = analyser_need(&an_context, &ai);
   if (need)
     {
       if (need & AN_NEED_TEXT)
@@ -1551,8 +1566,9 @@ analyse_card(struct odes *o)
 	  fbgrow_rewind(analyse_meta_fb);
 	  ai.metas = analyse_meta_fb;
 	}
-      analyser_run_needed(&ai);
+      analyser_run_needed(&an_context, &ai);
     }
+  mp_flush(an_pool);
 }
 
 /*
@@ -1568,14 +1584,57 @@ params_init(void)
 }
 
 static void
-params_open(struct out_index *out)
+params_close(struct out_index *out)
 {
   struct index_params par = global_params;
   par.type_mask = out->type_mask;
   par.id_mask = out->id_mask;
+  par.cards_out = out->card_cnt;
   params_save(&par);
 }
 
+/*
+ *  Processing of image signatures
+ */
+
+static inline void images_open(struct out_index *out UNUSED) {}
+static inline void images_close(struct out_index *out UNUSED) {}
+static inline void images_card(struct odes *o UNUSED) {}
+
+/*
+ *  Processing of admin export
+ */
+
+static struct fastbuf *admin_out;
+
+static void
+admin_init(void)
+{
+  admin_out = index_maybe_bopen(fn_admin_export, O_CREAT | O_TRUNC | O_WRONLY);
+}
+
+static void
+admin_end(void)
+{
+  if (admin_out)
+    bclose(admin_out);
+}
+
+static void
+admin_card(int idx, struct card_info *info)
+{
+  if (admin_out)
+    {
+      struct admin_export n = {
+	.subindex = idx,
+	.flags = info->note.flags,
+	.attr = info->attr,
+      };
+      memcpy(n.footprint, info->note.footprint, 16);
+      bwrite(admin_out, &n, sizeof(n));
+    }
+}
+
 /*
  *  Main loop
  */
@@ -1583,8 +1642,11 @@ params_open(struct out_index *out)
 static uns num_dropped_cards;
 
 static void
-chew_card(struct card_attr *attr, struct odes *o, struct card_note *note)
+chew_card(struct card_info *info, struct odes *o)
 {
+  struct card_attr *attr = &info->attr;
+  struct card_note *note = &info->note;
+
   int idx = out_find(attr, note);
   if (idx < 0)
     {
@@ -1594,6 +1656,9 @@ chew_card(struct card_attr *attr, struct odes *o, struct card_note *note)
   current_out = &out_index[idx];
   card_id = make_id(idx, current_out->card_cnt);
 
+#ifdef CUSTOM_CHEW_PREPROC
+  CUSTOM_CHEW_PREPROC(o, attr);
+#endif
   preproc_card(o);
   if (!raw_stage2_input)
     preproc_reftexts(o);
@@ -1603,11 +1668,16 @@ chew_card(struct card_attr *attr, struct odes *o, struct card_note *note)
   analyse_card(o);
   string_card(o);
   card_card(o, attr, note);
+  images_card(o);
+#ifdef CUSTOM_CHEW_ATTRS
+  CUSTOM_CHEW_ATTRS(o, attr);
+#endif
   bwrite(current_out->card_attrs, attr, sizeof(*attr));
+  admin_card(idx, info);
 
-  PROGRESS(card_id, "chewer: %d cards", card_cnt);
+  PROGRESS(fetch_id, "chewer: %d cards (%d%%)",
+	   fetch_id, (int)((float)fetch_id/fetch_num_ids*100));
   current_out->card_cnt++;
-  card_cnt++;
 }
 
 int
@@ -1629,6 +1699,7 @@ main(int argc, char **argv)
   string_init();
   params_init();
   analyse_init();
+  admin_init();
   byte *orig_directory = fn_directory;
   for (uns i=0; i<num_out_indices; i++)
     {
@@ -1640,11 +1711,11 @@ main(int argc, char **argv)
       string_open(out);
       cards_open(out);
       prints_open(out);
-      params_open(out);
+      images_open(out);
     }
   fn_directory = orig_directory;
 
-  log(L_INFO, "Chewing cards and creating indices");
+  log(L_INFO, "Chewing cards and creating indices with %d slices", num_slices);
   fetch_cards(chew_card);
 
   if (!raw_stage2_input)
@@ -1656,13 +1727,17 @@ main(int argc, char **argv)
       struct out_index *out = &out_index[i];
       ITRACE("Closing index %s", out->name);
       current_out = out;
+      fn_directory = out->directory;
       word_close(out);
       string_close(out);
       cards_close(out);
       prints_close(out);
+      images_close(out);
+      params_close(out);
     }
   cards_end();
   analyse_end();
+  admin_end();
 
   if (num_dropped_cards)
     log(L_INFO, "%d cards did not belong to any subindex", num_dropped_cards);
