@@ -1,12 +1,13 @@
 /*
  *	Sherlock Indexer -- Fetching of Cards for Second Indexer Pass
  *
- *	(c) 2001--2005 Martin Mares <mj@ucw.cz>
+ *	(c) 2001--2006 Martin Mares <mj@ucw.cz>
  *	(c) 2003--2005 Robert Spalek <robert@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/mempool.h"
 #include "lib/url.h"
 #include "sherlock/object.h"
@@ -18,84 +19,67 @@
 #include <fcntl.h>
 
 uns fetch_id;
+uns fetch_num_ids;
 
 static void
-fetch_cards_raw(void (*got_card)(struct card_attr *attr, struct odes *obj, struct card_note *note))
+fetch_cards_raw(void (*got_card)(struct card_info *info, struct odes *obj))
 {
   struct mempool *pool;
-  struct fastbuf *fb_attrs, *fb_notes;
-  struct odes *o;
-  struct card_attr attr;
-  struct card_note note;
-  struct bucket_source *src;
+  struct fastbuf *fb_infos;
+  struct card_info info;
+  struct get_buck gb;
 
   pool = mp_new(16384);
-  src = get_buck_init();
-  fb_attrs = index_bopen(fn_attributes, O_RDONLY);
-  fb_notes = index_bopen(fn_notes, O_RDONLY);
+  get_buck_init(&gb);
+  gb.pool = pool;
+  fb_infos = index_bopen(fn_card_info, O_RDONLY);
+  fetch_num_ids = bfilesize(fb_infos) / sizeof(struct card_info);
   fetch_id = -1;
-  while (breadb(fb_attrs, &attr, sizeof(attr)))
+  while (breadb(fb_infos, &info, sizeof(info)))
     {
       fetch_id++;
-      breadb(fb_notes, &note, sizeof(note));
       mp_flush(pool);
-      if (!src->get_next(src, pool, attr.card))
+      if (!get_buck_next(&gb, info.attr.card))
 	die("Ran out of buckets");
-      o = src->o;
-      got_card(&attr, src->o, &note);
+      got_card(&info, gb.o);
     }
-  src->cleanup(src);
-  bclose(fb_attrs);
-  bclose(fb_notes);
+  get_buck_cleanup(&gb);
+  bclose(fb_infos);
   mp_delete(pool);
   log(L_INFO, "Processed %d objects", fetch_id+1);
 }
 
 void
-fetch_cards(void (*got_card)(struct card_attr *attr, struct odes *obj, struct card_note *note))
+fetch_cards(void (*got_card)(struct card_info *info, struct odes *obj))
 {
   struct mempool *pool;
-  struct fastbuf *fb_labels, *fb_attrs, *fb_notes;
+  struct fastbuf *fb_labels, *fb_infos;
   struct odes *o;
   u32 next_label_id;
-  struct card_attr attr;
-  struct card_note note;
+  struct card_info info;
   struct odes *main_hdr, *url_hdr, *current_hdr;
   struct obj_read_state read_state;
-  struct bucket_source *src;
+  struct get_buck gb;
 
   if (raw_stage2_input)
     return fetch_cards_raw(got_card);
 
   pool = mp_new(16384);
-  src = get_buck_init();
+  get_buck_init(&gb);
+  gb.pool = pool;
   fb_labels = index_bopen(fn_labels, O_RDONLY);
-  fb_attrs = index_bopen(fn_attributes, O_RDONLY);
-  fb_notes = index_bopen(fn_notes, O_RDONLY);
+  fb_infos = index_bopen(fn_card_info, O_RDONLY);
+  fetch_num_ids = bfilesize(fb_infos) / sizeof(struct card_info);
   fetch_id = -1;
   next_label_id = bgetl(fb_labels);
-  while (breadb(fb_attrs, &attr, sizeof(attr)))
+  while (breadb(fb_infos, &info, sizeof(info)))
     {
       fetch_id++;
-      breadb(fb_notes, &note, sizeof(note));
-      if (attr.flags & (CARD_FLAG_EMPTY | CARD_FLAG_DUP))
-	{
-	  while (next_label_id == fetch_id)
-	    {
-	      bgetl(fb_labels);
-	      bgetl(fb_labels);
-	      uns cnt = bgetl(fb_labels);
-	      /* This really can happen, for example when we attach frame backlinks to empty frames */
-	      DBG("Ignoring label for skipped object %08x", fetch_id);
-	      bskip(fb_labels, cnt+1);
-	      next_label_id = bgetl(fb_labels);
-	    }
-	  continue;
-	}
+      ASSERT(!(info.attr.flags & (CARD_FLAG_EMPTY | CARD_FLAG_DUP)));	// should have been filtered out in stage 1
       mp_flush(pool);
-      if (!src->get_next(src, pool, attr.card))
+      if (!get_buck_next(&gb, info.attr.card))
 	die("Object pool changed unexpectedly");
-      o = src->o;
+      o = gb.o;
 
       /* Pick all per-URL attributes and move them to the main header */
       main_hdr = obj_new(pool);
@@ -109,7 +93,7 @@ fetch_cards(void (*got_card)(struct card_attr *attr, struct odes *obj, struct ca
 	  }
 	else
 	  Oa = &oa->next;
-      if (attr.flags & CARD_FLAG_MERGED)		// merged cards get all per-URL attributes from labels
+      if (info.attr.flags & CARD_FLAG_MERGED)		// merged cards get all per-URL attributes from labels
 	main_hdr = obj_new(pool);
       obj_add_son_ref(o, 'U' + OBJ_ATTR_SON, main_hdr);
       current_hdr = url_hdr = main_hdr;
@@ -158,9 +142,9 @@ fetch_cards(void (*got_card)(struct card_attr *attr, struct odes *obj, struct ca
 	  obj_read_end(&read_state);
 	  next_label_id = bgetl(fb_labels);
 	}
-      if (note.useful_size)
-	obj_set_attr_num(o, 'u', note.useful_size);
-      if (attr.flags & CARD_FLAG_FRAMESET)
+      if (info.note.useful_size)
+	obj_set_attr_num(o, 'u', info.note.useful_size);
+      if (info.attr.flags & CARD_FLAG_FRAMESET)
 	{
 	  /* Frameset to be deleted, but CARD_FLAG_EMPTY is no longer set,
 	   * which means that some labels have been attached, so we index it,
@@ -168,13 +152,38 @@ fetch_cards(void (*got_card)(struct card_attr *attr, struct odes *obj, struct ca
 	   */
 	  obj_set_attr(o, 'X', NULL);
 	}
-      got_card(&attr, o, &note);
+      got_card(&info, o);
     }
   ASSERT(next_label_id == ~0U);
-  src->cleanup(src);
-  bclose(fb_attrs);
-  bclose(fb_notes);
+  get_buck_cleanup(&gb);
+  bclose(fb_infos);
   bclose(fb_labels);
   mp_delete(pool);
   log(L_INFO, "Processed %d objects", fetch_id+1);
 }
+
+#ifdef TEST
+
+#include "lib/getopt.h"
+
+static struct fastbuf *out;
+static uns cnt;
+
+static void got(struct card_info *info, struct odes *o)
+{
+  bprintf(out, "### new=%x orig=%x\n", cnt++, info->orig_card);
+  obj_write(out, o, BUCKET_TYPE_PLAIN);
+  bputc(out, '\n');
+}
+
+int main(int argc, char **argv)
+{
+  log_init(argv[0]);
+  cf_getopt(argc, argv, CF_SHORT_OPTS, CF_NO_LONG_OPTS, NULL);
+  out = bfdopen_shared(1, 65536);
+  fetch_cards(got);
+  bclose(out);
+  return 0;
+}
+
+#endif
