@@ -15,7 +15,6 @@
 
 byte *fn_directory;
 byte *fn_source = "";
-byte *fn_incremental;
 byte *fn_parameters;
 byte *fn_fingerprints;
 byte *fn_labels_by_id;
@@ -23,9 +22,10 @@ byte *fn_attributes;
 byte *fn_checksums;
 byte *fn_links;
 byte *fn_urls;
+byte *fn_url_index;
 byte *fn_ref_texts;
-byte *fn_link_graph;
-byte *fn_link_graph_index;
+byte *fn_graph_obj;
+byte *fn_graph_skel;
 byte *fn_sites;
 byte *fn_labels;
 byte *fn_merges;
@@ -46,7 +46,7 @@ byte *fn_string_hash;
 byte *fn_cards;
 byte *fn_card_attrs;
 byte *fn_notes;
-byte *fn_notes_new;
+byte *fn_notes_skel;
 byte *fn_keywords;
 byte *fn_feedback_gath;
 byte *fn_card_prints;
@@ -54,13 +54,14 @@ byte *link_attrs = "";
 byte *indexer_filter_name;
 byte *fn_lex_classes;
 byte *fn_blacklist;
+byte *fn_card_info;
+byte *fn_admin_export;
 uns string_avg_bucket = 1024;
 uns indexer_fb_size = 65536;
 uns progress;
 uns progress_screen;
 uns progress_status_line;
 uns sort_delete_src;
-uns max_degree = 4096;
 uns ref_max_length = 256;
 uns ref_min_length = 1;
 uns ref_max_count = 1;
@@ -82,6 +83,24 @@ uns ic_retry_delay;
 uns ic_send_feedback;
 clist subindices;
 uns indexer_trace;
+uns num_slices = 1;
+uns indexer_threads = 1;
+uns indexer_thread_stack_size;
+
+/* Duplicate images */
+byte *fn_image_thumbnails;
+uns image_big_min_width = 64;
+uns image_big_min_height = 64;
+uns image_big_min_thumb_width = 16;
+uns image_big_min_thumb_height = 16;
+uns image_dup_max_cluster_count = ~0U;
+u64 image_dup_max_cluster_size = 1 << 30;
+/* Similar images */
+byte *fn_image_signatures_unsorted;
+byte *fn_image_signatures;
+byte *fn_image_clusters;
+uns image_sig_max_cluster_count = 1000;
+u64 image_sig_buf_size = 1000000;
 
 struct attr_set
   label_attr_set,
@@ -115,8 +134,6 @@ iconfig_commit(void *ptr UNUSED)
   if (nr > HARD_MAX_SUBINDICES)
     return "Too many subindices (see HARD_MAX_SUBINDICES)";
 
-  if (!fn_directory)
-    return "Indexer.Directory not set";
   return NULL;
 }
 
@@ -145,7 +162,6 @@ static struct cf_section iconfig = {
   CF_ITEMS {
     CF_STRING("Directory", &fn_directory),
     CF_STRING("Source", &fn_source),
-    CF_STRING("Incremental", &fn_incremental),
     CF_UNS("Trace", &indexer_trace),
     CF_STRING("Parameters", &fn_parameters),
     CF_STRING("Fingerprints", &fn_fingerprints),
@@ -154,9 +170,10 @@ static struct cf_section iconfig = {
     CF_STRING("Checksums", &fn_checksums),
     CF_STRING("Links", &fn_links),
     CF_STRING("URLList", &fn_urls),
+    CF_STRING("URLIndex", &fn_url_index),
     CF_STRING("RefTexts", &fn_ref_texts),
-    CF_STRING("LinkGraph", &fn_link_graph),
-    CF_STRING("LinkGraphIndex", &fn_link_graph_index),
+    CF_STRING("ObjectGraph", &fn_graph_obj),
+    CF_STRING("SkeletonGraph", &fn_graph_skel),
     CF_STRING("Sites", &fn_sites),
     CF_STRING("Labels", &fn_labels),
     CF_STRING("Merges", &fn_merges),
@@ -178,15 +195,17 @@ static struct cf_section iconfig = {
     CF_STRING("Cards", &fn_cards),
     CF_STRING("CardAttributes", &fn_card_attrs),
     CF_STRING("Notes", &fn_notes),
-    CF_STRING("NotesNew", &fn_notes_new),
+    CF_STRING("NotesSkel", &fn_notes_skel),
     CF_STRING("Keywords", &fn_keywords),
     CF_STRING("FeedbackGatherer", &fn_feedback_gath),
     CF_STRING("CardPrints", &fn_card_prints),
     CF_STRING("Blacklist", &fn_blacklist),
+    CF_STRING("CardInfo", &fn_card_info),
+    CF_STRING("AdminExport", &fn_admin_export),
     CF_LIST("LabelAttrs", &label_attr, &attr_set_cf),
     CF_LIST("OverrideLabelAttrs", &override_label_attr, &attr_set_cf),
     CF_LIST("OverrideBodyAttrs", &override_body_attr, &attr_set_cf),
-    CF_LIST("CardAttrs", &card_attr, &attr_set_cf),
+    CF_LIST("CardAttrs", &card_attr, &attr_set_cf_sub),
     CF_LIST("LinkAttrs", &link_attr, &attr_set_cf_sub),
     CF_LIST("RefLinkTypes", &ref_link_attr, &attr_set_cf_sub),
     CF_STRING("Filter", &indexer_filter_name),
@@ -196,7 +215,6 @@ static struct cf_section iconfig = {
     CF_UNS("ProgressScreen", &progress_screen),
     CF_UNS("ProgressStatusLine", &progress_status_line),
     CF_UNS("SortDeleteSrc", &sort_delete_src),
-    CF_UNS("MaxGraphDegree", &max_degree),
     CF_UNS("RefMaxLength", &ref_max_length),
     CF_UNS("RefMinLength", &ref_min_length),
     CF_UNS("RefMaxCount", &ref_max_count),
@@ -206,6 +224,9 @@ static struct cf_section iconfig = {
     CF_UNS("DefaultWeight", &default_weight),
     CF_UNS("RawStage2Input", &raw_stage2_input),
     CF_LIST("SubIndex", &subindices, &subindex_config),
+    CF_UNS("Slices", &num_slices),
+    CF_UNS("Threads", &indexer_threads),
+    CF_UNS("ThreadStackSize", &indexer_thread_stack_size),
     CF_END
   }
 };
@@ -223,10 +244,33 @@ static struct cf_section matcher_config = {
   }
 };
 
+#ifdef CONFIG_IMAGES
+static struct cf_section images_config = {
+  CF_ITEMS {
+    CF_STRING("ImageSignaturesUnsorted", &fn_image_signatures_unsorted),
+    CF_STRING("ImageSignatures", &fn_image_signatures),
+    CF_STRING("ImageClusters", &fn_image_clusters),
+    CF_STRING("ImageThumbnails", &fn_image_thumbnails),
+    CF_UNS("BigMinWidth", &image_big_min_width),
+    CF_UNS("BigMinHeight", &image_big_min_height),
+    CF_UNS("BigMinThumbWidth", &image_big_min_thumb_width),
+    CF_UNS("BigMinThumbHeight", &image_big_min_thumb_height),
+    CF_UNS("DupMaxClusterCount", &image_dup_max_cluster_count),
+    CF_U64("DupMaxClusterSize", &image_dup_max_cluster_size),
+    CF_UNS("SigMaxClusterCount", &image_sig_max_cluster_count),
+    CF_U64("SigBufSize", &image_sig_buf_size),
+    CF_END
+  }
+};
+#endif
+
 static void CONSTRUCTOR iconf_init(void)
 {
   cf_declare_section("Indexer", &iconfig, 0);
   cf_declare_section("Matcher", &matcher_config, 0);
+#ifdef CONFIG_IMAGES
+  cf_declare_section("ImageIndexer", &images_config, 0);
+#endif
 }
 
 byte *
@@ -234,6 +278,8 @@ index_name(byte *file)
 {
   if (!file)
     die("Missing indexer file name declarations in section Indexer");
+  if (!fn_directory)
+    die("Indexer.Directory not set");
   if (file[0] == '/')
     return file;
   return cf_printf("%s/%s", fn_directory, file);
