@@ -2,7 +2,7 @@
  *	Sherlock Indexer
  *
  *	(c) 2001--2006 Martin Mares <mj@ucw.cz>
- *	(c) 2002--2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2002--2006 Robert Spalek <robert@ucw.cz>
  */
 
 #include "sherlock/index.h"
@@ -13,45 +13,67 @@
 /* File names */
 extern byte *fn_directory;
 extern byte *fn_source;
-extern byte *fn_incremental;
-extern byte *fn_fingerprints, *fn_labels_by_id, *fn_attributes, *fn_checksums;
-extern byte *fn_links, *fn_urls, *fn_link_graph, *fn_link_graph_index, *fn_sites, *fn_labels, *fn_merges, *fn_signatures, *fn_matches;
+extern byte *fn_fingerprints, *fn_labels_by_id, *fn_attributes, *fn_checksums, *fn_card_info;
+extern byte *fn_links, *fn_urls, *fn_url_index, *fn_graph_obj, *fn_graph_skel, *fn_sites, *fn_labels, *fn_merges, *fn_signatures, *fn_matches;
 extern byte *fn_word_index, *fn_string_index, *fn_references, *fn_string_map, *fn_card_prints;
 extern byte *fn_string_hash, *fn_cards, *fn_card_attrs, *fn_parameters, *fn_ref_texts;
 extern byte *fn_lexicon, *fn_lex_raw, *fn_lex_ordered, *fn_lex_words, *fn_lex_by_freq;
-extern byte *fn_stems, *fn_stems_ordered, *fn_lex_classes, *fn_notes, *fn_notes_new, *fn_keywords, *fn_feedback_gath;
+extern byte *fn_stems, *fn_stems_ordered, *fn_lex_classes, *fn_notes, *fn_notes_skel, *fn_keywords, *fn_feedback_gath;
 extern byte *fn_blacklist;
-extern uns default_weight;
+extern byte *fn_admin_export;
 
 byte *index_name(byte *file);
 int index_name_defined(byte *file);
 #define index_bopen(file, flags) bopen(index_name(file), flags, indexer_fb_size)
 #define index_maybe_bopen(file, flags) (index_name_defined(file) ? index_bopen(file, flags) : NULL)
+#define index_atomic_open(file, clone, reclen) fbatomic_open((clone) ? NULL : index_name(file), clone, indexer_fb_size, reclen)
+#define index_maybe_atomic_open(file, clone, reclen) (index_name_defined(file) ? index_atomic_open(file, clone, reclen) : NULL)
+
+#define FN_GRAPH_INDEX "-index"
+#define FN_GRAPH_REAL "-real"
+#define FN_GRAPH_GOES "-goes"
+#define FN_GRAPH_DEG "-deg"
 
 /* Miscellaneous */
 extern struct attr_set label_attr_set, link_attr_set, ref_link_attr_set;
 extern struct attr_set override_label_attr_set, override_body_attr_set, card_attr_set;
-extern uns string_avg_bucket, indexer_fb_size, sort_delete_src, max_degree;
+extern uns string_avg_bucket, indexer_fb_size, sort_delete_src;
 extern uns progress, progress_screen, progress_status_line;
 extern uns ref_max_length, ref_min_length, ref_max_count;
 extern uns matcher_signatures, matcher_context, matcher_min_words, matcher_threshold, matcher_passes, matcher_block;
-extern uns max_num_objects, min_summed_size, frameset_to_redir;
+extern uns max_num_objects, min_summed_size, frameset_to_redir, num_slices;
 extern uns raw_stage2_input;
 extern uns indexer_trace;
+extern uns indexer_threads, indexer_thread_stack_size;
+extern uns default_weight;
 
 /* Filters */
 extern byte *indexer_filter_name;
 
-#define PROGRESS(i, msg, args...) do { if (progress && !((i) % progress)) { \
+/* Images */
+extern byte *fn_image_thumbnails;
+extern uns image_big_min_width, image_big_min_height, image_big_min_thumb_width, image_big_min_thumb_height;
+extern uns image_dup_max_cluster_count;
+extern u64 image_dup_max_cluster_size;
+extern byte *fn_image_signatures_unsorted, *fn_image_signatures, *fn_image_clusters;
+extern uns image_sig_max_cluster_count;
+extern u64 image_sig_buf_size;
+
+#define PROGRESS_REPORT(i) (progress && !((i) % progress))
+#define PROGRESS_PRINT(msg, args...) do { \
 	if (progress_status_line) setproctitle(msg, args); \
-	if (progress_screen) { printf(msg "\r", args); fflush(stdout); } } } while (0)
+	if (progress_screen) { printf(msg "\r", args); fflush(stdout); } \
+	} while (0)
+#define PROGRESS(i, msg, args...) do { if (PROGRESS_REPORT(i)) PROGRESS_PRINT(msg, args); } while (0)
+#define PROGRESS_LOCKED(i, lock, msg, args...) do { if (PROGRESS_REPORT(i)) { \
+	pthread_mutex_lock(&(lock)); PROGRESS_PRINT(msg, args); pthread_mutex_unlock(&(lock)); } } while (0)
 
 #define ITRACEN(level, msg, args...) do { if (indexer_trace >= level) log(L_DEBUG, msg,##args); } while(0)
 #define ITRACE(msg, args...) ITRACEN(1, msg,##args)
 
 /* Subindices */
 
-#define HARD_MAX_SUBINDICES 4		/* Need to update INDEX_ID_SHIFT in chewer.c when increasing this */
+#define HARD_MAX_SUBINDICES 8		/* Need to update INDEX_ID_SHIFT in chewer.c when increasing this */
 
 struct subindex {
   struct cnode n;
@@ -64,20 +86,25 @@ extern struct clist subindices;
 
 /* getbuck.c */
 
-struct mempool;
-struct buck2obj_buf;
-struct bucket_source {
+struct get_buck {
+  struct mempool *pool;			/* Filled in by the caller */
   u32 oid;				/* oid of the current bucket */
   u32 type;				/* type of the current bucket */
   struct odes *o;			/* parsed content of the current bucket */
-  int (*get_next)(struct bucket_source *src, struct mempool *mp, u32 oid); /* Get bucket with the given oid or the next one if oid == ~0U */
-  void (*cleanup)(struct bucket_source *src);
-  uns progress_current, progress_max;
+  uns progress_count;			/* object counter */
+  uns progress_current, progress_max;	/* percentage */
+
+  /* Set up by get_buck_init() */
   struct buck2obj_buf *buck_buf;
-  struct fastbuf *in_file;
+  byte *name, *aux;
+  const struct gb_ops *ops;
 };
 
-struct bucket_source *get_buck_init(void);
+extern uns gb_max_count;
+
+void get_buck_init(struct get_buck *gb);		/* Init and cleanup are synchronous, get on different contexts is thread-safe */
+void get_buck_cleanup(struct get_buck *gb);
+int get_buck_next(struct get_buck *gb, uns oid);
 
 /* Structure of files */
 
@@ -86,6 +113,11 @@ struct csum {
   u32 cardid;
 };
 
+struct link {
+  struct fingerprint fp;
+  u32 src;
+};
+
 #include "indexer/sites.h"
 
 struct card_note {
@@ -105,6 +137,14 @@ enum card_note_flag {
   CARD_NOTE_HAS_LINKS = 2,		/* even the unknown ones */
   CARD_NOTE_IS_LINKED = 4,
   CARD_NOTE_IMAGE = 8,			/* Is an image object [set by scanner] */
+  CARD_NOTE_REDIRECT = 16,		/* the object is just a redirect */
+  CARD_NOTE_AUDIO = 32,			/* Is an audio object [set by scanner] */
+};
+
+struct card_info {			/* Card information passed from stage 1 to stage 2 */
+  u32 orig_card;			/* Stage 1 ID */
+  struct card_attr attr;
+  struct card_note note;
 };
 
 static inline uns			/* id_mask works according to these id's */
@@ -118,6 +158,13 @@ get_subindexing_id(uns card_id UNUSED, struct card_note *note)
 
 typedef float rank_t;
 
+struct admin_export {
+  byte footprint[16];
+  struct card_attr attr;		/* Final attributes */
+  byte subindex;			/* Final subindex */
+  byte flags;				/* Note's flags */
+};
+
 /* Labels */
 
 struct lab {				/* Header of a label block */
@@ -136,17 +183,17 @@ struct lab {				/* Header of a label block */
 #include "lib/partmap.h"
 
 extern uns card_count;			/* Number of cards in the index */
-extern uns new_count;			/* Number of new URL's in the index */
+extern uns skel_count;			/* Number of skeletons in the index */
 
 void set_card_count(uns cc);
-void set_new_count(uns nc);
+void set_skel_count(uns sc);
 
-extern struct partmap *notes_partmap, *notes_new_partmap;
+extern struct partmap *notes_partmap, *notes_skel_partmap;
 
 void notes_part_map(uns rw);
 void notes_part_unmap(void);
-void notes_new_part_map(uns rw);
-void notes_new_part_unmap(void);
+void notes_skel_part_map(uns rw);
+void notes_skel_part_unmap(void);
 
 static inline struct card_note *
 bring_note_from(struct partmap *map, oid_t card)
@@ -161,9 +208,9 @@ bring_note(oid_t card)
 }
 
 static inline struct card_note *
-bring_new_note(oid_t card)
+bring_skel_note(oid_t card)
 {
-  return bring_note_from(notes_new_partmap, card);
+  return bring_note_from(notes_skel_partmap, card);
 }
 
 extern struct card_attr *attrs;
@@ -204,26 +251,31 @@ struct index_params;
 void params_load(struct index_params *params);
 void params_save(struct index_params *params);
 
+uns alloc_read_ary(byte *name, byte *suffix, void *Ptr, uns count, uns record);
+void write_free_ary(byte *name, byte *suffix, void *Ptr, uns count, uns record);
+
 /* fetch.c */
 
-extern uns fetch_id;
-void fetch_cards(void (*got_card)(struct card_attr *attr, struct odes *obj, struct card_note *note));
+extern uns fetch_id, fetch_num_ids;
+void fetch_cards(void (*got_card)(struct card_info *info, struct odes *obj));
 
 /* resolve.c */
 
-struct resolve_output {
-  u32 src;
-  u32 dest;
-};
-
 #define RESOLVE_SKIP_UNKNOWN	1
-#define RESOLVE_SKIP_NEW	2
+#define RESOLVE_SKIP_SKEL	2
+#define FIRST_ID_SKEL	0x80000000
+  /* Not downloaded documents are numbered from this ID (flags are independent
+   * of ETYPE_*, because they are applied to a different vertex) */
 
-sh_off_t resolve_optimize_run_length(struct fastbuf *in);
+static inline uns
+resolve_hash(struct fingerprint *fp, uns blocks)
+{
+  return fp_hash(fp) % blocks;
+}
+
+uns resolve_split_fingerprints(void);
+struct fastbuf *resolve_split_fastbuf(uns blocks, struct fastbuf **in, uns flags, uns record_size);
 struct fastbuf *resolve_fastbuf(struct fastbuf *in, uns flags, uns add_size);
-struct fastbuf *resolve_fingerprints(struct fastbuf *in, uns flags, uns mask);
-
-#define	FIRST_ID_NEW	0x20000000	/* Not downloaded documents are numbered from this ID (no interference with ETYPE_*) */
 
 /* feedback-gath.c */
 
@@ -233,20 +285,3 @@ struct feedback_gatherer {
   byte flags;				/* the same as card_note.flags */
   byte weight;
 };
-
-/* Graph edge types */
-
-#define ETYPE_NORMAL 0
-#define ETYPE_REDIRECT 0x40000000
-#define ETYPE_FRAME 0x80000000
-#define ETYPE_IMAGE 0xc0000000
-#define ETYPE_MASK 0xc0000000
-
-/* Incremental scanner */
-
-struct scanner_note {
-  byte footprint[16];
-#ifdef CONFIG_LANG
-  byte type_flags;			/* type flags, including detected language */
-#endif
-};
