@@ -14,10 +14,9 @@ Usage: indexer [-12RUacd:fi:luvwC:S:] [<source> [<dest-dir>]]
 -a	Ignore filters and accept all documents
 -c	Only clean files and exit (set -d)
 -d NUM	Delete files of level smaller than NUM (default=4)
-	9=index, 8=incremental, 7=logs, 4=useful/short debug files, 3=labels,
+	9=index, 7=logs, 4=useful/short debug files, 3=labels,
 	2=huge files, 1=really temporary files, 0=keep all versions
 -f	Force deletion of the old index
--i DIR	Incremental scanner looks into this directory for a log file
 -l	List index files between passes
 -u	Upload feedback to the gatherer
 -U	Do not upload feedback to the gatherer
@@ -32,7 +31,6 @@ EOF
 
 unset G LISTS STAGE1ONLY STAGE2ONLY WEIGHTSONLY CLEAN FORCE RESUME FEEDBACK NOWEIGHTS
 DELETE=4
-INCREMENTAL=
 VERBOSE=0
 set -e
 while getopts "12RUWacd:fi:luvwC:S:" OPT ; do
@@ -43,7 +41,7 @@ while getopts "12RUWacd:fi:luvwC:S:" OPT ; do
 			;;
 		R)	RESUME=1
 			;;
-		a)	G="$G -SIndexer.Filter="
+		a)	G="$G -SIndexer.Filter=\"\""
 			;;
 		c)	CLEAN=1
 			;;
@@ -51,8 +49,6 @@ while getopts "12RUWacd:fi:luvwC:S:" OPT ; do
 			;;
 		f)	FORCE=1
 			;;
-		i)	INCREMENTAL="-i $OPTARG"
-			;;
 		l)	LISTS=1
 			;;
 		u)	FEEDBACK=1
@@ -115,6 +111,17 @@ function delete
 	fi
 }
 
+function deleteall
+{
+	level=$1
+	if [ "$level" -lt "$DELETE" ] ; then
+		while [ -n "$2" ] ; do
+			rm -f $DIR/$2
+			shift
+		done
+	fi
+}
+
 function keep
 {
 	if [ "$DELETE" -le 0 ] ; then
@@ -170,14 +177,17 @@ delete 0 attributes.* merges.*
 delete 4 lexicon-by-freq
 if [ -n "$CLEAN" ]; then
 	delete 1 frame-graph image-graph
-	delete 2 links-by-url ref-texts url-list-translated
-	delete 2 lexicon-raw
+	delete 2 links-by-url ref-texts url-list-translated lexicon-raw
+	delete 2 graph-leaf graph-intra-deg graph-intra-real graph-intra-goes graph-intra-number
+	deleteall 2 graph-intra-[0-9]*
 	delete 3 labels-by-id labels
-	delete 4 merges fingerprints checksums signatures link-graph link-graph-index keywords url-list
-	delete 4 attributes notes notes-new
-	delete 4 rank-vector rank2-vector
+	delete 4 merges fingerprints checksums signatures keywords url-list
+	delete 4 graph-obj graph-obj-index graph-obj-deg graph-obj-real graph-obj-goes
+	delete 4 graph-skel graph-skel-index
+	delete 4 attributes notes notes-skel
+	delete 4 rank-obj rank-skel
+	delete 4 card-info
 	delete 7 large-classes matches weights lexicon-classes
-	delete 8 incremental
 	if [ -n "$SUBINDICES" ] ; then
 		for s in $SUBINDICES ; do
 			delete 2 $s/string-index $s/word-index $s/lexicon-ordered $s/lexicon-words $s/stems-ordered
@@ -223,19 +233,19 @@ fi
 #endif
 
 if [ -z "$STAGE2ONLY" ] ; then
-	bin/scanner $G $INCREMENTAL
+	bin/scanner $G
 	keep scanner attributes merges
+	[ -z "$STAGE1ONLY" -o "$FEEDBACK" == 1 ] || disconnect
 #ifndef CONFIG_BARE
 	bin/fpsort $G
 	bin/mkgraph $G
 	delete 2 links-by-url
-#endif
-#ifndef CONFIG_BARE
 	bin/backlinker $G -1
 	keep backlinker1 attributes merges
 #endif
 #ifdef CONFIG_BARE
 	bin/merger $G
+	bin/attrsort $G
 	bin/labelsort $G
 #else
 	bin/keywords $G
@@ -252,13 +262,17 @@ if [ -z "$STAGE2ONLY" ] ; then
 	bin/merger $G
 	bin/reftexts $G
 	delete 2 ref-texts
+	bin/attrsort $G
 	bin/labelsort $G
 	bin/ireport $G
 #endif
 	stats "after first stage"
 	delete 2 url-list-translated
 	delete 3 labels-by-id
-	delete 4 merges fingerprints checksums link-graph link-graph-index keywords url-list notes-new
+	delete 4 attributes notes notes-skel
+	delete 4 merges fingerprints checksums keywords url-list
+	delete 4 graph-obj graph-obj-index graph-obj-deg graph-obj-real graph-obj-goes
+	delete 4 graph-skel graph-skel-index
 	[ "$DELETE" -le 2 ] || stats "after cleanup"
 	[ -z "$STAGE1ONLY" ] || exit 0
 fi
@@ -270,9 +284,8 @@ bin/chewer $G
 disconnect
 stats "after chewing"
 delete 3 labels
-delete 4 attributes notes
+delete 4 card-info
 delete 7 large-classes matches weights lexicon-classes
-delete 8 incremental
 
 if [ -n "$SUBINDICES" ] ; then
 	for s in $SUBINDICES ; do
