@@ -2,6 +2,7 @@
  *	Sherlock Indexer -- Generate Index Reports
  *
  *	(c) 2003--2005 Martin Mares <mj@ucw.cz>
+ *	(c) 2006 Robert Spalek <robert@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
@@ -14,6 +15,7 @@
 #include "indexer/merges.h"
 #include "lang/lang.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
@@ -144,9 +146,13 @@ report_langs(void)
   byte buf[bufsize];
 
   byte *p = buf + sprintf(buf, "Languages:");
+  uns unknown_lang_cnt = 0;
   for (uns l=0; l<MAX_LANGUAGES; l++)
     if (lang_code_exists(l))
       p += sprintf(p, " %s=%d", lang_code_to_name(l), lang_cnt[l]);
+    else
+      unknown_lang_cnt += lang_cnt[l];
+  sprintf(p, " \?\?=%d", unknown_lang_cnt);
   log(L_INFO, "%s", buf);
 }
 #else
@@ -160,7 +166,7 @@ report_langs(void)
 
 struct domain {
   uns cnt;
-  char dom[1];
+  byte *dom;
 };
 
 static inline void
@@ -171,12 +177,19 @@ dom_init_data(struct domain *d)
 
 #define HASH_NODE struct domain
 #define HASH_PREFIX(x) dom_##x
-#define HASH_KEY_ENDSTRING dom
+#define HASH_KEY_STRING dom
 #define HASH_WANT_LOOKUP
 #define HASH_GIVE_INIT_DATA
 #define HASH_AUTO_POOL 4096
 #include "lib/hashtable.h"
 
+#define ASORT_PREFIX(x) domain_##x
+#define ASORT_KEY_TYPE struct domain
+#define ASORT_ELT(i) ar[i]
+#define ASORT_LT(x,y) (x.cnt > y.cnt)
+#define ASORT_EXTRA_ARGS , struct domain *ar
+#include "lib/arraysort.h"
+
 static void
 report_domains(void)
 {
@@ -204,16 +217,27 @@ report_domains(void)
 	  if (*d == '.')
 	    d++;
 	  struct domain *dom = dom_lookup(d);
+	  if (dom->dom == d)			// just created
+	    dom->dom = xstrdup(d);
 	  dom->cnt++;
 	}
     }
   bclose(b);
 
-  HASH_FOR_ALL(dom, dom)
-    {
-      log(L_INFO, "Domain %s: %d", dom->dom, dom->cnt);
-    }
+  uns count = 0;
+  HASH_FOR_ALL(dom, dom UNUSED)
+    count++;
   HASH_END_FOR;
+
+  struct domain domains[count];
+  uns i = 0;
+  HASH_FOR_ALL(dom, dom)
+    domains[i++] = *dom;
+  HASH_END_FOR;
+
+  domain_sort(count, domains);
+  for (i=0; i<count; i++)
+    log(L_INFO, "Domain %s: %d", domains[i].dom, domains[i].cnt);
   log(L_INFO, "Numeric addresses: %d", numeric);
 }
 
