@@ -21,6 +21,7 @@
 #include "lib/conf.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/chartype.h"
 #include "lib/mempool.h"
 #include "sherlock/object.h"
@@ -31,6 +32,7 @@
 #include "indexer/merges.h"
 #include "indexer/lexicon.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 
@@ -310,7 +312,7 @@ typedef struct {
   byte type, weight;
   byte source_pagerank;
   uns word;
-  word order;
+  u16 order;
 } keyword_t;
 
 keyword_t kw;
@@ -319,7 +321,7 @@ keyword_t kw;
 #include "indexer/lexhash.h"
 
 static enum word_class
-lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp)
+lm_lookup(enum word_class orig_class, u16 *uni, uns ulen, word_id_t *idp, void *user UNUSED)
 {
   struct verbum *v;
   enum word_class class = orig_class;
@@ -334,7 +336,7 @@ lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp)
 }
 
 static void
-lm_got_word(uns pos, uns cat, word_id_t w)
+lm_got_word(uns pos, uns cat, word_id_t w, void *user UNUSED)
 {
   keyword_t rec = kw;
   rec.word = w->id >> 3;
@@ -347,18 +349,18 @@ lm_got_word(uns pos, uns cat, word_id_t w)
 
 static uns cplx_pos = -1;
 static void
-lm_got_complex(uns pos, uns cat, word_id_t wroot, word_id_t wctxt, uns after)
+lm_got_complex(uns pos, uns cat, word_id_t wroot, word_id_t wctxt, uns after, void *user UNUSED)
 {
   TRACE(2, "\t-> @%d T%d <%x,%x,%d>\n", pos, cat, wroot->id >> 3, wctxt->id >> 3, after);
   if (pos != cplx_pos)		// avoid duplicate entries when a context word is registered for both sides
   {
-    lm_got_word(pos, cat, wroot);
+    lm_got_word(pos, cat, wroot, user);
     cplx_pos = pos;
   }
-  //lm_got_word(pos, cat, wctxt);
+  //lm_got_word(pos, cat, wctxt, user);
 }
 
-#define	LM_SEARCH
+#define	LM_SIMPLE
 #include "indexer/lexmap.h"
 
 static byte **lh_words;		// for friendly dumping
@@ -435,6 +437,7 @@ keyword_compare(keyword_t *a, keyword_t *b)
   }
   else
     ASSERT(0);
+  return 0;
 }
 
 static inline int
@@ -497,7 +500,9 @@ type_to_index(byte type)
   {
     case 'U': return 0;
     case 'F': return 1;
+#ifdef MT_CAT_KW_USER
     case 'C': return 2;
+#endif
     default: ASSERT(0); return 0;
   }
 }
@@ -672,8 +677,8 @@ dump_keywords(struct fastbuf *labels, byte *buf, keyword_t *rec, byte min_wt)
   if (rec->source == ~0U)	//TODO: dump nothing if all words have weight 0
     return;
   static byte types[3] = { MT_URL_KEYWD, MT_FILE,
-#ifdef	MT_CAT_KEYWD
-    MT_CAT_KEYWD
+#ifdef MT_CAT_KW_USER
+    MT_CAT_KW_USER
 #endif
   };
   uns idx = type_to_index(rec->type);
@@ -796,7 +801,7 @@ main(int argc, char **argv)
     bitarray_t is_image = xmalloc(BIT_ARRAY_BYTES(card_count));
     bit_array_zero(is_image, card_count);
     for (uns i=0; i<card_count; i++)
-      if (bring_note(i)->flags & CARD_NOTE_IMAGE)
+      if (bring_note(i)->flags & (CARD_NOTE_IMAGE | CARD_NOTE_AUDIO))
 	bit_array_set(is_image, i);
 
     merges_map(0);
@@ -830,7 +835,7 @@ main(int argc, char **argv)
       kw.source_site = 0;
       kw.target_site = 0;
       kw.source_pagerank = card_weights[kw.source];
-      lm_doc_start();
+      lm_doc_start(NULL);
       cplx_pos = -1;
       lm_map_text(buf + len);
     }
