@@ -8,10 +8,12 @@
 #include "sherlock/sherlock.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "sherlock/object.h"
 #include "indexer/indexer.h"
 #include "indexer/merges.h"
 
+#include <stdio.h>
 #include <fcntl.h>
 #include <stdlib.h>
 
@@ -105,10 +107,6 @@ lab_store_item(struct fastbuf *f, struct lab *k)
 int
 main(int argc, char **argv)
 {
-  struct fastbuf *in, *out;
-  uns redir_id;
-  uns count = 0;
-
   log_init(argv[0]);
   if (cf_getopt(argc, argv, CF_SHORT_OPTS, CF_NO_LONG_OPTS, NULL) >= 0 ||
       optind < argc)
@@ -117,21 +115,43 @@ main(int argc, char **argv)
     exit(1);
   }
 
-  out = index_bopen(fn_labels, O_RDWR | O_CREAT | O_TRUNC);
+  /* Collect information on cards */
 
-  /* Look at original by-ID labels through our merging lens */
-  log(L_INFO, "Searching for labels");
+  log(L_INFO, "Collecting card attributes");
   attrs_part_map(0);
   READ_ATTR(card_flags, flags);
   READ_ATTR(card_weights, weight);
-  merges_map(0);
-  in = index_bopen(fn_labels_by_id, O_RDONLY);
+  attrs_part_unmap();
+
+  /* Generate mapping from stage 1 ID's to those used by stage 2 */
+  struct fastbuf *infos = index_bopen(fn_card_info, O_RDONLY);
+  struct card_info info;
+  u32 *id_map = xmalloc(card_count * 4);
+  for (uns i=0; i<card_count; i++)
+    id_map[i] = ~0U;
+  uns id_count = 0;
+  while (breadb(infos, &info, sizeof(info)))
+    {
+      ASSERT(info.orig_card < card_count);
+      id_map[info.orig_card] = id_count++;
+    }
+
+  /* Look at original by-ID labels through our merging lens */
+
+  log(L_INFO, "Searching for labels");
+  struct fastbuf *in = index_bopen(fn_labels_by_id, O_RDONLY);
+  struct fastbuf *out = index_bopen(fn_labels, O_RDWR | O_CREAT | O_TRUNC);
   get_attr_set_type(BUCKET_TYPE_V33);
+  merges_map(0);
+
+  uns redir_id;
+  uns count = 0;
   while ((int) (redir_id = bgetl(in)) >= 0)
     {
       uns flags = bgetc(in);
       uns url_id = (card_flags[redir_id] & CARD_FLAG_EMPTY) ? merges[redir_id] : redir_id;
       uns merged_id = (url_id != ~0U) ? merges[url_id] : url_id;
+      uns stage2_id = (merged_id != ~0U) ? id_map[merged_id] : merged_id;
       if (merged_id != ~0U && merged_id != url_id)
 	{
 	  ASSERT(card_flags[url_id] & CARD_FLAG_DUP);
@@ -145,7 +165,7 @@ main(int argc, char **argv)
 	;
 
       struct lab l = {
-	.merged_id = merged_id,
+	.merged_id = stage2_id,
 	.url_id = url_id,
 	.redir_id = redir_id,
 	.count = btell(in) - start - 1,
@@ -153,8 +173,13 @@ main(int argc, char **argv)
       };
       if (url_id == ~0U || (card_flags[url_id] & CARD_FLAG_EMPTY))
 	{
-	  /* Empty cards will be skipped by fetch.c anyway, so don't bother with storing labels for them */
-	  ASSERT(merged_id == ~0U);
+	  /* Empty cards should have been already skipped */
+	  ASSERT(stage2_id == ~0U);
+	  continue;
+	}
+      else if (stage2_id == ~0U)
+	{
+	  /* Cards which don't pass to stage 2 don't need labels */
 	  continue;
 	}
       else if (flags & LABEL_FLAG_MERGED_ONLY)
@@ -176,6 +201,8 @@ main(int argc, char **argv)
       count++;
     }
   bclose(in);
+  merges_unmap();
+  xfree(id_map);
 
   log(L_INFO, "Extracted %u labels", count);
 
@@ -183,9 +210,7 @@ main(int argc, char **argv)
   brewind(out);
   lab_sort(out, index_name(fn_labels));
 
-  merges_unmap();
   xfree(card_weights);
   xfree(card_flags);
-  attrs_part_unmap();
   return 0;
 }
