@@ -7,11 +7,13 @@
 #include "sherlock/sherlock.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/mempool.h"
 #include "lib/unicode.h"
 #include "indexer/indexer.h"
 #include "indexer/lexicon.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 
