@@ -26,7 +26,7 @@ typedef struct verbum {
     u32 count;
     struct lentry *first_lent;
 #endif
-  } u PACKED;
+  } u;
 #if defined(LH_LEXORDER) || defined(LH_CHEWER)
   context_t context_class;
 #endif
@@ -206,7 +206,7 @@ lh_lookup_raw(byte *ww, byte *w)
 }
 
 static inline struct verbum *
-lh_lookup(word *uni, uns ulen)
+lh_lookup(u16 *uni, uns ulen)
 {
   byte ww[2*MAX_WORD_LEN+4], *w;
   uns u;
