@@ -26,7 +26,7 @@
 #include "indexer/lexhash.h"
 
 static enum word_class
-lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp, byte *orig, uns olen)
+lm_lookup(enum word_class orig_class, u16 *uni, uns ulen, word_id_t *idp, const byte *orig, uns olen, void *user UNUSED)
 {
   struct verbum *v;
   byte wbuf[MAX_WORD_LEN+1];
@@ -50,14 +50,14 @@ lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp, byte
 }
 
 static void
-lm_got_word(uns pos, uns cat, word_id_t w)
+lm_got_word(uns pos, uns cat, word_id_t w, void *user UNUSED)
 {
   printf("\t-> @%d T%d C%d I%x <%s>\n", pos, w->id & 7, cat, w->id, w->word);
   w->u.count++;
 }
 
 static void
-lm_got_complex(uns pos, uns cat, word_id_t wroot, word_id_t wctxt, uns after)
+lm_got_complex(uns pos, uns cat, word_id_t wroot, word_id_t wctxt, uns after, void *user UNUSED)
 {
   printf("\t-> @%d T%d <%x,%x,%d>\n", pos, cat, wroot->id, wctxt->id, after);
 }
@@ -109,7 +109,7 @@ main(int argc, char **argv)
 
   lm_init();
   lh_init();
-  lm_doc_start();
+  lm_doc_start(NULL);
   while (fgets(line, sizeof(line), stdin))
     {
       byte *e = strchr(line, '\n');
