@@ -1,23 +1,35 @@
 /*
  *	Sherlock Indexer -- Lexical Mapping
  *
- *	(c) 2001--2005 Martin Mares <mj@ucw.cz>
+ *	(c) 2001--2006 Martin Mares <mj@ucw.cz>
  *	(c) 2001--2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2005 Tomas Valla <tom@ucw.cz>
  *
- *	You define:
- *	   - word_id_t: a type representing a single word
- *	     If word_id_t is a compound type, you have to define WORD_ID_NULL
- *	     and WORD_ID_DEFINED_P(x).
+ *	Feature selection switches:
+ *	   - LM_SIMPLE if the input is a string without Sherlock tags and ligatures
+ *	   - LM_SEARCH_WILDCARDS to enable a special mode for sherlockd wildcards
+ *	   - LM_MAP_URL to include functions for mapping of URL's as well
  *	   - LM_TRACK_TEXT if you want lexmap to keep track of position of
  *	     the words in the original text. Expects the text to contain
  *	     no URL brackets.
- *	   - LM_SEARCH if you are search/words.c
- *	   - LM_CARDS if you are search/cards.c
+ *	   - LM_MULTI if you want to run multiple instances of the lexmapper
+ *	     in parallel. In this case, you should call lmap_* instead of lm_*
+ *	     and always pass a pointer to struct lm_state as the first argument.
+ *
+ *	You should supply:
+ *	   - word_id_t: a type representing a single word
+ *	     If word_id_t is a compound type, you have to define WORD_ID_NULL
+ *	     and WORD_ID_DEFINED_P(x).
+ *	and optionally:
+ *	   - LM_SCAN_CHAR(u) to read one character from input to u, return 0 on eof, 1 otherwise
+ *	   - LM_MAP_ARGS as the arguments of lm_map_text() which LM_SCAN_CHAR will see and could use
+ *
  *	You call:
  *	   - lm_init() before doing anything else
  *	   - lm_doc_start() at the start of each document
- *	   - lm_map_text() for each piece of document text
+ *	   - lm_map_text() to process data returned by LM_SCAN_CHAR
  *	   - lm_map_break() if you want to insert an explicit sentence break
+ *
  *	We call:
  *	   - lm_lookup() to look up and categorize a word
  *	   - lm_got_word() for each word found together with its position
@@ -31,12 +43,30 @@
 
 #include <string.h>
 
-/* Internal state variables */
-static uns lm_current_cat;
-static uns lm_pos;
-static uns lm_garb_cnt;
-static word_id_t lm_context_base, lm_last_word;
-static uns lm_context_cat;
+/* Internal state and various macros */
+
+typedef struct lm_state {
+  uns current_cat;
+  uns pos;
+  uns garb_cnt;
+  word_id_t context_base, last_word;
+  uns context_cat;
+  void *user_data;
+} lm_state;
+
+#ifndef LM_SCAN_CHAR
+#ifdef LM_SIMPLE
+#define LM_SCAN_CHAR(u) (text = utf8_get(text, &u), u)
+#define LM_MAP_ARGS const byte *text
+#else
+#define LM_SCAN_CHAR(u) ((text < stop) ? (text = get_tagged_char(text, &u), 1) : 0)
+#define LM_MAP_ARGS const byte *text, const byte *stop
+#endif
+#endif
+
+#ifndef LM_MAP_ARGS
+#define LM_MAP_ARGS void
+#endif
 
 #ifdef LM_TRACK_TEXT
 #define TRACK_TEXT(e) e
@@ -50,37 +80,38 @@ static uns lm_context_cat;
 #endif
 
 static inline void
-lm_doc_start(void)
+lmap_doc_start(lm_state *lm, void *user_data)
 {
-  lm_garb_cnt = 0;
-  lm_last_word = lm_context_base = WORD_ID_NULL;
-  lm_current_cat = WT_TEXT;
-  lm_pos = 0;
+  lm->garb_cnt = 0;
+  lm->last_word = lm->context_base = WORD_ID_NULL;
+  lm->current_cat = WT_TEXT;
+  lm->pos = 0;
+  lm->user_data = user_data;
 }
 
 static inline void
-lm_set_pos(uns pos)
+lmap_set_pos(lm_state *lm, uns pos)
 {
   /* Beware that this does not restore other state varibles.  You should only
    * call this procedure after lm_doc_start().  */
-  lm_pos = pos;
+  lm->pos = pos;
 }
 
 static inline uns
-lm_get_pos(void)
+lmap_get_pos(lm_state *lm)
 {
-  return lm_pos;
+  return lm->pos;
 }
 
-static inline void
-lm_map_word(word *uni, uns ulen, enum word_class class, byte *ostart UNUSED, uns olen UNUSED)
+static void
+lmap_map_word(lm_state *lm, u16 *uni, uns ulen, enum word_class class, const byte *ostart UNUSED, const uns olen UNUSED)
 {
   word_id_t thisw = WORD_ID_NULL;
 
 #ifdef LM_TRACK_TEXT
-  class = lm_lookup(class, uni, ulen, &thisw, ostart, olen);
+  class = lm_lookup(class, uni, ulen, &thisw, ostart, olen, lm->user_data);
 #else
-  class = lm_lookup(class, uni, ulen, &thisw);
+  class = lm_lookup(class, uni, ulen, &thisw, lm->user_data);
 #endif
 #ifdef LOCAL_DEBUG
   {
@@ -89,7 +120,7 @@ lm_map_word(word *uni, uns ulen, enum word_class class, byte *ostart UNUSED, uns
     for (i=0; i<ulen; i++)
       PUT_UTF8(p, uni[i]);
     *p = 0;
-    DBG("step: @%d <%s> class=%d cat=%d", lm_pos, w, class, lm_current_cat);
+    DBG("step: @%d <%s> class=%d cat=%d", lm->pos, w, class, lm->current_cat);
   }
 #endif
   /*
@@ -102,44 +133,44 @@ lm_map_word(word *uni, uns ulen, enum word_class class, byte *ostart UNUSED, uns
     case WC_COMPLEX:
       break;
     case WC_NORMAL:
-      if (lm_garb_cnt)
+      if (lm->garb_cnt)
 	{
-	  if (lm_pos)
-	    lm_pos += MIN(lm_garb_cnt, lex_max_gap);
-	  lm_garb_cnt = 0;
+	  if (lm->pos)
+	    lm->pos += MIN(lm->garb_cnt, lex_max_gap);
+	  lm->garb_cnt = 0;
 	}
-      else if (WORD_ID_DEFINED_P(lm_context_base))
-	lm_got_complex(lm_pos-1, lm_context_cat, lm_context_base, thisw, 1);
-      lm_context_base = WORD_ID_NULL;
-      lm_got_word(lm_pos, lm_current_cat, thisw);
-      lm_last_word = thisw;
-      lm_pos++;
+      else if (WORD_ID_DEFINED_P(lm->context_base))
+	lm_got_complex(lm->pos-1, lm->context_cat, lm->context_base, thisw, 1, lm->user_data);
+      lm->context_base = WORD_ID_NULL;
+      lm_got_word(lm->pos, lm->current_cat, thisw, lm->user_data);
+      lm->last_word = thisw;
+      lm->pos++;
       break;
     case WC_BREAK:
-      lm_garb_cnt += lex_max_gap;
+      lm->garb_cnt += lex_max_gap;
       /* Fall thru */
     case WC_GARBAGE:
-      lm_garb_cnt++;
-      lm_context_base = lm_last_word = WORD_ID_NULL;
+      lm->garb_cnt++;
+      lm->context_base = lm->last_word = WORD_ID_NULL;
       break;
     case WC_CONTEXT:
-      if (lm_garb_cnt)
+      if (lm->garb_cnt)
 	{
 	  /* because of "... gap gap gap complex [complex] word" */
-	  if (lm_pos)
-	    lm_pos += MIN(lm_garb_cnt, lex_max_gap);
-	  lm_garb_cnt = 0;
+	  if (lm->pos)
+	    lm->pos += MIN(lm->garb_cnt, lex_max_gap);
+	  lm->garb_cnt = 0;
 	}
       else
         {
-	  if (WORD_ID_DEFINED_P(lm_context_base))
-	    lm_got_complex(lm_pos-1, lm_current_cat, lm_context_base, thisw, 1);
-	  if (WORD_ID_DEFINED_P(lm_last_word))
-	    lm_got_complex(lm_pos, lm_current_cat, thisw, lm_last_word, 0);
+	  if (WORD_ID_DEFINED_P(lm->context_base))
+	    lm_got_complex(lm->pos-1, lm->current_cat, lm->context_base, thisw, 1, lm->user_data);
+	  if (WORD_ID_DEFINED_P(lm->last_word))
+	    lm_got_complex(lm->pos, lm->current_cat, thisw, lm->last_word, 0, lm->user_data);
 	}
-      lm_context_base = lm_last_word = thisw;
-      lm_context_cat = lm_current_cat;
-      lm_pos++;
+      lm->context_base = lm->last_word = thisw;
+      lm->context_cat = lm->current_cat;
+      lm->pos++;
       break;
     default:
       ASSERT(0);
@@ -147,9 +178,9 @@ lm_map_word(word *uni, uns ulen, enum word_class class, byte *ostart UNUSED, uns
 }
 
 static inline void
-lm_map_break(void)
+lmap_map_break(lm_state *lm)
 {
-  lm_map_word(NULL, 0, WC_BREAK, NULL, 0);
+  lmap_map_word(lm, NULL, 0, WC_BREAK, NULL, 0);
 }
 
 #define	ANALYSED_CHARS 0x91
@@ -181,8 +212,8 @@ lm_init(void)
     }
 }
 
-static inline int
-lm_sequence_valid(word *uni, uns len)
+static ALWAYS_INLINE int
+lmap_sequence_valid(u16 *uni, uns len)
 {
   byte cnts[cc_end];
   uns c, act_cat, i;
@@ -224,8 +255,8 @@ lm_sequence_valid(word *uni, uns len)
   return 0;
 }
 
-static inline void
-lm_map_sequence(word *uni, uns len, byte **cpos UNUSED)
+static void
+lmap_map_sequence(lm_state *lm, u16 *uni, uns len, const byte **cpos UNUSED)
 {
   uns start, u, flags, i, wlen;
   enum word_class class;
@@ -233,11 +264,11 @@ lm_map_sequence(word *uni, uns len, byte **cpos UNUSED)
   if (!len)
      return;
 
-#ifndef LM_SEARCH
+#ifndef LM_SIMPLE
   uni[len] = 0;
-  if (!lm_sequence_valid(uni, len))
+  if (!lmap_sequence_valid(uni, len))
     {
-      lm_map_word(NULL, 0, WC_GARBAGE, NULL, 0);
+      lmap_map_word(lm, NULL, 0, WC_GARBAGE, NULL, 0);
       return;
     }
 #endif
@@ -248,7 +279,7 @@ lm_map_sequence(word *uni, uns len, byte **cpos UNUSED)
   for (i=0; u = uni[i]; i++)
     {
       uns cat = alpha_class[u];
-#if defined(LM_SEARCH) && defined(LM_SEARCH_WILDCARDS)
+#ifdef LM_SEARCH_WILDCARDS
       if ((u == '*' && wildcard_asterisks) || (u == '?' && wildcard_qmarks))
 	cat = AC_ALPHA;
 #endif
@@ -273,40 +304,39 @@ lm_map_sequence(word *uni, uns len, byte **cpos UNUSED)
 	      else
 		class = WC_NORMAL;
 #ifdef LM_TRACK_TEXT
-	      lm_map_word(uni+start, i-start, class, cpos[start], cpos[i]-cpos[start]);
+	      lmap_map_word(lm, uni+start, i-start, class, cpos[start], cpos[i]-cpos[start]);
 #else
-	      lm_map_word(uni+start, i-start, class, NULL, 0);
+	      lmap_map_word(lm, uni+start, i-start, class, NULL, 0);
 #endif
 	    }
 	  if (cat == AC_SINGLETON)
 #ifdef LM_TRACK_TEXT
-	    lm_map_word(uni+i, 1, WC_NORMAL, cpos[i], cpos[i+1]-cpos[i]);
+	    lmap_map_word(lm, uni+i, 1, WC_NORMAL, cpos[i], cpos[i+1]-cpos[i]);
 #else
-	    lm_map_word(uni+i, 1, WC_NORMAL, NULL, 0);
+	    lmap_map_word(lm, uni+i, 1, WC_NORMAL, NULL, 0);
 #endif
 	  start = i+1;
 	  flags = 0;
 	}
     }
   if (alpha_class[uni[len-1]] == AC_BREAK)
-    lm_map_break();
+    lmap_map_break(lm);
 }
 
-#ifdef LM_SEARCH
+#ifdef LM_SIMPLE
 
 static int
-lm_map_text(byte *text)
+lmap_map_text(lm_state *lm, LM_MAP_ARGS)
 {
-  word uni[MAX_WORD_LEN+1];
+  u16 uni[MAX_WORD_LEN+1];
   uns u, l;
 
   l = 0;
-  while (*text)
+  while (LM_SCAN_CHAR(u))
     {
-      GET_UTF8(text, u);
       if (alpha_class[u] == AC_SPACE)
 	{
-	  lm_map_sequence(uni, l, NULL);
+	  lmap_map_sequence(lm, uni, l, NULL);
 	  l = 0;
 	}
       else if (l < MAX_WORD_LEN)
@@ -314,32 +344,33 @@ lm_map_text(byte *text)
       else
 	return 0;
     }
-  lm_map_sequence(uni, l, NULL);
+  lmap_map_sequence(lm, uni, l, NULL);
   return 1;
 }
 
 #else
 
 static void
-lm_map_text(byte *text, byte *stop)
+lmap_map_text(lm_state *lm, LM_MAP_ARGS)
 {
-  word uni[MAX_WORD_LEN+1];
-  byte *cpos[MAX_WORD_LEN+1];
-  const word *lig;
-  uns u, l;
+  u16 uni[MAX_WORD_LEN+1];
+  const byte *cpos[MAX_WORD_LEN+1];
+  const u16 *lig;
+  uns u, l, r;
 
   l = 0;
-  while (text < stop)
+  for (;;)
     {
       TRACK_TEXT(cpos[l] = text);
-      GET_TAGGED_CHAR(text, u);
+      if (!LM_SCAN_CHAR(u))
+	break;
     restart:
       if (u < 0x80000000)
 	{
 	  switch ((enum alphabet_class) alpha_class[u])
 	    {
 	    case AC_SPACE:
-	      lm_map_sequence(uni, l, cpos);
+	      lmap_map_sequence(lm, uni, l, cpos);
 	      l = 0;
 	      break;
 	    case AC_ALPHA:
@@ -352,22 +383,23 @@ lm_map_text(byte *text, byte *stop)
 	      else
 		{
 		over:
-		  lm_map_word(NULL, 0, WC_GARBAGE, NULL, 0);
+		  lmap_map_word(lm, NULL, 0, WC_GARBAGE, NULL, 0);
 		  l = 0;
-		  while (text < stop &&
-			 ((u < 0x80000000 && alpha_class[u] != AC_SPACE && alpha_class[u] != AC_SINGLETON) ||
-			  (u >= 0x80010000)))
+		  r = 1;
+		  while ( ((u < 0x80000000 && alpha_class[u] != AC_SPACE && alpha_class[u] != AC_SINGLETON) || (u >= 0x80010000)) && r )
 		    {
 		      TRACK_TEXT(cpos[0] = text);
-		      GET_TAGGED_CHAR(text, u);
+		      r = LM_SCAN_CHAR(u);
 		    }
-		  if (text < stop)
+		  if (r)
 		    goto restart;
+		  else
+		    return;
 		}
 	      break;
 	    case AC_LIGATURE:
 	      lig = Uexpand_lig(u);
-	      TRACK_TEXT(byte *lig_start = cpos[l]);
+	      TRACK_TEXT(const byte *lig_start = cpos[l]);
 	      while (*lig)
 		{
 		  if (l >= MAX_WORD_LEN)
@@ -381,38 +413,39 @@ lm_map_text(byte *text, byte *stop)
 	}
       else if (u < 0x80010000)		/* Word type tag, breaks words */
 	{
-	  lm_map_sequence(uni, l, cpos);
+	  lmap_map_sequence(lm, uni, l, cpos);
 	  l = 0;
-	  lm_current_cat = u & 0x0f;
+	  lm->current_cat = u & 0x0f;
 	  if (u & 0x10)
-	    lm_map_break();
+	    lmap_map_break(lm);
 	}
       /* else it's a bracket which we ignore */
     }
   TRACK_TEXT(cpos[l] = text);
-  lm_map_sequence(uni, l, cpos);
+  lmap_map_sequence(lm, uni, l, cpos);
 }
 
 #endif
 
-#ifdef LM_CARDS
+#ifdef LM_MAP_URL
 
 static int
-lm_map_url(byte *text, byte *stop)
+lmap_map_url(lm_state *lm, LM_MAP_ARGS)
 {
-  word uni[MAX_WORD_LEN+1];
-  byte *cpos[MAX_WORD_LEN+1];
+  u16 uni[MAX_WORD_LEN+1];
+  const byte *cpos[MAX_WORD_LEN+1];
   uns u, l;
 
   l = 0;
-  while (text < stop)
+  while (1)
     {
       TRACK_TEXT(cpos[l] = text);
-      GET_UTF8(text, u);
+      if (!LM_SCAN_CHAR(u))
+	break;
       uns c = alpha_class[u];
       if (c == AC_SPACE || u=='/' || u=='?' || u=='&' || u=='.')
 	{
-	  lm_map_sequence(uni, l, cpos);
+	  lmap_map_sequence(lm, uni, l, cpos);
 	  l = 0;
 	}
       else if (l < MAX_WORD_LEN)
@@ -421,8 +454,17 @@ lm_map_url(byte *text, byte *stop)
 	return 0;
     }
   TRACK_TEXT(cpos[l] = text);
-  lm_map_sequence(uni, l, cpos);
+  lmap_map_sequence(lm, uni, l, cpos);
   return 1;
 }
 
 #endif
+
+#ifndef LM_MULTI
+static struct lm_state lm_default_state;
+#define lm_doc_start(x) lmap_doc_start(&lm_default_state, x)
+#define lm_get_pos() lmap_get_pos(&lm_default_state)
+#define lm_set_pos(p) lmap_set_pos(&lm_default_state, p)
+#define lm_map_text(x...) lmap_map_text(&lm_default_state, x)
+#define lm_map_url(x...) lmap_map_url(&lm_default_state, x)
+#endif
