@@ -14,6 +14,7 @@
 #include "sherlock/sherlock.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/hashfunc.h"
 #include "lib/math.h"
 #include "lib/mempool.h"
@@ -22,6 +23,7 @@
 #include "indexer/indexer.h"
 #include "indexer/lexicon.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 
