@@ -11,6 +11,7 @@
 #include "sherlock/sherlock.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/unaligned.h"
 #include "lib/mempool.h"
 #include "lib/unicode.h"
@@ -19,6 +20,7 @@
 #include "indexer/indexer.h"
 #include "indexer/lexicon.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 
