@@ -44,7 +44,7 @@ gcd32(u32 a, u32 b)
 }
 
 #define	BIG_RANDOM	((random() & 0x7fffffff) | 0x80000000)
-struct hash_permuter *
+static struct hash_permuter *
 permuter_new(void)
 {
 	struct hash_permuter *pert;
@@ -111,15 +111,6 @@ cyclic_hash_update(u32 hash, uns shift, uns delete_shift, u32 added, u32 deleted
 	return ROL(hash, shift) ^ added ^ ROL(deleted, delete_shift);
 }
 
-static uns sign_delete_shift;
-static u32 sign_cyclic_hash;
-static u32 *sign_last_word_hashes;
-static uns sign_context;
-static uns sign_deleting;
-static uns sign_total_words;
-static u32 *sign_min;
-static struct hash_permuter *sign_permuter;
-
 /*
  * Call-backs called by the cutter-into-words indexer/lexmap.h
  */
@@ -127,47 +118,48 @@ static struct hash_permuter *sign_permuter;
 typedef int word_id_t;
 
 static enum word_class
-lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp UNUSED)
+lm_lookup(enum word_class orig_class, u16 *uni, uns ulen, word_id_t *idp UNUSED, struct matcher_context *ctxt)
 {
 	u32 new_hash;
 
 	if (orig_class != WC_NORMAL)
 		return orig_class;
-	new_hash = hash_block((void *) uni, ulen * sizeof(word));
-		/* The SHIFT_BITS==7, hence it is effienct for UCS-2 strings too.  */
-	if (sign_total_words >= sign_context)
+	new_hash = hash_block((void *) uni, ulen * sizeof(u16));
+		/* The SHIFT_BITS==7, hence it is efficient for UCS-2 strings too.  */
+	if (ctxt->sign_total_words >= ctxt->sign_context)
 	{
-		sign_cyclic_hash = cyclic_hash_update(sign_cyclic_hash,
-			CYCLIC_SHIFT, sign_delete_shift,
-			new_hash, sign_last_word_hashes[sign_deleting]);
-		sign_last_word_hashes[sign_deleting++] = new_hash;
-		if (sign_deleting >= sign_context)
-			sign_deleting = 0;
+		ctxt->sign_cyclic_hash = cyclic_hash_update(ctxt->sign_cyclic_hash,
+			CYCLIC_SHIFT, ctxt->sign_delete_shift,
+			new_hash, ctxt->sign_last_word_hashes[ctxt->sign_deleting]);
+		ctxt->sign_last_word_hashes[ctxt->sign_deleting++] = new_hash;
+		if (ctxt->sign_deleting >= ctxt->sign_context)
+			ctxt->sign_deleting = 0;
 	}
 	else
 	{
-		sign_cyclic_hash = cyclic_hash_update(sign_cyclic_hash,
+		ctxt->sign_cyclic_hash = cyclic_hash_update(ctxt->sign_cyclic_hash,
 			CYCLIC_SHIFT, 0, new_hash, 0);
-		sign_last_word_hashes[sign_total_words] = new_hash;
+		ctxt->sign_last_word_hashes[ctxt->sign_total_words] = new_hash;
 	}
-	sign_total_words++;
-	update_minima(sign_min, sign_cyclic_hash, sign_permuter);
+	ctxt->sign_total_words++;
+	update_minima(ctxt->sign_min, ctxt->sign_cyclic_hash, ctxt->perm);
 	return WC_IGNORED;
 }
 
 static inline void
-lm_got_word(uns pos UNUSED, uns cat UNUSED, word_id_t w UNUSED)
+lm_got_word(uns pos UNUSED, uns cat UNUSED, word_id_t w UNUSED, struct matcher_context *ctxt UNUSED)
 {
 	/* We do not need this call-back.  */
 }
 
 static inline void
-lm_got_complex(uns pos UNUSED, uns cat UNUSED, word_id_t root UNUSED, word_id_t w UNUSED, uns dir UNUSED)
+lm_got_complex(uns pos UNUSED, uns cat UNUSED, word_id_t root UNUSED, word_id_t w UNUSED, uns dir UNUSED, struct matcher_context *ctxt UNUSED)
 {
 	/* We do not need this call-back.  */
 }
 
 /* Import a simplified version of the cutter-into-words.  */
+#define LM_MULTI
 #include "indexer/lexmap.h"
 
 /*
@@ -223,31 +215,42 @@ lm_got_complex(uns pos UNUSED, uns cat UNUSED, word_id_t root UNUSED, word_id_t
  * re-entrant.
  */
 uns
-matcher_compute_minima(u32 *min, struct hash_permuter *perm, struct oattr *oa)
+matcher_compute_minima(u32 *min, struct matcher_context *ctxt, struct oattr *oa)
 {
-	sign_cyclic_hash = 0;
-	if (sign_context != matcher_context)
+	ctxt->sign_cyclic_hash = 0;
+	if (ctxt->sign_context != matcher_context)
 	{
-		sign_context = matcher_context;
-		sign_last_word_hashes = xrealloc(sign_last_word_hashes, sign_context * sizeof(u32));
-		sign_delete_shift = CYCLIC_DELETE_SHIFT(matcher_context, CYCLIC_SHIFT);
+		ctxt->sign_context = matcher_context;
+		ctxt->sign_last_word_hashes = xrealloc(ctxt->sign_last_word_hashes, ctxt->sign_context * sizeof(u32));
+		ctxt->sign_delete_shift = CYCLIC_DELETE_SHIFT(matcher_context, CYCLIC_SHIFT);
 	}
-	sign_deleting = sign_total_words = 0;
-	sign_min = min;
-	sign_permuter = perm;
+	ctxt->sign_deleting = ctxt->sign_total_words = 0;
+	ctxt->sign_min = min;
 
 	for (uns i=0; i<matcher_signatures; i += 2)
 	{
-	  min[i] = ~0U;
-	  min[i+1] = 0;
+		min[i] = ~0U;
+		min[i+1] = 0;
 	}
-	lm_doc_start();
+	struct lm_state lm;
+	lmap_doc_start(&lm, ctxt);
 	while (oa)
 	{
-		lm_map_text(oa->val, oa->val + str_len(oa->val));
+		lmap_map_text(&lm, oa->val, oa->val + str_len(oa->val));
 		oa = oa->same;
 	}
 	for (uns i=1; i<matcher_signatures; i += 2)
 	  min[i] ^= ~0U;
-	return sign_total_words;
+	return ctxt->sign_total_words;
+}
+
+struct matcher_context *
+matcher_new(struct matcher_context *clone)
+{
+	struct matcher_context *ctxt = xmalloc_zero(sizeof(*ctxt));
+	if (clone)
+		ctxt->perm = clone->perm;
+	else
+		ctxt->perm = permuter_new();
+	return ctxt;
 }
