@@ -12,6 +12,7 @@
 #include "indexer/indexer.h"
 #include "indexer/merges.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 
@@ -44,7 +45,7 @@ main(int argc, char **argv)
   while (breadb(in, &this, sizeof(this)))
     {
       sum_cnt++;
-      if (this.cardid >= FIRST_ID_NEW)
+      if (this.cardid >= FIRST_ID_SKEL)
       {
 	not_downloaded++;
 	continue;
