@@ -23,6 +23,7 @@
 #include "indexer/indexer.h"
 #include "indexer/merges.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
@@ -85,7 +86,7 @@ merge_cards(struct card_attr *attrs, uns card_count)
 	  }
 #ifdef CUSTOM_MERGE
 	if (merges[i] != ~0U)
-	  CUSTOM_MERGE(&attrs[merges[i]], &attrs[i])
+	  CUSTOM_MERGE(&attrs[merges[i]], &attrs[i]);
 #endif
       }
 
