@@ -7,12 +7,14 @@
 #include "sherlock/sherlock.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "sherlock/object.h"
 #include "lib/bitarray.h"
 #include "indexer/indexer.h"
 #include "indexer/merges.h"
 #include "indexer/matcher.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
 #include <alloca.h>
@@ -263,10 +265,10 @@ main(int argc, char **argv)
 			bit_array_set(untouchable, i);
 	attrs_part_unmap();
 
-	/* Images are untouchable, too */
+	/* Images and audio files are untouchable, too */
 	notes_part_map(0);
 	for (uns i=0; i<card_count; i++)
-		if (bring_note(i)->flags & CARD_NOTE_IMAGE)
+		if (bring_note(i)->flags & (CARD_NOTE_IMAGE | CARD_NOTE_AUDIO))
 			bit_array_set(untouchable, i);
 	notes_part_unmap();
 
