@@ -9,6 +9,7 @@
 #include "lib/getopt.h"
 #include "lib/unaligned.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/hashfunc.h"
 #include "sherlock/object.h"
 #include "sherlock/tagged-text.h"
@@ -27,7 +28,7 @@
 #include "indexer/lexhash.h"
 
 static enum word_class
-lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp)
+lm_lookup(enum word_class orig_class, u16 *uni, uns ulen, word_id_t *idp, void *user UNUSED)
 {
   struct verbum *v;
 
@@ -40,12 +41,12 @@ lm_lookup(enum word_class orig_class, word *uni, uns ulen, word_id_t *idp)
 }
 
 static inline void
-lm_got_word(uns pos UNUSED, uns cat UNUSED, word_id_t w UNUSED)
+lm_got_word(uns pos UNUSED, uns cat UNUSED, word_id_t w UNUSED, void *user UNUSED)
 {
 }
 
 static inline void
-lm_got_complex(uns pos UNUSED, uns cat UNUSED, word_id_t wroot UNUSED, word_id_t wcont UNUSED, uns dir UNUSED)
+lm_got_complex(uns pos UNUSED, uns cat UNUSED, word_id_t wroot UNUSED, word_id_t wcont UNUSED, uns dir UNUSED, void *user UNUSED)
 {
 }
 
@@ -54,7 +55,7 @@ lm_got_complex(uns pos UNUSED, uns cat UNUSED, word_id_t wroot UNUSED, word_id_t
 static inline void
 mklex_meta(byte *x)
 {
-  lm_doc_start();
+  lm_doc_start(NULL);
   if (*x >= '0' && *x <= '3')
     x++;
   lm_map_text(x, x + str_len(x));
@@ -85,12 +86,18 @@ mklex_reftexts(struct odes *o)
     }
 }
 
+#ifdef CUSTOM_MKLEX
 static void
-mklex_card(struct card_attr *attr UNUSED, struct odes *o, struct card_note *note UNUSED)
+do_lm_map_text(byte *start, byte *end)
 {
-  static uns ccnt;
+  lm_map_text(start, end);
+}
+#endif
 
-  lm_doc_start();
+static void
+mklex_card(struct card_info *info UNUSED, struct odes *o)
+{
+  lm_doc_start(NULL);
   for (struct oattr *a=obj_find_attr(o, 'X'); a; a=a->same)
     lm_map_text(a->val, a->val + str_len(a->val));
   for (struct oattr *u = obj_find_attr(o, 'U' + OBJ_ATTR_SON); u; u=u->same)
@@ -107,11 +114,14 @@ mklex_card(struct card_attr *attr UNUSED, struct odes *o, struct card_note *note
    */
   for (struct oattr *x = obj_find_attr(o, 'x' + OBJ_ATTR_SON); x; x=x->same)
     mklex_metas(x->son);
+#ifdef CUSTOM_MKLEX
+  CUSTOM_MKLEX(o, do_lm_map_text);
+#endif
   mklex_metas(o);
   mklex_reftexts(o);
-  ccnt++;
-  PROGRESS(ccnt, "mklex: %d cards, %d words in %d buckets",
-	   ccnt, lh_hash_count, lh_hash_size);
+  PROGRESS(fetch_id, "mklex: %d cards (%d%%), %d words in %d buckets",
+	   fetch_id, (int)((float)fetch_id/fetch_num_ids*100),
+	   lh_hash_count, lh_hash_size);
 }
 
 static inline void
