@@ -17,10 +17,20 @@ struct index_params {
   u32 objects_in;			/* Number of objects on the input of the indexer */
   u32 sites;				/* Number of unique sites */
   uns srand;				/* Random number generator setting for signatures */
-  sh_time_t last_download;		/* Time of the newest document for the incremental scanner */
   u32 database_version;			/* Database version of this index */
   u16 type_mask;			/* Split indices: data type mask */
   u16 id_mask;				/* Split indices: document ID mask */
+  u32 num_slices;			/* Number of slices per reference chain */
+  u32 cards_out;			/* Number of cards generated */
 };
 
+static inline void
+get_slice_breaks(struct index_params *par, uns *slice_breaks)
+{
+  uns i;
+  for (i=0; i<par->num_slices; i++)
+    slice_breaks[i] = (i+1)*par->cards_out/par->num_slices;
+  slice_breaks[i] = ~0U;
+}
+
 #endif
