@@ -10,9 +10,11 @@
 #include "lib/fastbuf.h"
 #include "lib/unicode.h"
 #include "lib/ff-utf8.h"
+#include "lib/ff-binary.h"
 #include "indexer/indexer.h"
 #include "indexer/params.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 
 struct ssk {
@@ -49,6 +51,7 @@ ss_copy_data(struct fastbuf *src, struct fastbuf *dest, struct ssk *x)
 }
 
 #include "indexer/refmerger.h"
+#include "indexer/refslicer.h"
 
 static void
 ss_merge_data(struct fastbuf *src1, struct fastbuf *src2, struct fastbuf *dest, struct ssk *x, struct ssk *y UNUSED)
@@ -68,17 +71,14 @@ split(struct fastbuf *sorted)
 
   smap = index_bopen(fn_string_map, O_WRONLY | O_CREAT | O_TRUNC);
   refs = index_bopen(fn_references, O_WRONLY | O_CREAT | O_APPEND);
+  slice_init();
 
   bseek(refs, 0, SEEK_END);
   while (ss_fetch_key(sorted, &key))
     {
-      uns rsize;
       bwrite(smap, &key.fp, sizeof(key.fp));
       bputo(smap, btell(refs));
-      rsize = bgetl(sorted);
-      while (rsize)
-	rsize -= 4 + bbcopy_chain(sorted, refs, bgetl(sorted));
-      bputl(refs, 0);
+      slice_chain(sorted, refs);
       entries++;
       if (unlikely(!entries))
 	die("Too many strings indexed. Try decreasing Chewer.StringMax as a work-around.");
@@ -87,6 +87,7 @@ split(struct fastbuf *sorted)
   bwrite(smap, &key.fp, sizeof(key.fp));
   bputo(smap, btell(refs));
 
+  slice_cleanup();
   bclose(smap);
   bclose(refs);
   log(L_INFO, "Indexed %d strings", entries);
