@@ -12,10 +12,12 @@
 #include "lib/mempool.h"
 #include "lib/unicode.h"
 #include "lib/ff-utf8.h"
+#include "lib/ff-binary.h"
 #include "indexer/indexer.h"
 #include "indexer/lexicon.h"
 #include "indexer/params.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 
 #define SORT_KEY u32
@@ -50,6 +52,7 @@ ws_copy_data(struct fastbuf *src, struct fastbuf *dest, u32 *x)
 }
 
 #include "indexer/refmerger.h"
+#include "indexer/refslicer.h"
 
 static void
 ws_merge_data(struct fastbuf *src1, struct fastbuf *src2, struct fastbuf *dest, u32 *x, u32 *y UNUSED)
@@ -76,6 +79,7 @@ split(struct fastbuf *sorted)
   wid_lex = 1;
   wcount = bgetl(lex_tmp);
   bputl(lex, wcount);
+  slice_init();
   while (wid_lex <= wcount)
     {
       u32 in_id = bgetl(lex_tmp);
@@ -88,17 +92,10 @@ split(struct fastbuf *sorted)
       bputo(lex, btell(refs));
       if (wid_lex >= wid_ref)
 	{
-	  uns rsize;
-	  rsize = bgetl(sorted);
+	  uns rsize = slice_chain(sorted, refs);
 	  if (rsize >= 0x10000000)
-	    {
-	      word[wlen] = 0;
-	      die("Reference chain #%d for word %s too long, maximum is 256MB. Ask a wizard to enlarge me.", wid_ref, word);
-	    }
-	  bputw(lex, (rsize + 4 + 0xfff) >> 12U);
-	  while (rsize)
-	    rsize -= 4 + bbcopy_chain(sorted, refs, bgetl(sorted));
-	  bputl(refs, 0);
+	    die("Reference chain #%d for word %s too long, maximum is 256MB. Ask a wizard to enlarge me.", wid_ref, word);
+	  bputw(lex, (rsize + 0xfff) >> 12U);
 	  wid_ref = bgetl(sorted);
 	}
       else
@@ -113,6 +110,7 @@ split(struct fastbuf *sorted)
       wid_lex++;
     }
   ASSERT(wid_ref = 0xffffffff);
+  slice_cleanup();
   bclose(lex_tmp);
   bclose(refs);
   bclose(lex);
