@@ -52,9 +52,12 @@ extern uns lang_detect_mode;
 extern uns lang_detect_min_doc_length;
 extern byte *lang_detect_tables_file;
 
-void lang_detect_build_automaton(void);
-static inline void lang_detect_start(void) {}
-void lang_detect_add_string(byte *str);
-void lang_detect_add_fastbuf(struct fastbuf *fb);
-struct lang_detect_results *lang_detect_compute(void);
-uns lang_detect_choose_best(void);
+struct lang_detect;
+
+void lang_detect_load_tables(void);
+struct lang_detect *lang_detect_alloc(struct mempool *pool);
+static inline void lang_detect_start(struct lang_detect *ld UNUSED) {}
+void lang_detect_add_string(struct lang_detect *ld, byte *str);
+void lang_detect_add_fastbuf(struct lang_detect *ld, struct fastbuf *fb);
+struct lang_detect_results *lang_detect_compute(struct lang_detect *ld);
+uns lang_detect_choose_best(struct lang_detect *ld);
