@@ -363,14 +363,16 @@ compute_frequencies(struct filter_args *fa)
 	printf("Computing frequencies of sequences of length 1..%d for %d+%d languages\n",
 		seq_length, detect_lang_count[LF_ACCENT], detect_lang_count[LF_NOACCENT]);
 	struct mempool *obj_pool = mp_new(1 << 12);
-	struct bucket_source *src = get_buck_init();
+	struct get_buck gb;
+	get_buck_init(&gb);
+	gb.pool = obj_pool;
 	struct fastbuf *b_buck = bopen(bucket_state_file, O_CREAT | O_TRUNC | O_WRONLY, 1024);
-	for (; src->get_next(src, obj_pool, ~0U); mp_flush(obj_pool))
+	for (; get_buck_next(&gb, ~0U); mp_flush(obj_pool))
 	{
-		struct odes *obj = src->o;
+		struct odes *obj = gb.o;
 		if (my_progress > 0 && !(count % my_progress))
 			fprintf(stderr, "Processed %d buckets, training %d, testing %d\r", count, training, testing);
-		bprintf(b_buck, "%08x ", src->oid);
+		bprintf(b_buck, "%08x ", gb.oid);
 		count++;
 
 		/* Parse the language of the object.  */
@@ -407,7 +409,7 @@ compute_frequencies(struct filter_args *fa)
 		bprintf(b_buck, ".. %6d %5d %s\n", 0, 0, url);	/* two dots and two numbers for the 3rd pass */
 	}
 	bclose(b_buck);
-	src->cleanup(src);
+	get_buck_cleanup(&gb);
 	mp_delete(obj_pool);
 	printf("Processed %d buckets, training %d, testing %d\nFound %d distinct sequences\n\n",
 		count, training, testing, seq_total_count);
@@ -648,7 +650,9 @@ test_detector(uns just_test_set)
 	uns count = 0;
 	uns stat[DM_MAX][OT_MAX];
 	uns lang_docs[MAX_LANGUAGES];
-	lang_detect_build_automaton();
+	lang_detect_load_tables();
+	struct lang_detect *lang_detect = lang_detect_alloc(NULL);
+
 	bzero(stat, sizeof(stat));
 	bzero(lang_docs, sizeof(lang_docs));
 
@@ -656,11 +660,13 @@ test_detector(uns just_test_set)
 	prof_init(&cnt);
 
 	struct mempool *obj_pool = mp_new(1 << 12);
-	struct bucket_source *src = get_buck_init();
+	struct get_buck gb;
+	gb.pool = obj_pool;
+	get_buck_init(&gb);
 	struct fastbuf *b_buck = just_test_set ? bopen(bucket_state_file, O_RDWR, 1024) : NULL;
 	struct fastbuf *b_threshold = bopen(threshold_file, O_CREAT | O_TRUNC | O_WRONLY, 1024);
 	prof_start(&cnt);
-	for (; src->get_next(src, obj_pool, ~0U); mp_flush(obj_pool))
+	for (; get_buck_next(&gb, ~0U); mp_flush(obj_pool))
 	{
 		enum doc_mode mode;
 		byte doc_lang[3];
@@ -686,14 +692,14 @@ test_detector(uns just_test_set)
 		count++;
 
 		/* Discard the recent object and read a new one.  */
-		struct odes *obj = src->o;
+		struct odes *obj = gb.o;
 		ASSERT(obj);
 
 		/* Run the language detector.  */
-		lang_detect_start();
+		lang_detect_start(lang_detect);
 		for (struct oattr *a=obj_find_attr(obj, 'X'); a; a=a->same)
-			lang_detect_add_string(a->val);
-		struct lang_detect_results *results = lang_detect_compute();
+			lang_detect_add_string(lang_detect, a->val);
+		struct lang_detect_results *results = lang_detect_compute(lang_detect);
 
 		/* Find the most probable language.  */
 		enum outcome_type outcome;
@@ -757,7 +763,7 @@ test_detector(uns just_test_set)
 		else
 		{
 			/* Output the chosen language.  */
-			printf("%08x %2.2s %6d %5d %s\n", src->oid, lang_name1, results->total_occur, results->ratio, obj_find_aval(obj, 'U'));
+			printf("%08x %2.2s %6d %5d %s\n", gb.oid, lang_name1, results->total_occur, results->ratio, obj_find_aval(obj, 'U'));
 		}
 	}
 	prof_stop(&cnt);
@@ -767,7 +773,7 @@ test_detector(uns just_test_set)
 		bclose(b_buck);
 	}
 	bclose(b_threshold);
-	src->cleanup(src);
+	get_buck_cleanup(&gb);
 	mp_delete(obj_pool);
 
 	printf("\nTested %d buckets\n", count);
