@@ -38,6 +38,7 @@
 #include "lib/conf.h"
 #include "lib/getopt.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/mempool.h"
 #include "lib/clists.h"
 #include "lib/chartype.h"
@@ -47,6 +48,7 @@
 #include "charset/unicat.h"
 #include "lang/stem-dict.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -191,7 +193,7 @@ static uns num_patterns;
 struct pattern {
   uns use_cnt;
   pattern_id_t id;
-  word flags;		// bottom 8 bits contain the prefix mask
+  u16 flags;		// bottom 8 bits contain the prefix mask
   suffix_id_t suff[1];	// suff[0] is the number of suffixes
 };
 
