@@ -24,9 +24,11 @@
 #include "lib/lib.h"
 #include "lib/mempool.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/chartype.h"
 #include "lang/stem-table.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 
