@@ -14,4 +14,9 @@
 #define	GBUF_PREFIX(x)	bb_##x
 #include "lib/gbuf.h"
 
+char *bb_vprintf(bb_t *bb, char *fmt, va_list args);
+char *bb_printf(bb_t *bb, char *fmt, ...);
+char *bb_vprintf_at(bb_t *bb, uns ofs, char *fmt, va_list args);
+char *bb_printf_at(bb_t *bb, uns ofs, char *fmt, ...);
+
 #endif
