@@ -13,9 +13,11 @@
 #include "lib/hashfunc.h"
 #include "lib/heap.h"
 #include "lib/url.h"
+#include "lib/unicode.h"
+#include "lib/workqueue.h"
+#include "lib/lfs.h"
 #include "sherlock/index.h"
 #include "sherlock/tagged-text.h"
-#include "lib/unicode.h"
 #include "sherlock/lizard-fb.h"
 #include "sherlock/object.h"
 #include "charset/unicat.h"
@@ -49,6 +51,7 @@ kmp_alloc(struct kmp_struct *kmp, uns size)
   return mp_alloc(kmp->u.pool, size);
 }
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <alloca.h>
@@ -274,6 +277,7 @@ dump_text_interval(byte *label, struct context_interval *range)
 	uns line_len;
 	uns space;
 	uns type;
+	uns whole_text = range->first_char == ft.text && range->last_char == ft.text_end;
 
 	text = range->first_char;
 	type = range->start_type;
@@ -281,8 +285,8 @@ dump_text_interval(byte *label, struct context_interval *range)
 	line_len = 0;
 	if (label)
 		dump_word(&line_len, label, strlen(label), 1);
-	DUMP_MASK(1, "<block c=%d-%d l=%d>",
-		(uns)(range->first_char - ft.text), (uns)(range->last_char - ft.text), range->context_len);
+	if (!whole_text)
+		DUMP_MASK(1, "<block c=%d-%d l=%d>", (uns)(range->first_char - ft.text), (uns)(range->last_char - ft.text), range->context_len);
 	DUMP_MASK(DUMP_INTERTAG_SPACES, "<%s>", type_names[type]);
 	struct ft_word *last_word = ft.words-1;
 	while (text < range->last_char)
@@ -364,13 +368,15 @@ dump_text_interval(byte *label, struct context_interval *range)
 		}
 	}
 	DUMP_MASK(1, "</%s>", type_names[type]);
-	DUMP_CONST(DUMP_INTERTAG_SPACES, "</block>");
+	if (!whole_text)
+		DUMP_CONST(DUMP_INTERTAG_SPACES, "</block>");
 	dump_word(&line_len, NULL, 0, 0);
 }
 
 /***** Computing intervals that will be dumped *****/
 
 #define	IS_SENTENCE_BREAK(c)	((c) == '.' || (c) == '?' || (c) == '!')
+#define	IS_WORD_BREAK(c)	(Uspace(c) || (c) == ',' || (c) == ';' || (c) == ':' || (c) == '-' || (c) == '/')
 
 #define	PREV_TAGGED_CHAR(pos, pos1, limit, c) ({\
 	byte *pos2;\
@@ -496,7 +502,7 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 		if (c < 0x8000000)
 		{
 			added++;
-			if (added >= limit2 && Uspace(c)
+			if (added >= limit2 && IS_WORD_BREAK(c)
 			|| added >= limit1 && IS_SENTENCE_BREAK(c))
 				break;
 		}
@@ -542,7 +548,7 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 		if (c < 0x8000000)
 		{
 			added++;
-			if (added >= limit2 && Uspace(c)
+			if (added >= limit2 && IS_WORD_BREAK(c)
 			|| added >= limit1 && IS_SENTENCE_BREAK(c))
 			{
 				if (Uspace(c))
@@ -793,7 +799,7 @@ parse_brackets(struct query *q, uns type, byte **start, byte *end, struct save_l
 					case 'y':	subtype = BT_REDIRECT; break;
 					case 'c':	subtype = BT_CATALOG; break;
 					case 'x':	subtype = BT_REFTEXT; break;
-					default:	ASSERT(pa.val[0] >= '0' && pa.val[0] <= '9');
+					default:	ASSERT((pa.val[0] >= '0' && pa.val[0] <= '9') || pa.val[0] == 'f');
 							subtype = BT_USER; break;
 				}
 				ASSERT(subtype > type);
@@ -977,6 +983,7 @@ dump_highlighted_url(byte *start, byte *end, byte *tag_name)
 			while (c < c_end)
 			{
 				struct kmp_search search;
+				search.u.best_pos = NULL; /* not necessary, only hides warning */
 				search.u.src_end = c_end;
 				kmp_search(ft_kmp, &search, c);
 				if (!search.u.best_len)
@@ -1004,7 +1011,7 @@ dump_highlighted_url(byte *start, byte *end, byte *tag_name)
 }
 
 static void
-dump_lines_between(struct query *q, struct save_lm_pos *saved_pos, byte *start, byte *end)
+dump_lines_between(struct query *q, struct save_lm_pos *saved_pos, byte *start, byte *end, struct bracket *n)
 {
 	struct parsed_attr pa;
 	for (byte *record = start; get_attr(&record, end, &pa) > 0; )
@@ -1036,9 +1043,9 @@ dump_lines_between(struct query *q, struct save_lm_pos *saved_pos, byte *start,
 
 		if (pa.attr == 'U')
 			dump_highlighted_url(pa.val, pa.val + pa.len, "url");
-		else if (pa.attr == 'y')
+		else if (pa.attr == 'y' && (n->sons & BT_MASK) != BT_CATALOG)
 			dump_highlighted_url(pa.val, pa.val + pa.len, "redirect");
-		else if (pa.attr == 'b')
+		else if (pa.attr == 'b' && (n->sons & BT_MASK) != BT_CATALOG)
 			dump_highlighted_url(pa.val, pa.val + pa.len, "frameof");
 		else if (strchr(url_attributes, pa.attr))
 		{
@@ -1073,7 +1080,7 @@ dump_brackets(struct query *q, struct bracket *n)
 	uns sons = n->sons & ~BT_MASK;
 	if (!sons)
 	{
-		dump_lines_between(q, &n->head_pos, n->start, n->end);
+		dump_lines_between(q, &n->head_pos, n->start, n->end, n);
 		return;
 	}
 	byte *head = n->son[0]->start;
@@ -1082,7 +1089,7 @@ dump_brackets(struct query *q, struct bracket *n)
 
 	bracket_sons_sort(sons, n);		// by ascending bracket-type and then by descending weight
 
-	dump_lines_between(q, &n->head_pos, n->start, head);
+	dump_lines_between(q, &n->head_pos, n->start, head, n);
 	uns end;
 	for (uns i=0; i<sons; i=end)
 	{
@@ -1096,7 +1103,141 @@ dump_brackets(struct query *q, struct bracket *n)
 			dump_brackets(q, n->son[j]);
 		}
 	}
-	dump_lines_between(q, &n->tail_pos, tail, n->end);
+	dump_lines_between(q, &n->tail_pos, tail, n->end, n);
+}
+
+/***** Fetching of cards *****/
+
+struct card_fetch {
+  struct work w;
+  struct card_attr *ca;
+  struct database *db;
+  void *card;
+  uns done;
+};
+
+static struct work_queue fetch_wqueue;
+static struct worker_pool fetch_wpool;
+
+static void
+fetch_single(struct worker_thread *t UNUSED, struct work *work)
+{
+  /* Called by worker threads to service fetch requests */
+  struct card_fetch *w = (struct card_fetch *) work;
+  struct card_attr *ca = w->ca;
+  struct database *db = w->db;
+  int size = ((ca+1)->card - ca->card) << CARD_POS_SHIFT;
+  if ((int)sh_pread(db->fd_cards, w->card, size, (sh_off_t) ca->card << CARD_POS_SHIFT) != size)
+    w->card = NULL;
+}
+
+static void
+fetch_start(struct query *q)
+{
+  struct card_fetch *fetches = q->card_fetches;
+  uns n = q->ncard_fetches;
+  ASSERT(fetches);
+  if (!n)
+    return;
+
+  prof_t *o = profiler_switch(&prof_resf);
+  if (fetch_threads)
+    {
+      ASSERT(!fetch_wqueue.nr_running);
+      for (uns i=0; i<n; i++)
+        {
+	  struct card_fetch *f = &fetches[i];
+	  f->w.go = fetch_single;
+	  f->card = mp_alloc(q->pool, ((f->ca+1)->card - f->ca->card) << CARD_POS_SHIFT);
+	}
+      for (uns i=0; i<n; i++)
+        work_submit(&fetch_wqueue, &fetches[i].w);
+    }
+  else
+    {
+      struct mmap_request mmaps[n];
+      for (uns i=0; i<n; i++)
+        {
+	  struct card_fetch *f = &fetches[i];
+          struct mmap_request *m = &mmaps[i];
+          m->u.req.fd = f->db->fd_cards;
+          m->u.req.start = (sh_off_t) f->ca->card << CARD_POS_SHIFT;
+          m->u.req.end = (sh_off_t) (f->ca+1)->card << CARD_POS_SHIFT;
+          m->userdata = i;
+	  f->done = 1;
+	}
+      if (mmap_regions(q, mmaps, n) < 0)
+        {
+	  /* If mmapping fails, return with all cards unread, but done */
+          log(L_ERROR, "Error mapping index cards for query result display");
+          return;
+        }
+      for (uns i=0; i<n; i++)
+	fetches[mmaps[i].userdata].card = mmaps[i].u.map.start;
+    }
+  profiler_switch(o);
+}
+
+static void
+fetch_wait(struct card_fetch *f)
+{
+  if (fetch_threads)
+    {
+      if (!f->done)
+	{
+	  prof_t *o = profiler_switch(&prof_resf);
+	  while (!f->done)
+	    {
+	      struct card_fetch *ok = (struct card_fetch *)work_wait(&fetch_wqueue);
+	      ASSERT(ok);
+	      ok->done = 1;
+	    }
+	  profiler_switch(o);
+	}
+    }
+  else
+    ASSERT(f->done);
+}
+
+static byte *
+fetch_decompress(struct card_fetch *f, byte **end)
+{
+  byte *ptr;
+  uns type;
+
+  fetch_wait(f);
+  if (!f->card)
+    {
+      log(L_ERROR, "Cannot load object %08x from database %s: %m", (uns)(f->ca - f->db->card_attrs), f->db->name);
+      return NULL;
+    }
+
+  int len = lizard_memread(liz_buf, f->card, &ptr, &type);
+  if (len < 0)
+    die("Cannot decompress object %08x from database %s: %m", (uns)(f->ca - f->db->card_attrs), f->db->name);
+  get_attr_set_type(type);
+  *end = ptr + len;
+  return ptr;
+}
+
+void
+prefetch_results_cleanup(struct query *q)
+{
+  /* There still could be some prefetches running, so better wait for all of them. */
+  for (uns i=0; i<q->ncard_fetches; i++)
+    fetch_wait(&q->card_fetches[i]);
+  ASSERT(!fetch_threads || !fetch_wqueue.nr_running);
+}
+
+static void
+card_fetch_init(void)
+{
+  if (fetch_threads)
+    {
+      fetch_wpool.num_threads = fetch_threads;
+      worker_pool_init(&fetch_wpool);
+      work_queue_init(&fetch_wpool, &fetch_wqueue);
+    }
 }
 
 /***** The body of the card-dumper *****/
@@ -1153,6 +1294,11 @@ check_result_set(struct query *q)
       break;
     case OUT_DUMP:
       ASSERT(q->out_range);
+      if (!q->db_list)
+	{
+	  add_err("-122 Missing DB specification");
+	  return 1;
+	}
       SLIST_FOR_EACH(struct dump_node *, n, *q->out_range)
 	{
 	  struct db_node *d = slist_head(q->db_list);
@@ -1196,7 +1342,6 @@ show_results_list(struct query *q)
 {
   struct results *res = q->results;
 
-  profiler_switch(&prof_results);
   SLIST_FOR_EACH(struct set_node *, rng, *q->out_range)
     {
       int from = rng->min - 1;
@@ -1218,14 +1363,11 @@ show_results_full(struct query *q)
 {
   struct results *res = q->results;
   struct result_note *notes[q->results_to_show];
-  struct mmap_request *mmaps[q->results_to_show];
   uns nres = 0;
-  uns i;
 
   if (!res->num_matches)
     return;
 
-  profiler_switch(&prof_resf);
   SLIST_FOR_EACH(struct set_node *, rng, *q->out_range)
     {
       uns from = rng->min - 1;
@@ -1234,84 +1376,69 @@ show_results_full(struct query *q)
 	notes[nres++] = &res->matches[from++];
     }
   ASSERT(nres <= q->results_to_show);
-  struct mmap_request mmap_array[nres];
-  for (i=0; i<nres; i++)
+  q->card_fetches = mp_alloc_zero(q->pool, sizeof(struct card_fetch) * nres);
+  q->ncard_fetches = nres;
+  for (uns i=0; i<nres; i++)
     {
-      struct mmap_request *m = &mmap_array[i];
-      struct card_attr *ca = notes[i]->attr;
-      struct database *db = attr_to_db(ca, NULL);
-      m->u.req.fd = db->fd_cards;
-      m->u.req.start = (sh_off_t) ca->card << CARD_POS_SHIFT;
-      m->u.req.end = (sh_off_t) (ca+1)->card << CARD_POS_SHIFT;
-      m->userdata = i;
+      struct card_fetch *f = &q->card_fetches[i];
+      f->ca = notes[i]->attr;
+      f->db = attr_to_db(f->ca, NULL);
     }
-  if (mmap_regions(q, mmap_array, nres) < 0)
-    {
-      log(L_ERROR, "Error mapping index cards for query result display");
-      return;
-    }
-  profiler_switch(&prof_results);
-  for (i=0; i<nres; i++)
-    mmaps[mmap_array[i].userdata] = &mmap_array[i];
+  fetch_start(q);
+
   ft_kmp = kmp_create(q);
-  for (i=0; i<nres; i++)
-  {
-    byte *ptr;
-    uns type;
-    int len = lizard_memread(liz_buf, mmaps[i]->u.map.start, &ptr, &type);
-    if (len < 0)
+  for (uns i=0; i<nres; i++)
     {
-      struct card_attr *ca = notes[i]->attr;
-      oid_t oid;
-      attr_to_db(ca, &oid);
-      die("Cannot decompress object %08x: %m", oid);
+      struct card_fetch *f = &q->card_fetches[i];
+      byte *ptr, *end;
+      if (!(ptr = fetch_decompress(f, &end)))
+	return;
+      show_card(q, notes[i]->attr, notes[i], ptr, end);
     }
-    get_attr_set_type(type);
-    show_card(q, notes[i]->attr, notes[i], ptr, ptr+len);
-  }
+}
+
+static void
+dump_results_prefetch(struct query *q)
+{
+  q->card_fetches = mp_alloc_zero(q->pool, sizeof(struct card_fetch) * q->results_to_show);
+  SLIST_FOR_EACH(struct dump_node *, n, *q->out_range)
+    {
+      struct card_fetch *f = &q->card_fetches[q->ncard_fetches++];
+      f->ca = n->db->card_attrs + n->id;
+      f->db = n->db;
+    }
+  fetch_start(q);
 }
 
 static void
 dump_results(struct query *q)
 {
-  profiler_switch(&prof_resf);
-  struct mmap_request mmap_array[q->results_to_show];
-  struct mmap_request *mmaps[q->results_to_show];
+  if (!q->card_fetches)
+    dump_results_prefetch(q);
+  ft_kmp = kmp_create(q);
   uns i = 0;
   SLIST_FOR_EACH(struct dump_node *, n, *q->out_range)
     {
-      struct card_attr *ca = n->db->card_attrs + n->id;
-      struct mmap_request *m = &mmap_array[i];
-      m->u.req.fd = n->db->fd_cards;
-      m->u.req.start = (sh_off_t) ca->card << CARD_POS_SHIFT;
-      m->u.req.end = (sh_off_t) (ca+1)->card << CARD_POS_SHIFT;
-      m->userdata = i++;
+      struct card_fetch *f = &q->card_fetches[i++];
+      byte *ptr, *end;
+      if (!(ptr = fetch_decompress(f, &end)))
+	return;
+      show_card(q, f->ca, NULL, ptr, end);
     }
-  if (mmap_regions(q, mmap_array, q->results_to_show) < 0)
-    {
-      log(L_ERROR, "Error mapping index cards for query result display");
-      return;
-    }
-  profiler_switch(&prof_results);
-  for (i=0; i<q->results_to_show; i++)
-    mmaps[mmap_array[i].userdata] = &mmap_array[i];
-  ft_kmp = kmp_create(q);
-  i = 0;
-  SLIST_FOR_EACH(struct dump_node *, n, *q->out_range)
-  {
-    byte *ptr;
-    uns type;
-    int len = lizard_memread(liz_buf, mmaps[i++]->u.map.start, &ptr, &type);
-    if (len < 0)
-      die("Cannot decompress object %08x from database %s: %m", n->id, n->db->name);
-    get_attr_set_type(type);
-    show_card(q, n->db->card_attrs + n->id, NULL, ptr, ptr+len);
-  }
+}
+
+void
+prefetch_results(struct query *q)
+{
+  /* In threaded mode, we can parallelize card prefetch for DUMPs with query analysis. */
+  if (q->out_mode == OUT_DUMP && fetch_threads)
+    dump_results_prefetch(q);
 }
 
 void
 show_results(struct query *q)
 {
+  profiler_switch(&prof_results);
   switch (q->out_mode)
     {
     case OUT_STATS:
@@ -1335,3 +1462,9 @@ cards_init(void)
 {
   alphabet_init();
 }
+
+void
+cards_init_process(void)
+{
+  card_fetch_init();
+}
