@@ -6,12 +6,11 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
 #include "lib/chartype.h"
 #include "lib/ipaccess.h"
 #include "lib/unicode.h"
 #include "lib/fastbuf.h"
-#include "sherlock/conf-parse.h"
+#include "sherlock/conf.h"
 #include "search/sherlockd.h"
 
 #include <string.h>
@@ -103,10 +102,16 @@ uns spell_common_penalty;
 uns spell_dwarf;
 uns spell_dwarf_margin;
 clist spell_common_pairs;
+clist spell_phrases;
 uns second_best_reduce = 1;
 uns magic_merge_bonus;
 clist access_list;
 uns hydra_processes;
+uns slice_threads;
+uns max_image_sims;
+uns image_sim_max_weight;
+uns image_sim_slope;
+uns fetch_threads;
 
 static byte *
 cf_database_init(struct database *db)
@@ -178,14 +183,20 @@ cf_search_commit(void *ptr UNUSED)
     count++;
   if (count >= HARD_MAX_DATABASES)
     return "Too many databases defined (HARD_MAX_DATABASES exceeded)";
-  uns max = 0;
+  uns max = 0, cnt = 0;
   for (uns i=0; i<16; i++)
     if (global_meta_chars[i] != ~0U)
-      max = MAX(max, global_meta_chars[i]);
-  CF_JOURNAL_VAR(global_meta_chars);
-  for (uns i=0; i<16; i++)
-    if (global_meta_chars[i] == ~0U)
-      global_meta_chars[i] = max;
+      {
+        max = MAX(max, global_meta_chars[i]);
+	cnt++;
+      }
+  if (cnt)
+    {
+      CF_JOURNAL_VAR(global_meta_chars);
+      for (uns i=0; i<16; i++)
+        if (global_meta_chars[i] == ~0U)
+          global_meta_chars[i] = max;
+    }
   return NULL;
 }
 
@@ -202,11 +213,12 @@ static struct cf_section database_config = {
   CF_ITEMS {
     CF_STRING("Name", F(name)),
     CF_STRING("Directory", F(directory)),
-    CF_BITMAP_LOOKUP("Parts", F(parts), ((byte *[]) {"words", "strings", "prints", "blacklist", NULL })),
+    CF_BITMAP_LOOKUP("Parts", F(parts), ((byte *[]) {"words", "strings", "prints", "image-signatures", NULL })),
     CF_SECTION("WordWeights", F(word_weights[0]), &word_weights_config),
     CF_SECTION("MetaWeights", F(meta_weights[0][0]), &meta_weights_config),
     CF_SECTION("StringWeights", F(string_weights[0]), &string_weights_config),
     CF_UNS("IsOptional", F(is_optional)),
+    CF_STRING_DYN("Blacklists", F(blacklists), CF_ANY_NUM),
     CF_END
   }
 };
@@ -222,7 +234,7 @@ init_database_config(void)
   cf_generate_word_type_config(&meta_chars_config, mt_user_names, 1, 0);
 }
 
-static struct cf_section spell_config = {
+static struct cf_section spell_pair_config = {
   CF_TYPE(struct spell_pair),
   CF_ITEMS {
     CF_USER("X", PTR_TO(struct spell_pair, x), &cf_type_unichar),
@@ -231,6 +243,18 @@ static struct cf_section spell_config = {
   }
 };
 
+#define F(x) PTR_TO(struct spell_phrase, x)
+static struct cf_section spell_phrase_config = {
+  CF_TYPE(struct spell_phrase),
+  CF_ITEMS {
+    CF_STRING("Src", F(src)),
+    CF_STRING("Dest", F(dest)),
+    CF_UNS("Penalty", F(penalty)),
+    CF_END
+  }
+};
+#undef F
+
 static struct cf_section search_config = {
   CF_INIT(cf_search_init),
   CF_COMMIT(cf_search_commit),
@@ -320,9 +344,15 @@ static struct cf_section search_config = {
     CF_UNS("Synonyming", &global_synonyming),
     CF_UNS("SynExpand", &global_syn_expand),
     CF_UNS("SecondBestReduce", &second_best_reduce),
-    CF_LIST("SpellCommonPairs", &spell_common_pairs, &spell_config),
+    CF_LIST("SpellCommonPairs", &spell_common_pairs, &spell_pair_config),
+    CF_LIST("SpellPhrases", &spell_phrases, &spell_phrase_config),
     CF_UNS("MagicMergeBonus", &magic_merge_bonus),
     CF_UNS("HydraProcesses", &hydra_processes),
+    CF_UNS("SliceThreads", &slice_threads),
+    CF_UNS("MaxImageSims", &max_image_sims),
+    CF_UNS("ImageSimMaxWeight", &image_sim_max_weight),
+    CF_UNS("ImageSimSlope", &image_sim_slope),
+    CF_UNS("FetchThreads", &fetch_threads),
     CF_END
   }
 };
