@@ -5,11 +5,13 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/lfs.h"
 #include "sherlock/index.h"
+#include "lib/lfs.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/mempool.h"
 #include "lib/conf.h"
+#include "lib/bitarray.h"
 #include "indexer/lexicon.h"
 #include "indexer/params.h"
 #include "search/sherlockd.h"
@@ -35,42 +37,46 @@ db_file_name(struct database *db, byte *fn)
 
 struct merge_status {
   struct merge_status *next;
-  struct card_attr *attrs;
   struct fastbuf *fb;
   struct card_print next_fp;
+  bitarray_t dup_flags;
 };
 
 static void
-db_reset_attrs(struct database *db)
+db_init_dup_flags(struct database *db)
 {
-  struct card_attr *attrs = db->card_attrs;
-  for (uns i=0; i<db->num_ids; i++)
-    attrs[i].flags &= ~CARD_FLAG_DUP;
+  db->dup_flags = bit_array_xmalloc_zero(db->num_ids);
 }
 
 static void
-db_apply_blacklist(struct database *db)
+db_apply_dup_flags(struct database *db)
 {
-  struct fastbuf *b = bopen_try(db_file_name(db, "blacklist"), O_RDONLY, 65536);
-  if (!b)
+  bitarray_t a = db->dup_flags;
+  for (uns i = 0; i < db->num_ids; i++)
+    if (bit_array_isset(a, i))
+      db->card_attrs[i].flags |= CARD_FLAG_DUP;
+    else
+      db->card_attrs[i].flags &= ~CARD_FLAG_DUP;
+  xfree(a);
+}
+
+static void
+db_apply_blacklists(struct database *db)
+{
+  bitarray_t a = db->dup_flags;
+  uns count = 0, x;
+  for (uns i = 0; i < DARY_LEN(db->blacklists); i++)
     {
-      log(L_INFO, "No blacklist found");
-      return;
+      struct fastbuf *b = bopen_try(db_file_name(db, db->blacklists[i]), O_RDONLY, 65536);
+      if (!b)
+	{
+	  log(L_INFO, "Blacklist `%s' not found", db->blacklists[i]);
+	  continue;
+	}
+      while ((x = bgetl(b)) != ~0U)
+	count += !bit_array_test_and_set(a, x);
+      bclose(b);
     }
-  uns x;
-  uns last = 0;
-  uns count = 0;
-  while ((x = bgetl(b)) != 0xffffffff)
-    {
-      while (last < x)
-	db->card_attrs[last++].flags &= ~CARD_FLAG_DUP;
-      ASSERT(last == x);
-      db->card_attrs[last++].flags |= CARD_FLAG_DUP;
-      count++;
-    }
-  while (last < db->num_ids)
-    db->card_attrs[last++].flags &= ~CARD_FLAG_DUP;
-  bclose(b);
   log(L_INFO, "Blacklisted %d cards", count);
 }
 
@@ -85,7 +91,7 @@ db_merge(void)
     if (db->fb_card_prints)
       {
 	m = alloca(sizeof(*m));
-	m->attrs = db->card_attrs;
+	m->dup_flags = db->dup_flags;
 	m->fb = db->fb_card_prints;
 	if (breadb(m->fb, &m->next_fp, sizeof(struct card_print)))
 	  {
@@ -123,7 +129,7 @@ db_merge(void)
 	    {
 	      if (!memcmp(&m->next_fp.fp, &mm->next_fp.fp, sizeof(struct fingerprint)))
 		{
-		  m->attrs[m->next_fp.cardid].flags |= CARD_FLAG_DUP;
+		  bit_array_set(m->dup_flags, m->next_fp.cardid);
 		  override_cnt++;
 		  if (unlikely(!breadb(m->fb, &m->next_fp, sizeof(struct card_print))))
 		    {
@@ -177,7 +183,7 @@ db_init(int merge_only)
 	  db->fb_card_prints = bopen(db_file_name(db, "card-prints"), O_RDONLY, 65536);
 	  seen_prints++;
 	}
-      int rw = !!(db->parts & (DB_PART_PRINTS | DB_PART_BLACKLIST));
+      int rw = (db->parts & DB_PART_PRINTS) || DARY_LEN(db->blacklists);
       uns size;
       db->card_attrs = mmap_file(db_file_name(db, "card-attrs"), &size, rw);
       db->num_ids = size / sizeof(struct card_attr);
@@ -198,15 +204,17 @@ db_init(int merge_only)
       db->ref_file_size = sh_seek(db->fd_refs, 0, SEEK_END);
 
       log(L_INFO, "Loading database %s: %d documents", db->name, db->num_ids);
-      if (db->parts & DB_PART_BLACKLIST)
-	db_apply_blacklist(db);
-      else if (db->parts & DB_PART_PRINTS)
-	db_reset_attrs(db);
+      if (db->parts & DB_PART_PRINTS || DARY_LEN(db->blacklists))
+	db_init_dup_flags(db);
+      if (DARY_LEN(db->blacklists))
+	db_apply_blacklists(db);
       if (!merge_only)
 	{
 	  words_init(db);
 	  strings_init(db);
 	}
+
+      images_init(db);
     }
   if (seen_prints)
     db_merge();
@@ -217,8 +225,9 @@ db_init(int merge_only)
 	  bclose(db->fb_card_prints);
 	  db->fb_card_prints = NULL;
 	}
-      if (db->parts & (DB_PART_PRINTS | DB_PART_BLACKLIST))
+      if ((db->parts & DB_PART_PRINTS) || DARY_LEN(db->blacklists))
 	{
+	  db_apply_dup_flags(db);
 	  msync(db->card_attrs, db->num_ids * sizeof(struct card_attr), MS_SYNC);
 	  if (mprotect(db->card_attrs, db->num_ids * sizeof(struct card_attr), PROT_READ) < 0)
 	    die("Cannot reprotect card attributes read-only: %m");
