@@ -13,22 +13,21 @@
 
 #include <stdlib.h>
 #include <sys/mman.h>
-#include <sys/user.h>
 
-static addr_int_t mem_map_zone_start, mem_map_zone_end;
+static uintptr_t mem_map_zone_start, mem_map_zone_end;
 struct lizard_buffer *liz_buf;
 
 void
 memory_init(void)
 {
   uns len = mem_map_zone_size << 20;
-  void *zone = mmap(NULL, len, PROT_READ, MAP_ANON | MAP_PRIVATE, 0, 0);
+  void *zone = mmap(NULL, len, PROT_READ, MAP_ANON | MAP_PRIVATE, -1, 0);
 
   if (zone == MAP_FAILED)
     die("Unable to find %dMB of mmap space (%m)", mem_map_zone_size);
-  mem_map_zone_start = (addr_int_t) zone;
+  mem_map_zone_start = (uintptr_t) zone;
   mem_map_zone_end = mem_map_zone_start + len;
-  log(L_INFO, "Memory mapping zone: %08lx-%08lx", mem_map_zone_start, mem_map_zone_end - 1);
+  log(L_INFO, "Memory mapping zone: %08llx-%08llx", (long long) mem_map_zone_start, (long long) mem_map_zone_end - 1);
 
   liz_buf = lizard_alloc();
 }
@@ -44,12 +43,19 @@ memory_flush(struct query *q)
 {
   if (q->last_mapping > mem_map_zone_start)
     {
-      if (mmap((void *) mem_map_zone_start, mem_map_zone_end - mem_map_zone_start, PROT_READ, MAP_FIXED | MAP_ANON | MAP_PRIVATE, 0, 0) == MAP_FAILED)
+      if (mmap((void *) mem_map_zone_start, mem_map_zone_end - mem_map_zone_start, PROT_READ, MAP_FIXED | MAP_ANON | MAP_PRIVATE, -1, 0) == MAP_FAILED)
 	die("Clearing of memory maps failed");
       q->last_mapping = mem_map_zone_start;
     }
 }
 
+/*
+ *  We order all mmap requests by their file offset, which gives
+ *  an approximation of the real ordering of the corresponding disk
+ *  blocks.  A surprisingly good one, it turns out -- on ext3,
+ *  large files are usually stored monotonously.
+ */
+
 static int
 mmap_compare(const void *A, const void *B)
 {
@@ -69,18 +75,18 @@ mmap_compare(const void *A, const void *B)
 static void
 mmap_prefetch(struct mmap_request *reqs, int count)
 {
-  addr_int_t mask = PAGE_SIZE - 1;
-  addr_int_t start, end;
+  uintptr_t mask = CPU_PAGE_SIZE - 1;
+  uintptr_t start, end;
   volatile byte x;
 
   while (count--)
     {
-      start = ((addr_int_t) reqs->u.map.start) & ~mask;
-      end = ((addr_int_t) reqs->u.map.end + mask) & ~mask;
+      start = ((uintptr_t) reqs->u.map.start) & ~mask;
+      end = ((uintptr_t) reqs->u.map.end + mask) & ~mask;
       while (start < end)
 	{
 	  x = *(byte *)start;
-	  start += PAGE_SIZE;
+	  start += CPU_PAGE_SIZE;
 	}
       reqs++;
     }
@@ -91,13 +97,13 @@ mmap_regions(struct query *q, struct mmap_request *reqs, int count)
 {
   int i, j;
   sh_off_t start, end, size, total_size;
-  sh_off_t mask = (sh_off_t)(PAGE_SIZE-1);
+  sh_off_t mask = (sh_off_t)(CPU_PAGE_SIZE-1);
   void *x;
-  addr_int_t addr;
+  uintptr_t addr;
 
   qsort(reqs, count, sizeof(struct mmap_request), mmap_compare);
   total_size = 0;
-  addr_int_t first_addr = q->last_mapping;
+  uintptr_t first_addr = q->last_mapping;
   for (i=0; i<count; )
     {
       j = i;
@@ -117,21 +123,21 @@ mmap_regions(struct query *q, struct mmap_request *reqs, int count)
 	  return -1;
 	}
       if (q->debug & DEBUG_FETCH)
-	add_reply(".FFetching block %Lx+%Lx from fd %d", (long long) start, (long long) size, reqs[j].u.req.fd);
+	add_reply(".FFetching block %llx+%llx from fd %d", (long long) start, (long long) size, reqs[j].u.req.fd);
       x = sh_mmap((void *) q->last_mapping, size, PROT_READ, MAP_FIXED | MAP_SHARED, reqs[j].u.req.fd, start);
       if (x == MAP_FAILED)
 	{
 	  log(L_ERROR, "mmap failed: %m");
 	  return -1;
 	}
-      addr = (addr_int_t) q->last_mapping;
+      addr = (uintptr_t) q->last_mapping;
       q->last_mapping += size;
       total_size += size;
       while (j < i)
 	{
-	  addr_int_t s = addr + (reqs[j].u.req.start - start);
-	  addr_int_t e = addr + (reqs[j].u.req.end - start);
-	  DBG("Map fd %d block %d @%Lx-%Lx -> mem @%lx-%lx (join %d)", reqs[j].u.req.fd, j, (long long) reqs[j].u.req.start, (long long) reqs[j].u.req.end, s, e, i);
+	  uintptr_t s = addr + (reqs[j].u.req.start - start);
+	  uintptr_t e = addr + (reqs[j].u.req.end - start);
+	  DBG("Map fd %d block %d @%llx-%llx -> mem @%lx-%lx (join %d)", reqs[j].u.req.fd, j, (long long) reqs[j].u.req.start, (long long) reqs[j].u.req.end, s, e, i);
 	  reqs[j].u.map.start = (void *) s;
 	  reqs[j].u.map.end = (void *) e;
 	  j++;
