@@ -82,6 +82,8 @@ prof_t *
 profiler_switch(prof_t *p)
 {
   prof_t *o = profiler_current;
+  if (p == o)
+    return o;
   if (p)
     {
       if (o)
@@ -112,7 +114,7 @@ format_expr(struct query *q, struct expr *e, byte *buf, byte *bend, uns lastop,
     case EX_MATCH:
       if (buf + 32 + strlen(e->u.match.word) > bend)
 	return NULL;
-      buf += sprintf(buf, "\"%s\"[T%08x t%d S%d W%d A%d M%d P%d Y%d X%Ld]",
+      buf += sprintf(buf, "\"%s\"[T%08x t%d S%d W%d A%d M%d P%d Y%d X%lld]",
 		     e->u.match.word,
 		     e->u.match.type_mask,
 		     e->u.match.is_string,
@@ -122,7 +124,7 @@ format_expr(struct query *q, struct expr *e, byte *buf, byte *bend, uns lastop,
 		     e->u.match.o.morphing,
 		     e->u.match.o.spelling,
 		     e->u.match.o.synonyming,
-		     e->u.match.o.syn_expand);
+		     (long long) e->u.match.o.syn_expand);
       if (e->u.match.next_simple && mode)
 	{
 	  *buf++ = '.';
@@ -130,13 +132,13 @@ format_expr(struct query *q, struct expr *e, byte *buf, byte *bend, uns lastop,
 	}
       break;
     case EX_OPTIONS:
-      buf += sprintf(buf, "OPTIONS(W=%d A=%d M=%d P=%d Y=%d X=%Ld T=%08x ",
+      buf += sprintf(buf, "OPTIONS(W=%d A=%d M=%d P=%d Y=%d X=%lld T=%08x ",
 		     e->u.options.o.weight,
 		     e->u.options.o.accent_mode,
 		     e->u.options.o.morphing,
 		     e->u.options.o.spelling,
 		     e->u.options.o.synonyming,
-		     e->u.options.o.syn_expand,
+		     (long long) e->u.options.o.syn_expand,
 		     e->u.options.o.default_word_types);
       buf = format_expr(q, e->u.options.inside, buf, bend, EX_OPTIONS, mode);
       if (!buf || buf + 2 >= bend)
@@ -179,6 +181,12 @@ format_expr(struct query *q, struct expr *e, byte *buf, byte *bend, uns lastop,
     case EX_PHRASE:
       buf += sprintf(buf, "P(%d)", e->u.phrase.p->index);
       break;
+    case EX_IMAGE_SIM_MATCH:
+      buf = image_sim_match_format(&e->u.image_sim_match, buf, bend);
+      break;
+    case EX_IMAGE_SIM:
+      buf = image_sim_format(e->u.image_sim, buf, bend);
+      break;
     default:
       ASSERT(0);
     }
@@ -259,6 +267,9 @@ propagate_opts(struct expr *e, struct options *o)
       return e;
     case EX_ANY:
       return e;
+    case EX_IMAGE_SIM_MATCH:
+      image_sim_match_propagate_opts(&e->u.image_sim_match, o);
+      return e;
     default:
       ASSERT(0);
       return e;
@@ -280,7 +291,7 @@ format_norm_query(struct query *q, byte *buf, uns size)
 
   buf += sprintf(buf, "[D%d S%llx M%d A%d P%d T%d O%d a%d G%d,%d X%x] ",
 		 q->debug,
-		 q->site_hash,
+		 (long long) q->site_hash,
 		 q->site_max,
 		 q->allow_approx,
 		 q->partial_answers,
@@ -507,6 +518,8 @@ analyse_query(struct query *q, struct expr *e)
       if (e->u.op.l != ll)
 	return new_op(e->type, ll, NULL);
       return e;
+    case EX_IMAGE_SIM_MATCH:
+      return image_sim_match_analyse(q, e);
     default:
       return e;
     }
@@ -562,7 +575,7 @@ debug_words(struct query *q)
   for (uns i=0; i<q->nwords; i++)
     {
       struct word *w = q->words[i];
-      add_reply(".W%d: <%s> class=%08x is_string=%d outer=%d wei=%d acc=%d mor=%d spl=%d syn=%d sex=%Lx stat=%d docs=%d vars=%d wcl=%d uc=%d hc=%d wild=%d",
+      add_reply(".W%d: <%s> class=%08x is_string=%d outer=%d wei=%d acc=%d mor=%d spl=%d syn=%d sex=%llx stat=%d docs=%d vars=%d wcl=%d uc=%d hc=%d wild=%d",
 		i,
 		w->word,
 		w->type_mask,
@@ -573,7 +586,7 @@ debug_words(struct query *q)
 		w->options.morphing,
 		w->options.spelling,
 		w->options.synonyming,
-		w->options.syn_expand,
+		(long long) w->options.syn_expand,
 		w->status,
 		w->doc_count,
 		w->var_count,
@@ -737,6 +750,8 @@ shortcut(struct query *q, struct expr *e)
       return -1;
     case EX_PHRASE:
       return ((e->u.phrase.p->boolean_id < 0) ? -1 :0);
+    case EX_IMAGE_SIM:
+      return 0;
     default:
       ASSERT(0);
     }
@@ -803,6 +818,10 @@ map_bool(struct query *q, struct expr *x, u32 *Z)
     case EX_WORD:
       ASSERT(x->u.word.w->use_count);
       i = x->u.word.w->boolean_id;
+      goto got_id;
+    case EX_IMAGE_SIM:
+      i = x->u.image_sim->boolean_id;
+      /* fall-thru */
     got_id:
       if (i < 0)
 	memset(Z, 0, bool_bytes);
@@ -911,6 +930,7 @@ optimistic_expr(struct expr *x)
       return x;
     case EX_PHRASE:
     case EX_NOT:
+    case EX_IMAGE_SIM:
       return NULL;
     case EX_ANY:
     case EX_NONE:
@@ -959,6 +979,8 @@ simplified_eval_expr(struct expr *x, uns bool_mask)
 	return ((bool_mask & (1 << x->u.word.w->boolean_id)) ? 1 : -1);
     case EX_PHRASE:
       return ((bool_mask & (1 << x->u.phrase.p->boolean_id)) ? 1 : -1);
+    case EX_IMAGE_SIM:
+      return ((bool_mask & (1 << x->u.image_sim->boolean_id)) ? 1 : -1);
     case EX_ANY:
       return 1;
     case EX_NONE:
@@ -1036,10 +1058,10 @@ show_query_stats(struct query *q)
 		qq--;
 		PUT_UTF8(qq, 0xa0);
 	      }
-	  add_reply("W%s %d %d %d %d %d%d%08x%d%d%d%Lx", disp, w->var_count, w->ref_total_len / 1024,
+	  add_reply("W%s %d %d %d %d %d%d%08x%d%d%d%llx", disp, w->var_count, w->ref_total_len / 1024,
 		    (q->results->status ? ((w->boolean_id < 0) ? -1 : 0) : (int)w->doc_count),
 		    w->status, w->is_string, w->options.accent_mode, w->type_mask,
-		    w->options.morphing, w->options.spelling, w->options.synonyming, w->options.syn_expand);
+		    w->options.morphing, w->options.spelling, w->options.synonyming, (long long) w->options.syn_expand);
 	}
     }
   for (uns i=0; i<q->nphrases; i++)
@@ -1054,7 +1076,7 @@ show_query_stats(struct query *q)
 
 /*** Caching of interim results ***/
 
-#define INTERIMS(x) x(words) x(nwords) x(phrases) x(nphrases) x(nears) x(nnears)
+#define INTERIMS(x) x(words) x(nwords) x(phrases) x(nphrases) x(nears) x(nnears) x(nimage_sims) x(image_sims)
 
 static struct expr *
 copy_expr(struct mempool *pool, struct expr *e)
@@ -1076,6 +1098,7 @@ copy_expr(struct mempool *pool, struct expr *e)
     case EX_NONE:
     case EX_WORD:
     case EX_PHRASE:
+    case EX_IMAGE_SIM:
       /* The literal copy done above is enough */
       break;
     default:
@@ -1147,6 +1170,8 @@ eval_query_db(struct query *q)
   q->nphrases = 0;
   q->nears = mp_alloc_zero(r->pool, sizeof(struct phrase *) * max_nears);
   q->nnears = 0;
+  q->image_sims = mp_alloc_zero(r->pool, sizeof(struct image_sim *) * max_image_sims);
+  q->nimage_sims = 0;
   init_stats(q, &q->stats);
 
   q->age_min = convert_age(r->create_time - MIN(r->create_time,q->age_raw_min), q->dbase->params->ref_time);
@@ -1185,6 +1210,12 @@ eval_query(struct query *q, struct results *r)
   struct reply *first_error = NULL;
   uns first_err_code = 0;
 
+  if (!images_eval(q))
+    {
+      memory_flush(q);
+      return;
+    }
+  
   r->status = 0;
   q->current_reply_buf = &r->reply_header;
   clist_init(&r->interims);
@@ -1311,7 +1342,7 @@ process_query(struct query *q)
   current_query = q;
   init_query(q);
   add_reply("V" SHER_VER);
-  add_reply("I%Lx", (long long) incarnation);
+  add_reply("I%llx", (long long) incarnation);
 
   profiler_init();
   profiler_switch(&prof_analyse);
@@ -1336,6 +1367,7 @@ process_query(struct query *q)
 
   if (check_result_set(q))
     return;
+  prefetch_results(q);
 
   debug_expr(q, q->expr, 'I');
   process_options(q);
