@@ -8,6 +8,7 @@
 
 #undef LOCAL_DEBUG
 #undef DEBUG_HEAP
+#undef DEBUG_SLICES
 
 #include "sherlock/sherlock.h"
 #include "lib/mempool.h"
@@ -15,6 +16,7 @@
 #include "lib/heap.h"
 #include "lib/prefetch.h"
 #include "lib/unicode.h"
+#include "lib/threads.h"
 #include "sherlock/index.h"
 #include "indexer/params.h"
 #include "search/sherlockd.h"
@@ -23,6 +25,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <alloca.h>
+#include <pthread.h>
 
 #ifdef DEBUG_HEAP
 #define HDBG(x...) log(L_DEBUG, x)
@@ -30,6 +33,12 @@
 #define HDBG(x...) do { } while(0)
 #endif
 
+#ifdef DEBUG_SLICES
+#define SDBG(x...) log(L_DEBUG, x)
+#else
+#define SDBG(x...) do { } while(0)
+#endif
+
 /* Declare all internal data structures and emit code for dealing with them */
 #define R_EMIT_CODE
 #include "search/refs.h"
@@ -37,19 +46,6 @@
 #define INFTY 1000000		/* All Q's and weights are between -INFTY and INFTY */
 #define PEN_INFTY 1024		/* The same for word bonuses and penalties */
 
-/*
- * Our internal representation of reference chains.
- */
-
-struct chain {
-  byte *pos;
-  u32 lang_mask;		/* Extracted from struct variant */
-  byte noaccent_only;
-  byte penalty;
-  byte word_index;		/* Which word does this chain belong to */
-  byte bool_index;		/* And its boolean ID for quick lookup */
-};
-
 /* The ref_heap is used for merging of reference chains on oid level */
 
 struct ref_heap_entry {
@@ -69,17 +65,19 @@ struct chain_match {
 
 /* Allocation of buffers */
 
-struct ref_buffers ref_buffers;
+struct ref_buffers *ref_buffers;
 
 static void
 init_ref_buffers(struct ref_buffers *b)
 {
   bb_init(&b->ref_heap);
-  bb_init(&b->chains);
+  bb_init(&b->raw_chains);
+  bb_init(&b->sliced_chains);
   bb_init(&b->matched_chains);
   bb_init(&b->fulltext_words);
   b->trail_buf_size = 1024;
   b->trail_buf = xmalloc(b->trail_buf_size * sizeof(struct trail_entry));
+  bb_init(&b->image_sims_chains);
 }
 
 void *
@@ -165,9 +163,8 @@ map_chains(struct ref_context *c)
 	if (v->refchain_len)
 	  n++;
     }
-  c->chains = ref_buf_alloc(&c->buffers->chains, n * sizeof(struct chain));
-  c->num_chains = n;
-  q->stat_num_chains = n;
+  c->num_raw_chains = q->stat_num_chains = n;
+  c->raw_chains = ref_buf_alloc(&c->buffers->raw_chains, (c->num_raw_chains + q->nimage_sims) * sizeof(struct chain));
 
   struct mmap_request *mm = alloca(sizeof(struct mmap_request) * n);
   DBG("Reference chains requested:");
@@ -179,13 +176,13 @@ map_chains(struct ref_context *c)
       SLIST_FOR_EACH(struct variant *, v, w->variants)
 	if (v->refchain_len)
 	  {
-	    struct chain *ch = &c->chains[j];
+	    struct chain *ch = &c->raw_chains[j];
 	    ch->lang_mask = v->lang_mask;
 	    ch->noaccent_only = v->noaccent_only;
 	    ch->penalty = v->penalty;
 	    ch->word_index = i;
 	    ch->bool_index = w->boolean_id;
-	    DBG("\t%d: @%Lx+%x word=%d bool=%d nonacc=%d pen=%d lmask=%08x", j, (long long)v->refchain_start,
+	    DBG("\t%d: @%llx+%x word=%d bool=%d nonacc=%d pen=%d lmask=%08x", j, (long long)v->refchain_start,
 		  v->refchain_len, ch->word_index, ch->bool_index, ch->noaccent_only, ch->penalty, ch->lang_mask);
 	    mm[j].u.req.fd = q->dbase->fd_refs;
 	    mm[j].u.req.start = v->refchain_start;
@@ -205,7 +202,60 @@ map_chains(struct ref_context *c)
   for (uns i=0; i<n; i++)
     {
       uns j = mm[i].userdata;
-      c->chains[j].pos = mm[i].u.map.start;
+      c->raw_chains[j].pos = mm[i].u.map.start;
+    }
+}
+
+static void
+select_slices_trivial(struct ref_context *c)
+{
+  /*
+   * Set up chain pointers if the index contains only a single slice.
+   * I.e., just use the chains as they are.
+   */
+  c->chains = c->raw_chains;
+  c->num_chains = c->num_raw_chains;
+}
+
+static void
+select_slices(struct ref_context *c, uns slice)
+{
+  /*
+   * Set up chain pointers in the given ref context to point to the beginning of the
+   * selected slice, filtering out empty slices.
+   */
+  c->chains = ref_buf_alloc(&c->buffers->sliced_chains, c->num_raw_chains * sizeof(struct chain));
+  c->num_chains = 0;
+  uns num_slices = c->dbase->params->num_slices;
+  DBG("Selecting slice #%d:", slice);
+  for (uns i=0; i<c->num_raw_chains; i++)
+    {
+      struct chain *orig = &c->raw_chains[i];
+      struct chain *new = &c->chains[c->num_chains];
+      byte *p = orig->pos;
+      uns slice_mask = *p++;
+      if (slice_mask & (1 << slice))
+	{
+	  uns pos = 0;
+	  for (uns j=0; j<num_slices; j++)
+	    if (slice_mask & (1 << j))
+	      {
+		slice_mask &= ~(1 << j);
+		if (slice_mask)
+		  {
+		    uns size;
+		    p = utf8_32_get(p, &size);
+		    if (j < slice)
+		      pos += size;
+		  }
+	      }
+	  DBG("\t%d: @%d", i, (uns)(p-orig->pos) + pos);
+	  *new = *orig;
+	  new->pos = p + pos;
+	  c->num_chains++;
+	}
+      else
+	DBG("\t%d: empty", i);
     }
 }
 
@@ -239,12 +289,13 @@ init_bare_ref_context(struct query *q, struct ref_buffers *buffers)
   for (uns i=0; i<q->nnears; i++)
     c->nears[i] = *q->nears[i];
   c->num_nears = q->nnears;
+  image_bare_ref_context_init(c);
 
   return c;
 }
 
 static struct ref_context *
-init_ref_context(struct query *q, struct ref_buffers *buffers)
+init_ref_context(struct query *q, struct ref_buffers *buffers, struct ref_context *clone)
 {
   /* Initialize a full ref context, including reference chains and result heaps. */
 
@@ -252,9 +303,18 @@ init_ref_context(struct query *q, struct ref_buffers *buffers)
   c->match_heap.max_matches = q->results->max_matches;
   c->match_heap.site_max = q->site_max;
   local_match_init(q->pool, &c->match_heap);
-  map_chains(c);
-  c->ref_heap = ref_buf_alloc(&c->buffers->ref_heap, sizeof(struct ref_heap_entry) * (c->num_chains+1));
-  c->matched_chains = ref_buf_alloc(&c->buffers->matched_chains, sizeof(struct chain_match) * (c->num_chains+1));
+
+  if (!clone)
+    map_chains(c);
+  else
+    {
+      c->raw_chains = clone->raw_chains;
+      c->num_raw_chains = clone->num_raw_chains;
+    }
+  image_ref_context_init(c, clone);
+  c->ref_heap = ref_buf_alloc(&c->buffers->ref_heap, sizeof(struct ref_heap_entry) * (c->num_raw_chains+1));
+  c->matched_chains = ref_buf_alloc(&c->buffers->matched_chains, sizeof(struct chain_match) * (c->num_raw_chains+1));
+
   return c;
 }
 
@@ -662,13 +722,8 @@ static inline void
 refs_card_record(struct ref_context *c, struct card_attr *attr, oid_t oid, int qq, uns static_weight_mul)
 {
   struct local_match_note *note = local_match_get_note(&c->match_heap);
-  if (static_weight_mul)
-    qq += attr->weight * doc_weight_scale * static_weight_mul;
-  else
-    {
-      qq += attr->weight * doc_weight_scale;
-      static_weight_mul = 1;
-    }
+  /* static_weight_mul can be zero for ANY or IMAGESIM query */
+  qq += attr->weight * doc_weight_scale * static_weight_mul;
   if (c->query->custom_sort_only)
     qq = 0;
   EXPLAIN(c, "Static weight %d x %d (Q=%d) => total Q=%d", attr->weight, static_weight_mul,
@@ -680,6 +735,12 @@ refs_card_record(struct ref_context *c, struct card_attr *attr, oid_t oid, int q
   local_match_insert(&c->match_heap, note);
 }
 
+static void UNUSED
+image_sim_explain_msg(byte *s UNUSED, void *param UNUSED)
+{
+  EXPLAIN(((struct ref_context *)param), "%s", s);
+}
+
 static void
 refs_card(struct ref_context *c, struct card_attr *attr, oid_t oid, u32 words_found)
 {
@@ -867,6 +928,100 @@ refs_card_any(struct ref_context *c, struct card_attr *attr, oid_t oid)
 }
 #endif
 
+static uns
+refs_slice_find_type_mask(byte **refchain, uns want_mask, uns want_all, uns *found_mask)
+{
+  byte *pos = *refchain;
+  uns card_id, found = 0;
+  while (card_id = GET_U32(pos) & 0x0fffffff)
+    {
+      uns len = get_chain_len(&pos);
+      byte *end = pos + len;
+      while (pos < end)
+        {
+	  uns x = *pos++, type, mask;
+	  if (x < 0xe0) /* Delta-encoded word types */
+	    {
+	      if (x < 0x80) /* 0tpp pppp */
+		type = x >> 6;
+	      else if (x < 0xc0) /* 10pp pttt +u8 */
+	        {
+		  type = x & 7;
+		  pos++;
+		}
+	      else /* 110p pttt +u16 */
+	        {
+		  type = x & 7;
+		  pos += 2;
+		}
+	    }
+	  else if (x < 0xf8) /* Meta types */
+	    {
+	      if (x < 0xf0) /* 1110 tttt wwpp pppp */
+	        {
+		  type = x & 0x0f;
+		  pos++;
+		}
+	      else /* 1111 0tww pppp pttt +u8 */
+	        {
+		  type = ((x & 4) << 1) | (*pos & 7);
+		  pos += 2;
+		}
+	      type += 16;
+	    }
+#ifdef CONFIG_32BIT_REFERENCES
+	  else if (x < 0xfc) /* 1111 10tt tppp pppp +u16 (large word delta) */
+	    {
+	      type = (x & 3) | ((*pos & 0x80) >> 5);
+	      pos += 3;
+	    }
+	  else if (x < 0xfe) /* 1111 110w wppp tttt +u16 (large meta absolute) */
+	    {
+	      type = (*pos & 0x0f) + 16;
+	      pos += 3;
+	    }
+#endif
+	  else
+	    ASSERT(0);
+	  if (mask = (1 << type) & want_mask)
+	    {
+	      found |= mask;
+	      if (!want_all)
+		break;
+	    }
+	}
+      if (found)
+	break;
+    }
+  *refchain = pos;
+  *found_mask = found;
+  return card_id;
+}
+
+uns
+refs_chain_find_type_mask(byte **refchain, uns want_mask, uns want_all, uns *found_mask, uns have_slices)
+{
+  if (!have_slices)
+    return refs_slice_find_type_mask(refchain, want_mask, want_all, found_mask);
+  else
+    {
+      byte *p = *refchain;
+      uns slice_mask = *p++, slice_count = 0, card_id = 0, size;
+      for (; slice_mask; slice_mask >>= 1)
+        if ((slice_mask & 1) && slice_count++)
+	  p = utf8_32_get(p, &size);
+      for (uns i = 0; i < slice_count; i++)
+        {
+	  card_id = refs_slice_find_type_mask(&p, want_mask, want_all, found_mask);
+	  if (card_id)
+	    break;
+	  p += 4;
+	}
+      *refchain = p;
+      return card_id;
+    }
+}
+
 static u32
 refs_chains(struct ref_context *c, struct card_attr *attr, struct chain_match *mch)
 {
@@ -890,7 +1045,7 @@ refs_chains(struct ref_context *c, struct card_attr *attr, struct chain_match *m
       uns (*meta_weight_array)[16][4] = w->meta_weight_array;
 
       uns type_mask = tweak_type_mask(w->type_mask, ch->lang_mask, ch->noaccent_only, lang_mask, is_accented);
-      DBG("\tRef chain %d: word %d, type_mask %x", ch-c->chains, word_index, type_mask);
+      DBG("\tRef chain %d: word %d, type_mask %x", (int)(ch-c->chains), word_index, type_mask);
 
       if (!(words_seen & (1 << word_index)))
 	{
@@ -1021,18 +1176,6 @@ refs_chains(struct ref_context *c, struct card_attr *attr, struct chain_match *m
   return words_seen;
 }
 
-static inline uns
-get_chain_len(byte **pp)
-{
-  byte *p = *pp;
-  uns len = GET_U32(p) >> 28;
-  p += 4;
-  if (!len)
-    p = utf8_32_get(p, &len);
-  *pp = p;
-  return len;
-}
-
 static void
 refs_go(struct ref_context *c)
 {
@@ -1052,7 +1195,7 @@ refs_go(struct ref_context *c)
   HEAP_INIT(struct ref_heap_entry, rheap, rcnt, REF_HEAP_LESS, REF_HEAP_SWAP);
 
 #ifdef CONFIG_ALLOW_ANY
-  oid_t last_oid = -1;
+  oid_t last_oid = 0;
 #endif
 
   while (rcnt > 0)
@@ -1078,14 +1221,15 @@ refs_go(struct ref_context *c)
 	  byte *p = ch->pos;
 	  uns len = get_chain_len(&p);
 
-	  DBG("\tFound in ref chain %d size %d", ch-c->chains, len);
-	  mch->chain = ch;
-	  mch->pos = p;
-	  mch->stop = p + len;
-	  mch++;
+	  DBG("\tFound in ref chain %d size %d", (int)(ch-c->chains), len);
+	    {
+	      mch->chain = ch;
+	      mch->pos = p;
+	      mch->stop = p + len;
+	      mch++;
+	    }
 	  bool |= (1 << ch->bool_index);
 
-	  /* FIXME: This is not cache friendly at all! Change the format of chains to make the accesses continuous */
 	  p += len;
 	  oid_t next_oid = GET_U32(p) & 0x0fffffff;
 	  if (next_oid)
@@ -1133,21 +1277,113 @@ refs_go(struct ref_context *c)
 #endif
 }
 
+static void
+process_refs_single(struct ref_context *c)
+{
+  uns num_slices = c->dbase->params->num_slices;
+  if (num_slices == 1)
+    {
+      SDBG("--> Processing the only slice");
+      select_slices_trivial(c);
+      refs_go(c);
+    }
+  else
+    {
+      for (uns i=0; i<num_slices; i++)
+	{
+	  SDBG("--> Processing slice %d", i);
+	  select_slices(c, i);
+	  if (c->num_chains)
+	    refs_go(c);
+	}
+    }
+  merge_ref_context(c);
+}
+
+static void *
+ref_thread(void *arg)
+{
+  struct ref_context *c = arg;
+
+  for (uns i=0; i < c->dbase->params->num_slices; i++)
+    if (c->thread_slice_mask & (1 << i))
+      {
+	SDBG("--> Thread #%d processing slice %d", c->thread_id, i);
+	select_slices(c, i);
+	refs_go(c);
+      }
+  SDBG("--> Thread #%d finished", c->thread_id);
+  return c;
+}
+
+static void
+process_refs_threaded(struct ref_context *c)
+{
+  uns num_slices = c->dbase->params->num_slices;
+  uns num_threads = MIN(num_slices, slice_threads);
+  pthread_t threads[num_threads];
+  struct ref_context *ctxt[num_threads];
+
+  pthread_attr_t attr;
+  if (pthread_attr_init(&attr) < 0 ||
+      pthread_attr_setstacksize(&attr, default_thread_stack_size) < 0)
+    ASSERT(0);
+
+  /*
+   * The pthread interface is pretty silly, because it doesn't allow to wait
+   * for an arbitrary thread. Therefore we don't try to balance the load and
+   * we assign a fixed set of slices to each thread instead.
+   */
+
+  for (uns i=0; i<num_threads; i++)
+    {
+      struct ref_context *tc = i ? init_ref_context(c->query, &ref_buffers[i], c) : c;
+      ctxt[i] = tc;
+      tc->thread_id = i;
+      tc->thread_slice_mask = 0;
+      for (uns s=i; s<num_slices; s+=num_threads)
+	tc->thread_slice_mask |= (1 << s);
+      SDBG("--> Thread #%d started with slice_mask=%08x", i, tc->thread_slice_mask);
+      if (pthread_create(&threads[i], &attr, ref_thread, tc) < 0)
+	die("Unable to create thread: %m");
+    }
+  for (uns i=0; i<num_threads; i++)
+    {
+      void *ret;
+      if (pthread_join(threads[i], &ret) < 0)
+	die("Unable to join thread: %m");
+      SDBG("--> Thread #%d joined", i);
+      ASSERT(ret == ctxt[i]);
+      merge_ref_context(ctxt[i]);
+    }
+
+  pthread_attr_destroy(&attr);
+}
+
 void
 process_refs(struct query *q)
 {
+  /* Initialize the main ref_context and map all chains */
   prof_t *oldp = profiler_switch(&prof_reff);
-  struct ref_context *c = init_ref_context(q, &ref_buffers);
+  struct ref_context *c = init_ref_context(q, &ref_buffers[0], NULL);
+
+  /* Process references, possibly in several passes if there are multiple slices */
   profiler_switch(&prof_refs);
-  refs_go(c);
+  if (slice_threads <= 1 || c->dbase->params->num_slices == 1)
+    process_refs_single(c);
+  else
+    process_refs_threaded(c);
+
   profiler_switch(oldp);
-  merge_ref_context(c);
 }
 
 void
 refs_init(void)
 {
-  init_ref_buffers(&ref_buffers);
+  ASSERT(slice_threads);
+  ref_buffers = xmalloc(slice_threads * sizeof(struct ref_buffers));
+  for (uns i=0; i<slice_threads; i++)
+    init_ref_buffers(&ref_buffers[i]);
 }
 
 void
