@@ -55,6 +55,7 @@ static void
 free_query(struct query *q)
 {
   clist_remove(&q->n);
+  prefetch_results_cleanup(q);
   memory_flush(q);
   mp_delete(q->pool);
 }
@@ -249,6 +250,8 @@ mainloop(int watch_fd)
   struct sockaddr_in addr;
   uns timeout;
 
+  cards_init_process();
+
   signal(SIGPIPE, SIG_IGN);
   signal(SIGALRM, watchdog_timeout);
   sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
@@ -479,6 +482,7 @@ main(int argc, char **argv)
   fulltext_init();
   memory_init();
   cards_init();
+  spell_init();
 #ifdef CUSTOM_INIT
   CUSTOM_INIT();
 #endif
