@@ -6,8 +6,10 @@
 
 #include "lib/clists.h"
 #include "lib/slists.h"
+#include "lib/bitarray.h"
 #include "sherlock/index.h"
 #include "indexer/sites.h"
+#include "search/images.h"
 
 #define PROFILE_TOD
 #include "lib/profile.h"
@@ -25,10 +27,11 @@
 
 extern byte *log_name, *status_name;
 extern uns log_incoming, log_rejected, log_requests, log_replies;
-extern uns port, listen_queue, connection_timeout, hydra_processes;
+extern uns port, listen_queue, connection_timeout, hydra_processes, slice_threads;
 extern byte *control_password;
 extern clist databases;
 extern clist spell_common_pairs;
+extern clist spell_phrases;
 extern uns num_matches, max_matches, cache_size, max_output_matches;
 extern uns max_words, max_word_matches, max_phrases, max_nears, max_bools;
 extern uns global_accent_mode, wildcard_asterisks, wildcard_qmarks, max_wildcard_zone, min_wildcard_prefix_len;
@@ -41,6 +44,7 @@ extern uns global_allow_approx, prox_penalty;
 extern int prox_limit;
 extern uns global_partial_answers, default_word_types, global_debug, global_sorting, global_sort_reverse;
 extern uns mem_map_zone_size, mem_map_elide_gaps, mem_map_prefetch;
+extern uns fetch_threads;
 extern uns query_watchdog, second_best_reduce;
 extern uns magic_complexes, magic_merge_words, magic_merge_classes;
 extern uns magic_keyphrases, magic_keyphrases_classes, magic_keyphrases_string_types, magic_keyphrases_bonus;
@@ -59,10 +63,12 @@ struct database {
     DB_PART_WORDS = 1,
     DB_PART_STRINGS = 2,
     DB_PART_PRINTS = 4,
-    DB_PART_BLACKLIST = 8
+    DB_PART_IMAGE_SIGNATURES = 8
   } parts;
   uns is_optional;
+  byte **blacklists;
   struct mempool *pool;			/* Contains all information local to this database */
+  bitarray_t dup_flags;
 
   /* Parameters */
   struct index_params *params;
@@ -95,6 +101,10 @@ struct database {
   /* Fingerprints */
   struct fastbuf *fb_card_prints;
 
+  /* Similar images */
+  int fd_image_signatures;
+  uns image_clusters_depth;
+  struct image_cluster *image_clusters;
 };
 
 struct spell_pair {
@@ -102,6 +112,12 @@ struct spell_pair {
   uns x, y;
 };
 
+struct spell_phrase {
+  cnode n;
+  byte *src, *dest;
+  uns penalty;
+};
+
 /*** Reply buffers: we keep fragments of the reply in them (reply.c) ***/
 
 struct reply {
@@ -151,15 +167,17 @@ enum expr_type {
   EX_NONE,				/* PIO  Logical 0 */
   EX_IGNORE,				/* .A.  Non-indexed word */
   EX_WORD,				/* ..O  Match a word */
-  EX_PHRASE				/* ..O  Match a phrase */
+  EX_PHRASE,				/* ..O  Match a phrase */
+  EX_IMAGE_SIM_MATCH,			/* PI.  Match a similar image */
+  EX_IMAGE_SIM,				/* ..O  A bit larger structure used in refs.c */
 };
 
 struct options {
   int weight;
-  sbyte accent_mode;
-  sbyte morphing;
-  sbyte spelling;
-  sbyte synonyming;
+  s8 accent_mode;
+  s8 morphing;
+  s8 spelling;
+  s8 synonyming;
   u64 syn_expand;
   uns default_word_types;		/* ~0U = unset */
 };
@@ -178,7 +196,7 @@ struct expr {
       byte *word;
       uns type_mask;			/* Can be ~0U (unset) before propagation of options */
       byte is_string;			/* Set after option propagation */
-      sbyte sense;			/* Matching sense: 1=YES, -1=NOT, 0=MAYBE */
+      s8 sense;				/* Matching sense: 1=YES, -1=NOT, 0=MAYBE */
       struct expr *next_simple;		/* Next in simple search chain */
     } match;
     struct {
@@ -191,6 +209,8 @@ struct expr {
     struct {
       struct phrase *p;
     } phrase;
+    struct image_sim_match image_sim_match;
+    struct image_sim *image_sim;
   } u;
 };
 
@@ -311,6 +331,8 @@ struct query {				/* Keep the most frequently used parts near the top */
   uns stat_num_chains;			/* Statistics: total number of reference chains seen */
   uns stat_len_chains;			/* Statistics: total length of reference chains seen */
   uns query_word_count;			/* The number of query words */
+  struct image_sim **image_sims;	/* Similar images matcher */
+  uns nimage_sims;
 
   /* Display parameters */
   enum out_mode out_mode;		/* Type of output requested */
@@ -330,7 +352,11 @@ struct query {				/* Keep the most frequently used parts near the top */
   char *profile_stats;
 
   /* Memory mappings */
-  addr_int_t last_mapping;
+  uintptr_t last_mapping;
+
+  /* Card prefetching */
+  struct card_fetch *card_fetches;	/* Running asynchronous card fetches */
+  uns ncard_fetches;
 
   /* Connection */
   sh_time_t established;		/* Time this connection was established */
@@ -359,6 +385,7 @@ struct query {				/* Keep the most frequently used parts near the top */
 #define DEBUG_WORDS		8	/* Debug processing of words */
 #define DEBUG_CARD_INFO		16	/* Show result notes and card attributes (used by mux, too) */
 #define DEBUG_FETCH		32	/* Show which parts of files are fetched */
+#define DEBUG_IMAGES		64	/* Debug processing of images */
 
 /*** Words and phrases ***/
 
@@ -471,9 +498,10 @@ struct cached_interims {		/* Cached preprocessed parts of query used when dumpin
   cnode n;
   struct database *db;			/* Which database is this related to */
   struct expr *expr;			/* Preprocessed expression */
-  struct word **words;			/* Words, phrases and nears */
+  struct word **words;			/* Words, phrases, nears and images */
   struct phrase **phrases, **nears;
-  uns nwords, nphrases, nnears;
+  struct image_sim **image_sims;
+  uns nwords, nphrases, nnears, nimage_sims;
 };
 
 /* query.c */
@@ -512,8 +540,12 @@ struct database *attr_to_db(struct card_attr *attr, oid_t *ooid);
 /* cards.c */
 
 void cards_init(void);
+void cards_init_process(void);
 int check_result_set(struct query *q);
 void show_results(struct query *q);
+void prefetch_results(struct query *q);
+void prefetch_results_cleanup(struct query *q);
+
 extern byte *wt_names[];
 extern byte *mt_names[];
 extern byte *st_names[];
@@ -532,9 +564,11 @@ void word_extract_variant(struct word *w, struct variant *v, byte *buf);
 void strings_init(struct database *db);
 struct expr *string_analyse(struct query *q, struct expr *e);
 void string_analyse_simple(struct query *q, clist *l);
+int string_db_find_refchain(struct query *q, struct database *db, byte *s, sh_off_t *refchain_start, uns *refchain_len);
 
 /* spell.c */
 
+void spell_init(void);
 void spell_check(struct query *q);
 
 /* memory.c */
@@ -549,7 +583,7 @@ struct mmap_request {
       void *start, *end;
     } map;
   } u;
-  addr_int_t userdata;
+  uintptr_t userdata;
 };
 
 void memory_init(void);
@@ -566,6 +600,7 @@ void refs_init(void);
 void process_refs(struct query *q);
 void query_init_refs(struct query *q);
 void query_finish_refs(struct query *q);
+uns refs_chain_find_type_mask(byte **refchain, uns want_mask, uns want_all, uns *found_mask, uns have_slices);
 
 /* cmds.c */
 
