@@ -10,6 +10,7 @@
 #include "lib/url.h"
 #include "lib/mempool.h"
 #include "lib/unicode.h"
+#include "lib/unaligned.h"
 #include "charset/unicat.h"
 #include "search/sherlockd.h"
 
@@ -76,44 +77,59 @@ string_split(struct query *q, struct expr *e, uns mask)
   return new_op(EX_OR, w1, w2);
 }
 
-static void
-string_lookup(struct query *q, struct word *w)
+int
+string_db_find_refchain(struct query *q, struct database *db, byte *s, sh_off_t *refchain_start, uns *refchain_len)
 {
-  struct database *db = q->dbase;
   struct fingerprint fp;
   uns hash;
   sh_off_t buck_start, buck_size;
   byte *bk, *bkend;
 
-  fingerprint(w->word, &fp);
+  fingerprint(s, &fp);
   hash = fp_hash(&fp) >> db->string_hash_order;
   buck_start = db->string_hash[hash];
   buck_size = db->string_hash[hash+1] - buck_start;
   if (!buck_size)
-    return;
+    return 0;
   buck_start *= sizeof(struct fingerprint) + BYTES_PER_O;
   buck_size *= sizeof(struct fingerprint) + BYTES_PER_O;
-  bk = mmap_region(q, db->fd_string_map, buck_start, buck_start + buck_size - 1);
+  bk = mmap_region(q, db->fd_string_map, buck_start, buck_start + buck_size);
   if (!bk)
-    return;
+    return 0;
   bkend = bk + buck_size;
+
   while (bk < bkend)
     {
       if (!memcmp(bk, &fp, sizeof(fp)))
 	{
-	  struct variant *v = mp_alloc_zero(current_query->results->pool, sizeof(*v));
-	  v->lang_mask = ~0U;
-	  v->flags = VF_QUERY;
 	  bk += sizeof(fp);
-	  v->refchain_start = GET_O(bk);
+	  *refchain_start = GET_O(bk);
 	  bk += BYTES_PER_O + sizeof(fp);
-	  v->refchain_len = GET_O(bk) - v->refchain_start;
-	  slist_add_tail(&w->variants, &v->n);
-	  w->var_count++;
-	  return;
+	  *refchain_len = GET_O(bk) - *refchain_start;
+	  return 1;
 	}
       bk += sizeof(fp) + BYTES_PER_O;
     }
+
+  return 0;
+}
+
+static void
+string_lookup(struct query *q, struct word *w)
+{
+  sh_off_t refchain_start;
+  uns refchain_len;
+
+  if (string_db_find_refchain(q, q->dbase, w->word, &refchain_start, &refchain_len))
+    {
+      struct variant *v = mp_alloc_zero(current_query->results->pool, sizeof(*v));
+      v->lang_mask = ~0U;
+      v->flags = VF_QUERY;
+      v->refchain_start = refchain_start;
+      v->refchain_len = refchain_len;
+      slist_add_tail(&w->variants, &v->n);
+      w->var_count++;
+    }
 }
 
 struct expr *
@@ -179,7 +195,7 @@ strings_init(struct database *db)
   if (db->fd_string_map < 0)
     die("Unable to open %s: %m", fn_map);
   sh_off_t len = sh_seek(db->fd_string_map, 0, SEEK_END);
-  if (len > (sh_off_t)0xffffffff * (sh_off_t)(sizeof(struct fingerprint) + BYTES_PER_O))
+  if ((u64)len > (u64)0xffffffff * (sizeof(struct fingerprint) + BYTES_PER_O))
     die("String map %s too large", fn_map);
   db->string_map_file_size = len;
   db->string_count = db->string_map_file_size / (sizeof(struct fingerprint) + BYTES_PER_O);
