@@ -10,9 +10,8 @@
 #include "sherlock/sherlock.h"
 #include "sherlock/object.h"
 #include "sherlock/attrset.h"
-#include "lib/conf.h"
 #include "lib/clists.h"
-#include "sherlock/conf-parse.h"
+#include "sherlock/conf.h"
 
 struct attr_node {
   cnode n;
