@@ -239,7 +239,9 @@ decode_attributes(byte *ptr, byte *end, struct odes *o, uns can_overwrite)
 }
 
 int
-buck2obj_parse(struct buck2obj_buf *buf, uns buck_type, uns buck_len, struct fastbuf *body, struct odes *o_hdr, uns *body_start, struct odes *o_body)
+buck2obj_parse(struct buck2obj_buf *buf, uns buck_type, uns buck_len, struct fastbuf *body,
+	       struct odes *o_hdr, uns *body_start, struct odes *o_body,
+	       uns allow_zero_copy)
 {
   struct obj_read_state st;
   if (buck_type <= BUCKET_TYPE_PLAIN)
@@ -247,12 +249,12 @@ buck2obj_parse(struct buck2obj_buf *buf, uns buck_type, uns buck_len, struct fas
     if (body_start)			// there is no header part
       *body_start = 0;
     obj_read_start(&st, o_hdr);
+    byte *b;
     // ignore empty lines and read until the end of the bucket
     sh_off_t end = btell(body) + buck_len;
-    byte buf[MAX_ATTR_SIZE];
-    while (btell(body) < end && bgets(body, buf, sizeof(buf)))
-      if (buf[0])
-	obj_read_attr(&st, buf[0], buf+1);
+    while (btell(body) < end && bgets_bb(body, &buf->bb, ~0U))
+      if ((b = buf->bb.ptr)[0])
+	obj_read_attr(&st, b[0], b+1);
     ASSERT(btell(body) == end);
     obj_read_end(&st);
   }
@@ -260,20 +262,20 @@ buck2obj_parse(struct buck2obj_buf *buf, uns buck_type, uns buck_len, struct fas
   {
     sh_off_t start = btell(body);
     sh_off_t end = start + buck_len;
-    byte buf[MAX_ATTR_SIZE];
+    byte *b;
     struct obj_read_state st;
     obj_read_start(&st, o_hdr);
-    while (btell(body) < end && bgets(body, buf, sizeof(buf)) && buf[0])
-      obj_read_attr(&st, buf[0], buf+1);
+    while (btell(body) < end && bgets_bb(body, &buf->bb, ~0U) && (b = buf->bb.ptr)[0])
+      obj_read_attr(&st, b[0], b+1);
     obj_read_end(&st);
     if (body_start)
       *body_start = btell(body) - start;
     else
     {
       obj_read_start(&st, o_body);
-      while (btell(body) < end && bgets(body, buf, sizeof(buf)))
-	if (buf[0])
-	  obj_read_attr(&st, buf[0], buf+1);
+      while (btell(body) < end && bgets_bb(body, &buf->bb, ~0U))
+	if ((b = buf->bb.ptr)[0])
+	  obj_read_attr(&st, b[0], b+1);
       ASSERT(btell(body) == end);
       obj_read_end(&st);
     }
@@ -306,8 +308,8 @@ buck2obj_parse(struct buck2obj_buf *buf, uns buck_type, uns buck_len, struct fas
     byte *ptr, *end;
     uns len = bdirect_read_prepare(body, &ptr);
     uns copied = 0;
-    if (len < buck_len
-    || (body->can_overwrite_buffer < 2 && buck_type == BUCKET_TYPE_V33))
+    if (len < buck_len ||
+	((body->can_overwrite_buffer < 2 || !allow_zero_copy) && buck_type == BUCKET_TYPE_V33))
     {
       /* Copy if the original buffer is too small.
        * If it is write-protected, copy it also if it is uncompressed.  */
@@ -361,32 +363,44 @@ buck2obj_parse(struct buck2obj_buf *buf, uns buck_type, uns buck_len, struct fas
 }
 
 struct odes *
-obj_read_bucket(struct buck2obj_buf *buf, struct mempool *pool, uns buck_type, uns buck_len, struct fastbuf *body, uns *body_start)
+obj_read_bucket(struct buck2obj_buf *buf, struct mempool *pool, uns buck_type, uns buck_len, struct fastbuf *body,
+		uns *body_start, uns allow_zero_copy)
 {
   struct odes *o = obj_new(pool);
-  if (buck2obj_parse(buf, buck_type, buck_len, body, o, body_start, o) < 0)
+  if (buck2obj_parse(buf, buck_type, buck_len, body, o, body_start, o, allow_zero_copy) < 0)
     return NULL;
   else
     return o;
 }
 
+static int
+obj_read_line(struct fastbuf *f, struct obj_read_state *st)
+{
+  byte *buf = bgets_stk(f);
+  if (buf)
+    {
+      if (!buf[0])
+	return 1;
+      obj_read_attr(st, buf[0], buf+1);
+      return -1;
+    }
+  else
+    return 0;
+}
+
 int
 obj_read(struct fastbuf *f, struct odes *o)
 {
-  byte buf[MAX_ATTR_SIZE];
   struct obj_read_state st;
   int rc = 0;
-
   obj_read_start(&st, o);
-  while (bgets(f, buf, sizeof(buf)))
-    {
-      if (!buf[0])
-	{
-	  rc = 1;
-	  break;
-	}
-      obj_read_attr(&st, buf[0], buf+1);
-    }
+  while ((rc = obj_read_line(f, &st)) < 0);
   obj_read_end(&st);
   return rc;
 }
+
+void
+default_obj_read_error(struct obj_read_state *st UNUSED, byte *msg)
+{
+  log(L_ERROR, "%s", msg);
+}
