@@ -1,8 +1,9 @@
 /*
  *	Sherlock Library -- Object Buckets
  *
- *	(c) 2001--2004 Martin Mares <mj@ucw.cz>
+ *	(c) 2001--2006 Martin Mares <mj@ucw.cz>
  *	(c) 2004--2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2007 Pavel Charvat <pchar@ucw.cz>
  *
  *	This software may be freely distributed and used according to the terms
  *	of the GNU Lesser General Public License.
@@ -13,6 +14,7 @@
 #include "sherlock/sherlock.h"
 #include "sherlock/bucket.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/lfs.h"
 #include "lib/conf.h"
 
@@ -20,28 +22,30 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/file.h>
+#include <sys/stat.h>
 #include <alloca.h>
 
-static int obuck_fd;
-static struct obuck_header obuck_hdr, obuck_create_hdr;
-static sh_off_t bucket_find_pos;
-static struct fastbuf *obuck_write_fb;
-
 /*** Configuration ***/
 
-byte *obuck_name = "not/configured";
-static uns obuck_io_buflen = 65536;
-static int obuck_shake_buflen = 1048576;
-static uns obuck_shake_security;
-static uns obuck_slurp_buflen = 65536;
+static struct obuck_params obuck_params = {
+  .io_buflen = 65536,
+  .shake_buflen = 1048576,
+  .shake_security = 0,
+  .slurp_buflen = 65536,
+  .prefetch_size = 8192,
+};
+
+byte *bucket_file_name = "db/objects";
+struct obuck bucket_file;
 
 static struct cf_section obuck_config = {
   CF_ITEMS {
-    CF_STRING("BucketFile", &obuck_name),
-    CF_UNS("BufSize", &obuck_io_buflen),
-    CF_INT("ShakeBufSize", &obuck_shake_buflen),
-    CF_UNS("ShakeSecurity", &obuck_shake_security),
-    CF_UNS("SlurpBufSize", &obuck_slurp_buflen),
+    CF_STRING("BucketFile", &bucket_file_name),
+    CF_UNS("BufSize", &obuck_params.io_buflen),
+    CF_INT("ShakeBufSize", &obuck_params.shake_buflen),
+    CF_UNS("ShakeSecurity", &obuck_params.shake_security),
+    CF_UNS("SlurpBufSize", &obuck_params.slurp_buflen),
+    CF_UNS("PrefetchSize", &obuck_params.prefetch_size),
     CF_END
   }
 };
@@ -54,9 +58,9 @@ static void CONSTRUCTOR obuck_init_config(void)
 /*** Internal operations ***/
 
 static void
-obuck_broken(char *msg, sh_off_t pos)
+obuck_broken(struct obuck *obuck, char *msg, sh_off_t pos)
 {
-  die("Object pool corrupted: %s (pos=%Lx)", msg, (long long) pos);
+  die("Object pool %s corrupted: %s (pos=%llx)", obuck->name, msg, (long long) pos);
 }
 
 /*
@@ -70,6 +74,7 @@ obuck_broken(char *msg, sh_off_t pos)
  *  Multiple read and scan locks can co-exist together.
  *  Scan locks can co-exist with an append lock.
  *  There can be at most one write/append lock at a time.
+ *  File handle position (and thus seek) is locked by write/append.
  *
  *  These lock types map to a pair of normal read-write locks which
  *  we represent as fcntl() locks on the first and second byte of the
@@ -84,8 +89,8 @@ obuck_broken(char *msg, sh_off_t pos)
  *	Scan		-		<read>
  */
 
-static inline void
-obuck_do_lock(int type, int start, int len)
+static void
+obuck_do_lock(struct obuck *obuck, int type, int start, int len)
 {
   struct flock fl;
 
@@ -93,56 +98,72 @@ obuck_do_lock(int type, int start, int len)
   fl.l_whence = SEEK_SET;
   fl.l_start = start;
   fl.l_len = len;
-  if (fcntl(obuck_fd, F_SETLKW, &fl) < 0)
+  if (fcntl(obuck->fd, F_SETLKW, &fl) < 0)
     die("fcntl lock: %m");
 }
 
-inline void
-obuck_lock_read(void)
+void
+obuck_lock_read(struct obuck *obuck)
 {
-  obuck_do_lock(F_RDLCK, 0, 2);
+  obuck_do_lock(obuck, F_RDLCK, 0, 2);
 }
 
-inline void
-obuck_lock_write(void)
+void
+obuck_lock_write(struct obuck *obuck)
 {
-  obuck_do_lock(F_WRLCK, 0, 2);
+  obuck_do_lock(obuck, F_WRLCK, 0, 2);
 }
 
-static inline void
-obuck_lock_append(void)
+static void
+obuck_lock_append(struct obuck *obuck)
 {
-  obuck_do_lock(F_WRLCK, 0, 1);
+  obuck_do_lock(obuck, F_WRLCK, 0, 1);
 }
 
-static inline void
-obuck_lock_read_to_scan(void)
+void
+obuck_lock_scan(struct obuck *obuck)
 {
-  obuck_do_lock(F_UNLCK, 0, 1);
+  obuck_do_lock(obuck, F_RDLCK, 1, 1);
 }
 
-inline void
-obuck_unlock(void)
+void
+obuck_unlock(struct obuck *obuck)
 {
-  obuck_do_lock(F_UNLCK, 0, 2);
+  obuck_do_lock(obuck, F_UNLCK, 0, 2);
+}
+
+void
+obuck_unlock_scan(struct obuck *obuck)
+{
+  obuck_do_lock(obuck, F_UNLCK, 1, 1);
+}
+
+static void
+obuck_unlock_append(struct obuck *obuck)
+{
+  obuck_do_lock(obuck, F_UNLCK, 0, 1);
+}
+
+static void
+obuck_relock_read_to_scan(struct obuck *obuck)
+{
+  obuck_unlock_append(obuck);
 }
 
 /*** FastIO emulation ***/
 
 struct fb_bucket {
   struct fastbuf fb;
+  struct obuck *obuck;
   sh_off_t start_pos;
   uns bucket_size;
   byte buffer[0];
 };
 #define FB_BUCKET(f) ((struct fb_bucket *)(f)->is_fastbuf)
 
-static int obuck_fb_count;
-
 static void
 obuck_fb_close(struct fastbuf *f)
 {
-  obuck_fb_count--;
   xfree(f);
 }
 
@@ -167,23 +188,23 @@ obuck_fb_refill(struct fastbuf *f)
     }
   else
     size = datasize = bufsize;
-  int l = sh_pread(obuck_fd, f->buffer, size, pos);
+  int l = sh_pread(FB_BUCKET(f)->obuck->fd, f->buffer, size, pos);
   if (l < 0)
     die("Error reading bucket: %m");
   if ((unsigned) l != size)
-    obuck_broken("Short read", FB_BUCKET(f)->start_pos);
+    obuck_broken(FB_BUCKET(f)->obuck, "Short read", FB_BUCKET(f)->start_pos);
   f->bptr = f->buffer;
   f->bstop = f->buffer + datasize;
   f->pos += datasize;
   if (datasize < size)
     {
       if (GET_U32(f->buffer + size - 4) != OBUCK_TRAILER)
-	obuck_broken("Missing trailer", FB_BUCKET(f)->start_pos);
+	obuck_broken(FB_BUCKET(f)->obuck, "Missing trailer", FB_BUCKET(f)->start_pos);
     }
   return datasize;
 }
 
-static void
+static int
 obuck_fb_seek(struct fastbuf *f, sh_off_t pos, int whence)
 {
   ASSERT(whence == SEEK_SET || whence == SEEK_END);
@@ -191,6 +212,7 @@ obuck_fb_seek(struct fastbuf *f, sh_off_t pos, int whence)
     pos += FB_BUCKET(f)->bucket_size;
   ASSERT(pos >= 0 && pos <= (sh_off_t) FB_BUCKET(f)->bucket_size);
   f->pos = pos;
+  return 1;
 }
 
 static void
@@ -201,7 +223,7 @@ obuck_fb_spout(struct fastbuf *f)
 
   while (l)
     {
-      int z = sh_pwrite(obuck_fd, c, l, FB_BUCKET(f)->start_pos + sizeof(struct obuck_header) + f->pos);
+      int z = sh_pwrite(FB_BUCKET(f)->obuck->fd, c, l, FB_BUCKET(f)->start_pos + sizeof(struct obuck_header) + f->pos);
       if (z <= 0)
 	die("Error writing bucket: %m");
       f->pos += z;
@@ -214,109 +236,129 @@ obuck_fb_spout(struct fastbuf *f)
 /*** Exported functions ***/
 
 void
-obuck_init(int writeable)
+bucket_open(int writeable)
+{
+  obuck_init(&bucket_file, bucket_file_name, writeable);
+}
+
+void
+bucket_close(void)
+{
+  obuck_cleanup(&bucket_file);
+}
+
+void
+bucket_set_name(byte *name)
+{
+  CF_JOURNAL_VAR(bucket_file_name);
+  bucket_file_name = name;
+}
+
+void
+obuck_init(struct obuck *obuck, byte *name, int writeable)
 {
   sh_off_t size;
 
-  obuck_fd = sh_open(obuck_name, (writeable ? O_RDWR | O_CREAT : O_RDONLY), 0666);
-  if (obuck_fd < 0)
-    die("Unable to open bucket file %s: %m", obuck_name);
-  obuck_lock_read();
-  size = sh_seek(obuck_fd, 0, SEEK_END);
+  ASSERT(name && *name);
+  obuck->p = obuck_params;
+  obuck->name = name;
+  obuck->fd = sh_open(name, (writeable ? O_RDWR | O_CREAT : O_RDONLY), 0666);
+  if (obuck->fd < 0)
+    die("Unable to open bucket file %s: %m", obuck->name);
+  obuck_lock_read(obuck);
+  size = sh_seek(obuck->fd, 0, SEEK_END);
   if (size)
     {
       /* If the bucket pool is not empty, check consistency of its end */
       u32 check;
-      if (sh_pread(obuck_fd, &check, 4, size-4) != 4 ||
+      if (sh_pread(obuck->fd, &check, 4, size-4) != 4 ||
 	  check != OBUCK_TRAILER)
-	obuck_broken("Missing trailer of last object", size - 4);
+	obuck_broken(obuck, "Missing trailer of last object", size - 4);
     }
-  obuck_unlock();
+  obuck_unlock(obuck);
 }
 
 void
-obuck_cleanup(void)
+obuck_cleanup(struct obuck *obuck)
 {
-  close(obuck_fd);
-  if (obuck_fb_count)
-    log(L_ERROR, "Bug: Unbalanced bucket opens/closes: %d streams remain", obuck_fb_count);
-  if (obuck_write_fb)
+  close(obuck->fd);
+  if (obuck->write_fb)
     log(L_ERROR, "Bug: Forgot to close bucket write stream");
 }
 
 void
-obuck_sync(void)
+obuck_sync(struct obuck *obuck)
 {
-  if (obuck_write_fb)
-    bflush(obuck_write_fb);
-  fsync(obuck_fd);
+  if (obuck->write_fb)
+    bflush(obuck->write_fb);
+  fsync(obuck->fd);
 }
 
 static void
-obuck_get(oid_t oid)
+obuck_get(struct obuck *obuck, oid_t oid, struct obuck_context *ctx)
 {
-  bucket_find_pos = obuck_get_pos(oid);
-  if (sh_pread(obuck_fd, &obuck_hdr, sizeof(obuck_hdr), bucket_find_pos) != sizeof(obuck_hdr))
-    obuck_broken("Short header read", bucket_find_pos);
-  if (obuck_hdr.magic != OBUCK_MAGIC)
-    obuck_broken("Missing magic number", bucket_find_pos);
-  if (obuck_hdr.oid == OBUCK_OID_DELETED)
-    obuck_broken("Access to deleted bucket", bucket_find_pos);
-  if (obuck_hdr.oid != oid)
-    obuck_broken("Invalid backlink", bucket_find_pos);
+  ctx->pos = obuck_get_pos(oid);
+  if (sh_pread(obuck->fd, &ctx->hdr, sizeof(ctx->hdr), ctx->pos) != sizeof(ctx->hdr))
+    obuck_broken(obuck, "Short header read", ctx->pos);
+  if (ctx->hdr.magic != OBUCK_MAGIC)
+    obuck_broken(obuck, "Missing magic number", ctx->pos);
+  if (ctx->hdr.oid == OBUCK_OID_DELETED)
+    obuck_broken(obuck, "Access to deleted bucket", ctx->pos);
+  if (ctx->hdr.oid != oid)
+    obuck_broken(obuck, "Invalid backlink", ctx->pos);
 }
 
 void
-obuck_find_by_oid(struct obuck_header *hdrp)
+obuck_find_by_oid(struct obuck *obuck, struct obuck_context *ctx, uns flags)
 {
-  oid_t oid = hdrp->oid;
+  oid_t oid = ctx->hdr.oid;
 
   ASSERT(oid < OBUCK_OID_FIRST_SPECIAL);
-  obuck_lock_read();
-  obuck_get(oid);
-  obuck_unlock();
-  memcpy(hdrp, &obuck_hdr, sizeof(obuck_hdr));
+  if (!(flags & OBUCK_NO_LOCK))
+    obuck_lock_read(obuck);
+  obuck_get(obuck, oid, ctx);
+  if (!(flags & OBUCK_NO_LOCK))
+    obuck_unlock(obuck);
 }
 
 int
-obuck_find_first(struct obuck_header *hdrp, int full)
+obuck_find_first(struct obuck *obuck, struct obuck_context *ctx, uns flags)
 {
-  bucket_find_pos = 0;
-  obuck_hdr.magic = 0;
-  return obuck_find_next(hdrp, full);
+  ctx->pos = 0;
+  ctx->hdr.magic = 0;
+  return obuck_find_next(obuck, ctx, flags);
 }
 
 int
-obuck_find_next(struct obuck_header *hdrp, int full)
+obuck_find_next(struct obuck *obuck, struct obuck_context *ctx, uns flags)
 {
   int c;
 
   for(;;)
     {
-      if (obuck_hdr.magic)
-	bucket_find_pos += obuck_bucket_size(obuck_hdr.length);
-      obuck_lock_read();
-      c = sh_pread(obuck_fd, &obuck_hdr, sizeof(obuck_hdr), bucket_find_pos);
-      obuck_unlock();
+      if (ctx->hdr.magic)
+	ctx->pos += obuck_bucket_size(ctx->hdr.length);
+      if (!(flags & OBUCK_NO_LOCK))
+        obuck_lock_read(obuck);
+      c = sh_pread(obuck->fd, &ctx->hdr, sizeof(ctx->hdr), ctx->pos);
+      if (!(flags & OBUCK_NO_LOCK))
+        obuck_unlock(obuck);
       if (!c)
 	return 0;
-      if (c != sizeof(obuck_hdr))
-	obuck_broken("Short header read", bucket_find_pos);
-      if (obuck_hdr.magic != OBUCK_MAGIC)
-	obuck_broken("Missing magic number", bucket_find_pos);
-      if (obuck_hdr.oid != OBUCK_OID_DELETED || full)
-	{
-	  memcpy(hdrp, &obuck_hdr, sizeof(obuck_hdr));
-	  return 1;
-	}
+      if (c != sizeof(ctx->hdr))
+	obuck_broken(obuck, "Short header read", ctx->pos);
+      if (ctx->hdr.magic != OBUCK_MAGIC)
+	obuck_broken(obuck, "Missing magic number", ctx->pos);
+      if (ctx->hdr.oid != OBUCK_OID_DELETED || (flags & OBUCK_FULL))
+	return 1;
     }
 }
 
 struct fastbuf *
-obuck_fetch(void)
+obuck_fetch(struct obuck *obuck, struct obuck_context *ctx)
 {
   struct fastbuf *b;
-  uns official_buflen = ALIGN(MIN(obuck_hdr.length, obuck_io_buflen), OBUCK_ALIGN);
+  uns official_buflen = ALIGN_TO(MIN(ctx->hdr.length, obuck->p.io_buflen), OBUCK_ALIGN);
   uns real_buflen = official_buflen + OBUCK_ALIGN;
 
   b = xmalloc(sizeof(struct fb_bucket) + real_buflen);
@@ -330,41 +372,89 @@ obuck_fetch(void)
   b->close = obuck_fb_close;
   b->config = NULL;
   b->can_overwrite_buffer = 2;
-  FB_BUCKET(b)->start_pos = bucket_find_pos;
-  FB_BUCKET(b)->bucket_size = obuck_hdr.length;
-  obuck_fb_count++;
+  FB_BUCKET(b)->start_pos = ctx->pos;
+  FB_BUCKET(b)->bucket_size = ctx->hdr.length;
+  FB_BUCKET(b)->obuck = obuck;
   return b;
 }
 
-oid_t
-obuck_predict_last_oid(void)
+struct obuck_fetch_oid_fb {
+  struct fastbuf fb;
+  struct obuck_header buf;
+};
+
+static void
+obuck_fetch_oid_fb_close(struct fastbuf *b)
 {
-  sh_off_t size = sh_seek(obuck_fd, 0, SEEK_END);
-  return (oid_t)(size >> OBUCK_SHIFT);
+  xfree(b);
 }
 
 struct fastbuf *
-obuck_create(void)
+obuck_fetch_oid(struct obuck *obuck, struct obuck_context *ctx, uns flags)
 {
-  ASSERT(!obuck_write_fb);
+  oid_t oid = ctx->hdr.oid;
+  ASSERT(oid < OBUCK_OID_FIRST_SPECIAL);
+  sh_off_t pos = ctx->pos = obuck_get_pos(oid);
+  uns prefetch = MAX(sizeof(struct obuck_header), obuck->p.prefetch_size);
+  struct obuck_fetch_oid_fb *b = xmalloc(sizeof(*b) - sizeof(struct obuck_header) + prefetch);
+  bzero(b, sizeof(*b));
+  if (!(flags & OBUCK_NO_LOCK))
+    obuck_lock_read(obuck);
+  sh_off_t size = sh_pread(obuck->fd, (byte *)&b->buf, prefetch, pos);
+  if (size < (uns)sizeof(b->buf))
+    obuck_broken(obuck, "Short header read", pos);
+  if (b->buf.magic != OBUCK_MAGIC)
+    obuck_broken(obuck, "Missing magic number", pos);
+  if (b->buf.oid == OBUCK_OID_DELETED)
+    obuck_broken(obuck, "Access to deleted bucket", pos);
+  if (b->buf.oid != oid)
+    obuck_broken(obuck, "Invalid backlink", pos);
+  ctx->hdr = b->buf;
+  uns size2 = b->buf.length + 4 + ((uns)(OBUCK_ALIGN - sizeof(struct obuck_header) - b->buf.length - 4) & (OBUCK_ALIGN - 1));
+  b = xrealloc(b, sizeof(*b) + size2);
+  sh_off_t rest = size2 + sizeof(struct obuck_header) - size;
+  if (rest > 0 && sh_pread(obuck->fd, ((byte *)&b->buf) + size, rest, pos + size) != rest)
+    obuck_broken(obuck, "Short read", pos);
+  if (!(flags & OBUCK_NO_LOCK))
+    obuck_unlock(obuck);
+  if (GET_U32((byte *)&b->buf + sizeof(struct obuck_header) + size2 - 4) != OBUCK_TRAILER)
+    obuck_broken(obuck, "Missing trailer", pos);
+  fbbuf_init_read(&b->fb, (byte *)(&b->buf + 1), b->buf.length, 1);
+  b->fb.name = "bucket-fetch";
+  b->fb.close = obuck_fetch_oid_fb_close;
+  return &b->fb;
+}
 
-  obuck_lock_append();
-  sh_off_t start = sh_seek(obuck_fd, 0, SEEK_END);
+oid_t
+obuck_predict_last_oid(struct obuck *obuck)
+{
+  sh_stat_t buf;
+  sh_fstat(obuck->fd, &buf);
+  return (oid_t)(buf.st_size >> OBUCK_SHIFT);
+}
+
+struct fastbuf *
+obuck_create(struct obuck *obuck)
+{
+  ASSERT(!obuck->write_fb);
+
+  obuck_lock_append(obuck);
+  sh_off_t start = sh_seek(obuck->fd, 0, SEEK_END);
   if (start & (OBUCK_ALIGN - 1))
-    obuck_broken("Misaligned file", start);
+    obuck_broken(obuck, "Misaligned file", start);
   if (start >= ((sh_off_t) OBUCK_OID_FIRST_SPECIAL << OBUCK_SHIFT))
-    obuck_broken("Maximum bucket file size exceeded", start);
-  obuck_create_hdr.magic = OBUCK_INCOMPLETE_MAGIC;
-  obuck_create_hdr.oid = start >> OBUCK_SHIFT;
-  obuck_create_hdr.length = 0;
-  obuck_create_hdr.type = ~0U;
+    obuck_broken(obuck, "Maximum bucket file size exceeded", start);
+  obuck->create_hdr.magic = OBUCK_INCOMPLETE_MAGIC;
+  obuck->create_hdr.oid = start >> OBUCK_SHIFT;
+  obuck->create_hdr.length = 0;
+  obuck->create_hdr.type = ~0U;
 
-  struct fastbuf *b = xmalloc(sizeof(struct fb_bucket) + obuck_io_buflen);
-  obuck_write_fb = b;
+  struct fastbuf *b = xmalloc(sizeof(struct fb_bucket) + obuck->p.io_buflen);
+  obuck->write_fb = b;
   b->buffer = FB_BUCKET(b)->buffer;
   b->bptr = b->bstop = b->buffer;
-  b->bufend = b->buffer + obuck_io_buflen;
-  b->pos = -(int)sizeof(obuck_create_hdr);
+  b->bufend = b->buffer + obuck->p.io_buflen;
+  b->pos = -(int)sizeof(obuck->create_hdr);
   b->name = "bucket-write";
   b->refill = NULL;
   b->spout = obuck_fb_spout;
@@ -374,63 +464,62 @@ obuck_create(void)
   b->can_overwrite_buffer = 0;
   FB_BUCKET(b)->start_pos = start;
   FB_BUCKET(b)->bucket_size = 0;
-  bwrite(b, &obuck_create_hdr, sizeof(obuck_create_hdr));
+  FB_BUCKET(b)->obuck = obuck;
+  bwrite(b, &obuck->create_hdr, sizeof(obuck->create_hdr));
 
   return b;
 }
 
 void
-obuck_create_end(struct fastbuf *b, uns type, struct obuck_header *hdrp)
+obuck_create_end(struct obuck *obuck, struct fastbuf *b, uns type, struct obuck_header *hdrp)
 {
-  ASSERT(b == obuck_write_fb);
-  obuck_write_fb = NULL;
+  ASSERT(b == obuck->write_fb);
+  obuck->write_fb = NULL;
 
-  obuck_create_hdr.magic = OBUCK_MAGIC;
-  obuck_create_hdr.length = btell(b);
-  obuck_create_hdr.type = type;
-  int pad = (OBUCK_ALIGN - sizeof(obuck_create_hdr) - obuck_create_hdr.length - 4) & (OBUCK_ALIGN - 1);
+  obuck->create_hdr.magic = OBUCK_MAGIC;
+  obuck->create_hdr.length = btell(b);
+  obuck->create_hdr.type = type;
+  int pad = (OBUCK_ALIGN - sizeof(obuck->create_hdr) - obuck->create_hdr.length - 4) & (OBUCK_ALIGN - 1);
   while (pad--)
     bputc(b, 0);
   bputl(b, OBUCK_TRAILER);
   bflush(b);
-  ASSERT(!((FB_BUCKET(b)->start_pos + sizeof(obuck_create_hdr) + b->pos) & (OBUCK_ALIGN - 1)));
-  if (sh_pwrite(obuck_fd, &obuck_create_hdr, sizeof(obuck_create_hdr), FB_BUCKET(b)->start_pos) != sizeof(obuck_create_hdr))
+  ASSERT(!((FB_BUCKET(b)->start_pos + sizeof(obuck->create_hdr) + b->pos) & (OBUCK_ALIGN - 1)));
+  if (sh_pwrite(obuck->fd, &obuck->create_hdr, sizeof(obuck->create_hdr), FB_BUCKET(b)->start_pos) != sizeof(obuck->create_hdr))
     die("Bucket header update failed: %m");
-  obuck_unlock();
-  memcpy(hdrp, &obuck_create_hdr, sizeof(obuck_create_hdr));
+  obuck_unlock_append(obuck);
+  memcpy(hdrp, &obuck->create_hdr, sizeof(obuck->create_hdr));
   xfree(b);
 }
 
 void
-obuck_delete(oid_t oid)
+obuck_delete(struct obuck *obuck, oid_t oid)
 {
-  obuck_lock_write();
-  obuck_get(oid);
-  obuck_hdr.oid = OBUCK_OID_DELETED;
-  sh_pwrite(obuck_fd, &obuck_hdr, sizeof(obuck_hdr), bucket_find_pos);
-  obuck_unlock();
+  obuck_lock_write(obuck);
+  struct obuck_context ctx;
+  obuck_get(obuck, oid, &ctx);
+  ctx.hdr.oid = OBUCK_OID_DELETED;
+  sh_pwrite(obuck->fd, &ctx.hdr, sizeof(ctx.hdr), ctx.pos);
+  obuck_unlock(obuck);
 }
 
 /*** Fast reading of the whole pool ***/
 
-static struct fastbuf *obuck_rpf;
-static uns slurp_remains;
-static sh_off_t slurp_start, slurp_current, slurp_end;
-
 static int
 obuck_slurp_refill(struct fastbuf *f)
 {
-  if (!slurp_remains)
+  struct obuck *obuck = SKIP_BACK(struct obuck, limiter, f);
+  if (!obuck->slurp_remains)
     return 0;
-  uns l = bdirect_read_prepare(obuck_rpf, &f->buffer);
+  uns l = bdirect_read_prepare(obuck->rpf, &f->buffer);
   if (!l)
-    obuck_broken("Incomplete object", slurp_start);
-  l = MIN(l, slurp_remains);
+    obuck_broken(obuck, "Incomplete object", obuck->slurp_start);
+  l = MIN(l, obuck->slurp_remains);
   /* XXX: This probably should be bdirect_read_commit_modified() in some cases,
    *      but it doesn't hurt since we aren't going to seek.
    */
-  bdirect_read_commit(obuck_rpf, f->buffer + l);
-  slurp_remains -= l;
+  bdirect_read_commit(obuck->rpf, f->buffer + l);
+  obuck->slurp_remains -= l;
   f->bptr = f->buffer;
   f->bufend = f->bstop = f->buffer + l;
   f->pos += l;
@@ -438,75 +527,73 @@ obuck_slurp_refill(struct fastbuf *f)
 }
 
 void
-obuck_slurp_end(void)
+obuck_slurp_end(struct obuck *obuck)
 {
-  if (obuck_rpf)
+  if (obuck->rpf)
     {
-      bclose(obuck_rpf);
-      obuck_rpf = NULL;
-      obuck_unlock();
+      bclose(obuck->rpf);
+      obuck->rpf = NULL;
+      obuck_unlock_scan(obuck);
     }
 }
 
 struct fastbuf *
-obuck_slurp_pool(struct obuck_header *hdrp, oid_t next_oid)
+obuck_slurp_pool(struct obuck *obuck, struct obuck_header *hdrp, oid_t next_oid)
 {
-  static struct fastbuf limiter;
-  uns l;
-
   do
     {
-      if (!obuck_rpf)
+      if (!obuck->rpf)
 	{
-	  obuck_lock_read();
-	  obuck_rpf = bopen(obuck_name, O_RDONLY, obuck_slurp_buflen);
-	  slurp_end = bfilesize(obuck_rpf);
-	  obuck_lock_read_to_scan();
+	  obuck_lock_read(obuck);
+	  /* Note: obuck->rpf has its own file handle, and therefore its own file position */
+	  obuck->rpf = bopen(obuck->name, O_RDONLY, obuck->p.slurp_buflen);
+	  obuck->slurp_end = bfilesize(obuck->rpf);
+	  obuck_relock_read_to_scan(obuck);
 	}
       else
 	{
-	  bsetpos(obuck_rpf, slurp_current - 4);
-	  if (bgetl(obuck_rpf) != OBUCK_TRAILER)
-	    obuck_broken("Missing trailer", slurp_start);
+	  bsetpos(obuck->rpf, obuck->slurp_current - 4);
+	  if (bgetl(obuck->rpf) != OBUCK_TRAILER)
+	    obuck_broken(obuck, "Missing trailer", obuck->slurp_start);
 	}
       if (next_oid == OBUCK_OID_ANY)
-	slurp_start = btell(obuck_rpf);
+	obuck->slurp_start = btell(obuck->rpf);
       else
 	{
-	  slurp_start = obuck_get_pos(next_oid);
-	  bsetpos(obuck_rpf, slurp_start);
+	  obuck->slurp_start = obuck_get_pos(next_oid);
+	  bsetpos(obuck->rpf, obuck->slurp_start);
 	}
-      if (slurp_start < slurp_end)
-	l = bread(obuck_rpf, hdrp, sizeof(struct obuck_header));
-      else
+      if (obuck->slurp_start >= obuck->slurp_end)
 	{
-	  obuck_slurp_end();
+	  obuck_slurp_end(obuck);
 	  return NULL;
 	}
-      if (l != sizeof(struct obuck_header))
-	obuck_broken("Short header read", slurp_start);
+      if (bread(obuck->rpf, hdrp, sizeof(struct obuck_header)) != sizeof(struct obuck_header))
+	obuck_broken(obuck, "Short header read", obuck->slurp_start);
       if (hdrp->magic != OBUCK_MAGIC)
-	obuck_broken("Missing magic number", slurp_start);
-      slurp_current = slurp_start + obuck_bucket_size(hdrp->length);
+	obuck_broken(obuck, "Missing magic number", obuck->slurp_start);
+      obuck->slurp_current = obuck->slurp_start + obuck_bucket_size(hdrp->length);
     }
   while (hdrp->oid == OBUCK_OID_DELETED);
-  if (obuck_get_pos(hdrp->oid) != slurp_start)
-    obuck_broken("Invalid backlink", slurp_start);
-  slurp_remains = hdrp->length;
-  limiter.bptr = limiter.bstop = limiter.buffer = limiter.bufend = NULL;
-  limiter.name = "Bucket";
-  limiter.pos = 0;
-  limiter.refill = obuck_slurp_refill;
-  limiter.can_overwrite_buffer = obuck_rpf->can_overwrite_buffer;
-  return &limiter;
+  if (obuck_get_pos(hdrp->oid) != obuck->slurp_start)
+    obuck_broken(obuck, "Invalid backlink", obuck->slurp_start);
+  obuck->slurp_remains = hdrp->length;
+
+  struct fastbuf *lim = &obuck->limiter;
+  lim->bptr = lim->bstop = lim->buffer = lim->bufend = NULL;
+  lim->name = "Bucket";
+  lim->pos = 0;
+  lim->refill = obuck_slurp_refill;
+  lim->can_overwrite_buffer = obuck->rpf->can_overwrite_buffer;
+  return lim;
 }
 
 /*** Shakedown ***/
 
 static inline void
-shake_write(void *addr, int len, sh_off_t pos)
+shake_write(struct obuck *obuck, void *addr, int len, sh_off_t pos)
 {
-  int l = sh_pwrite(obuck_fd, addr, len, pos);
+  int l = sh_pwrite(obuck->fd, addr, len, pos);
   if (l != len)
     {
       if (l < 0)
@@ -517,14 +604,14 @@ shake_write(void *addr, int len, sh_off_t pos)
 }
 
 static inline void
-shake_sync(void)
+shake_sync(struct obuck *obuck)
 {
-  if (obuck_shake_security > 1)
-    fdatasync(obuck_fd);
+  if (obuck->p.shake_security > 1)
+    sh_fdatasync(obuck->fd);
 }
 
 static void
-shake_write_backup(sh_off_t bpos, byte *norm_buf, int norm_size, byte *fragment, int frag_size, sh_off_t frag_pos, int more_size)
+shake_write_backup(struct obuck *obuck, sh_off_t bpos, byte *norm_buf, int norm_size, byte *fragment, int frag_size, sh_off_t frag_pos, int more_size)
 {
   struct obuck_header *bhdr;
   int boff = 0;
@@ -532,7 +619,7 @@ shake_write_backup(sh_off_t bpos, byte *norm_buf, int norm_size, byte *fragment,
   oid_t old_oid;
 
   /* First of all, the "normal" part -- everything that will be written in this pass */
-  DBG("Backing up first round of changes at position %Lx + %x", (long long) bpos, norm_size);
+  DBG("Backing up first round of changes at position %llx + %x", (long long) bpos, norm_size);
   while (boff < norm_size)
     {
       /* This needn't be optimized for speed. */
@@ -541,7 +628,7 @@ shake_write_backup(sh_off_t bpos, byte *norm_buf, int norm_size, byte *fragment,
       l = obuck_bucket_size(bhdr->length);
       old_oid = bhdr->oid;
       bhdr->oid = bpos >> OBUCK_SHIFT;
-      shake_write(bhdr, l, bpos);
+      shake_write(obuck, bhdr, l, bpos);
       bhdr->oid = old_oid;
       boff += l;
       bpos += l;
@@ -557,7 +644,7 @@ shake_write_backup(sh_off_t bpos, byte *norm_buf, int norm_size, byte *fragment,
       ASSERT(bhdr->magic == OBUCK_MAGIC);
       old_oid = bhdr->oid;
       bhdr->oid = bpos >> OBUCK_SHIFT;
-      shake_write(bhdr, frag_size, bpos);
+      shake_write(obuck, bhdr, frag_size, bpos);
       bhdr->oid = old_oid;
       bpos += frag_size;
 
@@ -567,9 +654,9 @@ shake_write_backup(sh_off_t bpos, byte *norm_buf, int norm_size, byte *fragment,
       while (l < more_size)
 	{
 	  int j = MIN(more_size-l, 65536);
-	  if (sh_pread(obuck_fd, auxbuf, j, frag_pos + frag_size + l) != j)
+	  if (sh_pread(obuck->fd, auxbuf, j, frag_pos + frag_size + l) != j)
 	    die("obuck_shakedown read error: %m");
-	  shake_write(auxbuf, j, bpos);
+	  shake_write(obuck, auxbuf, j, bpos);
 	  bpos += j;
 	  l += j;
 	}
@@ -577,30 +664,31 @@ shake_write_backup(sh_off_t bpos, byte *norm_buf, int norm_size, byte *fragment,
 }
 
 static void
-shake_erase(sh_off_t start, sh_off_t end)
+shake_erase(struct obuck *obuck, sh_off_t start, sh_off_t end)
 {
   if (start > end)
     die("shake_erase called with negative length, that's a bug");
   ASSERT(!(start & (OBUCK_ALIGN-1)) && !(end & (OBUCK_ALIGN-1)));
   while (start < end)
     {
+      struct obuck_context ctx;
       u32 check = OBUCK_TRAILER;
-      obuck_hdr.magic = OBUCK_MAGIC;
-      obuck_hdr.oid = OBUCK_OID_DELETED;
+      ctx.hdr.magic = OBUCK_MAGIC;
+      ctx.hdr.oid = OBUCK_OID_DELETED;
       uns len = MIN(0x40000000, end-start);
-      obuck_hdr.length = len - sizeof(obuck_hdr) - 4;
-      DBG("Erasing %08x bytes at %Lx", len, (long long) start);
-      shake_write(&obuck_hdr, sizeof(obuck_hdr), start);
+      ctx.hdr.length = len - sizeof(ctx.hdr) - 4;
+      DBG("Erasing %08x bytes at %llx", len, (long long) start);
+      shake_write(obuck, &ctx.hdr, sizeof(ctx.hdr), start);
       start += len;
-      shake_write(&check, 4, start-4);
+      shake_write(obuck, &check, 4, start-4);
     }
 }
 
 void
-obuck_shakedown(int (*kibitz)(struct obuck_header *old, oid_t new, byte *buck))
+obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid_t new, byte *buck))
 {
   byte *buf;						/* Shakedown buffer and its size */
-  int buflen = ALIGN(obuck_shake_buflen, OBUCK_ALIGN);
+  int buflen = ALIGN_TO(obuck->p.shake_buflen, OBUCK_ALIGN);
   byte *msg;						/* Error message we will print */
   sh_off_t rstart, wstart;				/* Original and new position of buffer start */
   sh_off_t r_bucket_start, w_bucket_start;		/* Original and new position of the current bucket */
@@ -618,13 +706,13 @@ obuck_shakedown(int (*kibitz)(struct obuck_header *old, oid_t new, byte *buck))
   roff = woff = rsize = 0;
 
   /* We need to be the only accessor, all the object ID's are becoming invalid */
-  obuck_lock_write();
-  r_file_size = sh_seek(obuck_fd, 0, SEEK_END);
+  obuck_lock_write(obuck);
+  r_file_size = sh_seek(obuck->fd, 0, SEEK_END);
   ASSERT(!(r_file_size & (OBUCK_ALIGN - 1)));
   if (r_file_size >= (0x100000000 << OBUCK_SHIFT) - buflen)
     die("Bucket file is too large for safe shakedown. Shaking down with Bucket.ShakeSecurity=0 will still work.");
 
-  DBG("Starting shakedown. Buffer size is %d, original length %Lx", buflen, (long long) r_file_size);
+  DBG("Starting shakedown. Buffer size is %d, original length %llx", buflen, (long long) r_file_size);
 
   for(;;)
     {
@@ -652,7 +740,7 @@ obuck_shakedown(int (*kibitz)(struct obuck_header *old, oid_t new, byte *buck))
 	      goto broken;
 	    }
 	  /* Empty buckets are allowed to be large, but we need to handle them extra */
-	  DBG("Tricking around an extra-large empty bucket at %Lx + %x", (long long)r_bucket_start, l);
+	  DBG("Tricking around an extra-large empty bucket at %llx + %x", (long long)r_bucket_start, l);
 	  rsize = roff + l;
 	}
       else
@@ -705,19 +793,19 @@ obuck_shakedown(int (*kibitz)(struct obuck_header *old, oid_t new, byte *buck))
 	  /* Write the new contents of the bucket file */
 	  if (!wrote_anything)
 	    {
-	      if (obuck_shake_security)
+	      if (obuck->p.shake_security)
 		{
 		  /* But first write a backup at the end of the file to ensure nothing can be lost. */
-		  shake_write_backup(r_file_size, buf, woff, buf+roff, rsize-roff, rstart+roff, more);
-		  shake_sync();
+		  shake_write_backup(obuck, r_file_size, buf, woff, buf+roff, rsize-roff, rstart+roff, more);
+		  shake_sync(obuck);
 		}
 	      wrote_anything = 1;
 	    }
 	  if (woff)
 	    {
-	      DBG("Write %Lx %x", wstart, woff);
-	      shake_write(buf, woff, wstart);
-	      shake_sync();
+	      DBG("Write %llx %x", (long long)wstart, woff);
+	      shake_write(obuck, buf, woff, wstart);
+	      shake_sync(obuck);
 	    }
 	}
       else
@@ -739,8 +827,8 @@ obuck_shakedown(int (*kibitz)(struct obuck_header *old, oid_t new, byte *buck))
 
       /* And refill the buffer */
       r_bucket_start = rstart+rsize;	/* Also needed for error messages */
-      l = sh_pread(obuck_fd, buf+rsize, MIN(buflen-rsize, r_file_size - r_bucket_start), r_bucket_start);
-      DBG("Read  %Lx %x (%x inherited)", (long long)r_bucket_start, l, rsize);
+      l = sh_pread(obuck->fd, buf+rsize, MIN(buflen-rsize, r_file_size - r_bucket_start), r_bucket_start);
+      DBG("Read  %llx %x (%x inherited)", (long long)r_bucket_start, l, rsize);
       if (l < 0)
 	die("obuck_shakedown read error: %m");
       if (!l)
@@ -759,23 +847,23 @@ obuck_shakedown(int (*kibitz)(struct obuck_header *old, oid_t new, byte *buck))
       roff = 0;
     }
 
-  DBG("Finished at position %Lx", (long long) wstart);
-  sh_ftruncate(obuck_fd, wstart);
-  shake_sync();
+  DBG("Finished at position %llx", (long long) wstart);
+  sh_ftruncate(obuck->fd, wstart);
+  shake_sync(obuck);
 
-  obuck_unlock();
+  obuck_unlock(obuck);
   xfree(buf);
   return;
 
  broken:
-  log(L_ERROR, "Error during object pool shakedown: %s (pos=%Ld, id=%x), gathering debris",
+  log(L_ERROR, "Error during object pool shakedown: %s (pos=%lld, id=%x), gathering debris",
       msg, (long long) r_bucket_start, (uns)(r_bucket_start >> OBUCK_SHIFT));
   /*
    * We can attempt to clean up the bucket file by erasing everything between the last
    * byte written and the next byte to be read. If the secure mode is switched on, we can
    * guarantee that no data are lost, only some might be duplicated.
    */
-  shake_erase(wstart, rstart);
+  shake_erase(obuck, wstart, rstart);
   die("Fatal error during object pool shakedown");
 }
 
@@ -839,7 +927,7 @@ int main(int argc, char **argv)
 	for(i=0; i<h.length; i++)
 	  if ((unsigned) bgetc(b) != (i+j) % 256)
 	    die("Contents mismatch");
-	if (bgetc(b) != EOF)
+	if (bgetc(b) >= 0)
 	  die("EOF mismatch");
 	bclose(b);
       }
