@@ -11,6 +11,7 @@
 #include "sherlock/sherlock.h"
 #include "sherlock/bucket.h"
 #include "lib/fastbuf.h"
+#include "lib/ff-binary.h"
 #include "lib/lfs.h"
 #include "lib/getopt.h"
 #include "lib/mempool.h"
@@ -40,6 +41,7 @@ CF_USAGE
 -r\t\tDo not parse V33 buckets, but print the raw content\n\
 -v\t\tBe verbose\n\
 \nCommands:\n\
+-b <path>\tBucket file name instead of the default one\n\
 -c\t\tConcatenate and dump all buckets\n\
 -d <obj>\tDelete bucket\n\
 -f\t\tAudit bucket file structure (fsck)\n\
@@ -67,36 +69,44 @@ parse_id(char *c)
 static void
 list(int full)
 {
-  struct obuck_header h;
+  struct obuck_context c;
+  uns flags = full ? OBUCK_FULL : 0;
 
-  obuck_init(0);
-  if (obuck_find_first(&h, full))
+  bucket_open(0);
+  if (obuck_find_first(&bucket_file, &c, flags))
     do
       {
-	if (h.oid == OBUCK_OID_DELETED)
-	  printf("DELETED  %6d\n", h.length);
+	if (c.hdr.oid == OBUCK_OID_DELETED)
+	  printf("DELETED  %6d\n", c.hdr.length);
 	else
-	  printf("%08x %6d %08x\n", h.oid, h.length, h.type);
+	  printf("%08x %6d %08x\n", c.hdr.oid, c.hdr.length, c.hdr.type);
       }
-    while (obuck_find_next(&h, full));
-  obuck_cleanup();
+    while (obuck_find_next(&bucket_file, &c, flags));
+  bucket_close();
 }
 
 static void
 delete(char *id)
 {
   oid_t oid = parse_id(id);
-  obuck_init(1);
-  obuck_delete(oid);
-  obuck_cleanup();
+  bucket_open(1);
+  obuck_delete(&bucket_file, oid);
+  bucket_close();
 }
 
-static inline void
+static void
 dump_oattrs(struct fastbuf *out, struct oattr *oa)
 {
   for (; oa; oa = oa->next)
     for (struct oattr *a=oa; a; a = a->same)
-      bprintf(out, "%c%s\n", a->attr, a->val);
+      if (a->attr < OBJ_ATTR_SON)
+	bprintf(out, "%c%s\n", a->attr, a->val);
+      else
+        {
+	  bprintf(out, "(%c\n", a->attr - OBJ_ATTR_SON);
+	  dump_oattrs(out, a->son->attrs);
+	  bputs(out, ")\n");
+	}
 }
 
 static void
@@ -106,7 +116,7 @@ dump_parsed_bucket(struct fastbuf *out, struct obuck_header *h, struct fastbuf *
   mp_flush(pool);
   o_hdr = obj_new(pool);
   o_body = obj_new(pool);
-  if (buck2obj_parse(buck_buf, h->type, h->length, b, o_hdr, NULL, o_body) < 0)
+  if (buck2obj_parse(buck_buf, h->type, h->length, b, o_hdr, NULL, o_body, 1) < 0)
     bprintf(out, ".Cannot parse bucket %x of type %x and length %d: %m\n", h->oid, h->type, h->length);
   else
     {
@@ -120,22 +130,22 @@ static void
 extract(char *id)
 {
   struct fastbuf *b, *out;
-  struct obuck_header h;
+  struct obuck_context c;
 
-  h.oid = parse_id(id);
-  obuck_init(0);
-  obuck_find_by_oid(&h);
+  c.hdr.oid = parse_id(id);
+  bucket_open(0);
+  obuck_find_by_oid(&bucket_file, &c, 0);
   out = bfdopen_shared(1, 65536);
   if (verbose)
-    bprintf(out, "### %08x %6d %08x\n", h.oid, h.length, h.type);
-  b = obuck_fetch();
-  if (h.type < BUCKET_TYPE_V33 || !buck_buf)
+    bprintf(out, "### %08x %6d %08x\n", c.hdr.oid, c.hdr.length, c.hdr.type);
+  b = obuck_fetch(&bucket_file, &c);
+  if (c.hdr.type < BUCKET_TYPE_V33 || !buck_buf)
     bbcopy_slow(b, out, ~0U);
   else
-    dump_parsed_bucket(out, &h, b);
+    dump_parsed_bucket(out, &c.hdr, b);
   bclose(b);
   bclose(out);
-  obuck_cleanup();
+  bucket_close();
 }
 
 static void
@@ -159,7 +169,7 @@ insert(byte *arg)
   put_attr_set_type(type);
 
   in = bfdopen_shared(0, 4096);
-  obuck_init(1);
+  bucket_open(1);
   do
     {
       uns lizard_filled = 0;
@@ -174,7 +184,7 @@ insert(byte *arg)
 	    in_body = 1;
 	  }
 	  if (!b)
-	    b = obuck_create();
+	    b = obuck_create(&bucket_file);
 	  if (in_body == 1)
 	  {
 	    if (type < BUCKET_TYPE_V33)
@@ -219,14 +229,14 @@ insert(byte *arg)
       }
       if (b)
 	{
-	  obuck_create_end(b, type, &h);
+	  obuck_create_end(&bucket_file, b, type, &h);
 	  printf("%08x %d %08x\n", h.oid, h.length, h.type);
 	}
     }
   while (e);
   bb_done(&lizard_buf);
   bb_done(&compressed_buf);
-  obuck_cleanup();
+  bucket_close();
   bclose(in);
 }
 
@@ -237,9 +247,9 @@ cat(void)
   struct fastbuf *b, *out;
   byte buf[1024];
 
-  obuck_init(0);
+  bucket_open(0);
   out = bfdopen_shared(1, 65536);
-  while (b = obuck_slurp_pool(&h, OBUCK_OID_ANY))
+  while (b = obuck_slurp_pool(&bucket_file, &h, OBUCK_OID_ANY))
     {
       bprintf(out, "### %08x %6d %08x\n", h.oid, h.length, h.type);
       if (h.type < BUCKET_TYPE_V33 || !buck_buf)
@@ -258,7 +268,7 @@ cat(void)
       bputc(out, '\n');
     }
   bclose(out);
-  obuck_cleanup();
+  bucket_close();
 }
 
 static void
@@ -273,9 +283,9 @@ fsck(int fix)
   int errors = 0;
   int fatal_errors = 0;
 
-  fd = sh_open(obuck_name, O_RDWR);
+  fd = sh_open(bucket_file_name, O_RDWR);
   if (fd < 0)
-    die("Unable to open the bucket file %s: %m", obuck_name);
+    die("Unable to open the bucket file %s: %m", bucket_file_name);
   for(;;)
     {
       if (pos >= ((sh_off_t) OBUCK_OID_FIRST_SPECIAL << OBUCK_SHIFT))
@@ -374,16 +384,16 @@ shake_kibitz(struct obuck_header *old, oid_t new, byte *buck UNUSED)
 static void
 shake(void)
 {
-  obuck_init(1);
-  obuck_shakedown(shake_kibitz);
-  obuck_cleanup();
+  bucket_open(1);
+  obuck_shakedown(&bucket_file, shake_kibitz);
+  bucket_close();
 }
 
 static void
 quickcheck(void)
 {
-  obuck_init(1);
-  obuck_cleanup();
+  bucket_open(1);
+  bucket_close();
 }
 
 int
@@ -395,7 +405,7 @@ main(int argc, char **argv)
 
   log_init(NULL);
   op = 0;
-  while ((i = cf_getopt(argc, argv, CF_SHORT_OPTS "cd:fFi::lLqrsvx:", CF_NO_LONG_OPTS, NULL)) != -1)
+  while ((i = cf_getopt(argc, argv, CF_SHORT_OPTS "b:cd:fFi::lLqrsvx:", CF_NO_LONG_OPTS, NULL)) != -1)
     if (i == '?' || op)
       help();
     else if (i == 'v')
@@ -447,6 +457,9 @@ main(int argc, char **argv)
     case 's':
       shake();
       break;
+    case 'b':
+      bucket_set_name(arg);
+      break;
     default:
       help();
     }
