@@ -2,6 +2,7 @@
  *	Sherlock Library -- Adding Formatted Attributes
  *
  *	(c) 2005 Martin Mares <mj@ucw.cz>
+ *	(c) 2007 Pavel Charvat <pchar@ucw.cz>
  *
  *	This software may be freely distributed and used according to the terms
  *	of the GNU Lesser General Public License.
@@ -9,16 +10,14 @@
 
 #include "sherlock/sherlock.h"
 #include "sherlock/object.h"
+#include "lib/stkstring.h"
 
 #include <stdio.h>
 
 struct oattr *
 obj_add_attr_vformat(struct odes *o, uns x, char *fmt, va_list args)
 {
-  char buf[MAX_ATTR_SIZE-1];
-  int len = vsnprintf(buf, sizeof(buf), fmt, args);
-  ASSERT(len >= 0 && len < (int)sizeof(buf));
-  return obj_add_attr(o, x, buf);
+  return obj_add_attr(o, x, stk_vprintf(fmt, args));
 }
 
 struct oattr *obj_add_attr_format(struct odes *o, uns x, char *fmt, ...)
@@ -33,10 +32,7 @@ struct oattr *obj_add_attr_format(struct odes *o, uns x, char *fmt, ...)
 struct oattr *
 obj_set_attr_vformat(struct odes *o, uns x, char *fmt, va_list args)
 {
-  char buf[MAX_ATTR_SIZE-1];
-  int len = vsnprintf(buf, sizeof(buf), fmt, args);
-  ASSERT(len >= 0 && len < (int)sizeof(buf));
-  return obj_set_attr(o, x, buf);
+  return obj_set_attr(o, x, stk_vprintf(fmt, args));
 }
 
 struct oattr *obj_set_attr_format(struct odes *o, uns x, char *fmt, ...)
