@@ -13,6 +13,7 @@
 #include "lib/ff-utf8.h"
 #include "sherlock/object.h"
 
+#include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
 
@@ -70,7 +71,6 @@ size_object(struct odes *d)
 inline byte *
 put_attr(byte *ptr, uns type, byte *val, uns len)
 {
-  ASSERT(len <= MAX_ATTR_SIZE-2);
   if (use_v33)
   {
     PUT_UTF8_32(ptr, len+1);
@@ -124,7 +124,6 @@ put_attr_vformat(byte *ptr, uns type, byte *mask, va_list va)
     ptr += len;
     *ptr++ = '\n';
   }
-  ASSERT(len <= MAX_ATTR_SIZE-2);
   return ptr;
 }
 
@@ -210,7 +209,6 @@ bput_attr_large(struct fastbuf *b, uns type, byte *val, uns len)
 inline void
 bput_attr(struct fastbuf *b, uns type, byte *val, uns len)
 {
-  ASSERT(len <= MAX_ATTR_SIZE-2);
   bput_attr_large(b, type, val, len);
 }
 
@@ -242,7 +240,6 @@ bput_attr_vformat(struct fastbuf *b, uns type, byte *mask, va_list va)
     len = vbprintf(b, mask, va);
     bputc(b, '\n');
   }
-  ASSERT(len <= MAX_ATTR_SIZE-2);
 }
 
 void
