@@ -32,6 +32,27 @@ obj_dump(struct odes *d)
 	printf("%c%s\n", (a==b ? a->attr : ' '), b->val);
 }
 
+void
+obj_dump_indented(struct odes *d, uns indent)
+{
+  for(struct oattr *a=d->attrs; a; a=a->next)
+    for(struct oattr *b=a; b; b=b->same)
+      {
+	for (uns i=0; i<indent; i++)
+	  putchar('\t');
+	if (a->attr >= OBJ_ATTR_SON)
+	  {
+	    printf("(%c\n", a->attr - OBJ_ATTR_SON);
+	    obj_dump_indented(b->son, indent+1);
+	    for (uns i=0; i<=indent; i++)
+	      putchar('\t');
+	    printf(")\n");
+	  }
+	else
+	  printf("%c%s\n", (a==b ? a->attr : ' '), b->val);
+      }
+}
+
 static struct oattr *
 oa_new(struct odes *o, uns x, byte *v)
 {
@@ -155,6 +176,13 @@ obj_find_anum(struct odes *o, uns x, uns def)
   return a ? (uns)atol(a->val) : def;
 }
 
+u32
+obj_find_x32(struct odes *o, uns x, u32 def)
+{
+  struct oattr *a = obj_find_attr(o, x);
+  return a ? (u32)strtoul(a->val, NULL, 16) : def;
+}
+
 u64
 obj_find_x64(struct odes *o, uns x, u64 def)
 {
