@@ -34,11 +34,6 @@ struct mempool;
 
 /* object.c: In-memory representation of objects */
 
-#define MAX_ATTR_SIZE 2500
-  /* Maximum length an attribute can ever have (including name and trailing 0).
-   * It has to be long enough to contain 1 URL, 1 reftext, and a few numbers
-   * (see 'x' attribute in labels).  */
-
 struct odes {				/* Object description */
   struct oattr *attrs;
   struct mempool *pool;
@@ -58,12 +53,14 @@ struct oattr {				/* Object attribute */
 #define OBJ_ATTR_SON 256
 
 void obj_dump(struct odes *);
+void obj_dump_indented(struct odes *, uns);
 struct odes *obj_new(struct mempool *);
 struct oattr *obj_find_attr(struct odes *, uns);
 struct oattr *obj_find_attr_last(struct odes *, uns);
 uns obj_del_attr(struct odes *, struct oattr *);
 byte *obj_find_aval(struct odes *, uns);
 uns obj_find_anum(struct odes *, uns, uns);
+u32 obj_find_x32(struct odes *, uns, u32);
 u64 obj_find_x64(struct odes *, uns, u64);
 struct oattr *obj_set_attr(struct odes *, uns, byte *);
 struct oattr *obj_set_attr_num(struct odes *, uns, uns);
@@ -98,6 +95,7 @@ struct parsed_attr {
 };
 struct buck2obj_buf;
 
+/* note: get_attr routines are not thread-safe */
 void get_attr_set_type(uns type);
 int get_attr(byte **pos, byte *end, struct parsed_attr *attr);
 int bget_attr(struct fastbuf *b, struct parsed_attr *attr);
@@ -106,8 +104,11 @@ void copy_parsed_attr(struct mempool *pool, struct parsed_attr *attr);
 struct buck2obj_buf *buck2obj_alloc(void);
 void buck2obj_free(struct buck2obj_buf *buf);
 
-int buck2obj_parse(struct buck2obj_buf *buf, uns buck_type, uns buck_len, struct fastbuf *body, struct odes *o_hdr, uns *body_start, struct odes *o_body);
-struct odes *obj_read_bucket(struct buck2obj_buf *buf, struct mempool *pool, uns buck_type, uns buck_len, struct fastbuf *body, uns *body_start);
+int buck2obj_parse(struct buck2obj_buf *buf, uns buck_type, uns buck_len, struct fastbuf *body,
+		   struct odes *o_hdr, uns *body_start, struct odes *o_body,
+		   uns allow_zero_copy);
+struct odes *obj_read_bucket(struct buck2obj_buf *buf, struct mempool *pool, uns buck_type, uns buck_len, struct fastbuf *body,
+			     uns *body_start, uns allow_zero_copy);
   /* If body_start != NULL, then only the header is parsed and *body_start is
    * set to the position of the body. This function does a plenty of optimizations
    * and if the body fastbuf is overwritable (body->can_overwrite_buffer), it can keep the
