@@ -27,7 +27,7 @@
 
 typedef u32 oid_t;			/* Object ID */
 
-#ifdef CONFIG_LARGE_DB			/* File positions */
+#ifdef CONFIG_LARGE_FILES		/* File positions */
 #define BYTES_PER_O 5
 #define bgeto(f) bget5(f)
 #define bputo(f,l) bput5(f,l)
