@@ -1,7 +1,7 @@
 /*
  *	Sherlock Library -- Processing of tagged characters
  *
- *	(c) 2001--2003 Martin Mares <mj@ucw.cz>
+ *	(c) 2001--2006 Martin Mares <mj@ucw.cz>
  *
  *	This software may be freely distributed and used according to the terms
  *	of the GNU Lesser General Public License.
@@ -15,40 +15,50 @@
 
 /* Reading of tagged text (Unicode values, tags mapped to 0x80000000 and higher) */
 
-#define GET_TAGGED_CHAR(p,u) do {				\
-  u = *p;							\
-  if (u >= 0xc0)						\
-    GET_UTF8(p,u);						\
-  else if (u >= 0x80)						\
-    {								\
-      p++;							\
-      if (u >= 0xb0)						\
-        {							\
-	  ASSERT(u == 0xb0);					\
-	  u += 0x80020000;					\
-        }							\
-      else if (u >= 0xa0)					\
-        {							\
-	  ASSERT(*p >= 0x80 && *p <= 0xbf);			\
-	  u = 0x80010000 + ((u & 0x0f) << 6) + (*p++ & 0x3f);	\
-        }							\
-      else							\
-	u += 0x80000000;					\
-    }								\
-  else								\
-    p++;							\
-} while (0)
+static inline const byte *
+get_tagged_char(const byte *p, uns *uu)
+{
+  uns u = *p;
+  if (u >= 0xc0)
+    return utf8_get(p, uu);
+  else if (u >= 0x80)
+    {
+      p++;
+      if (u >= 0xb0)
+        {
+	  ASSERT(u == 0xb0);
+	  u += 0x80020000;
+        }
+      else if (u >= 0xa0)
+        {
+	  ASSERT(*p >= 0x80 && *p <= 0xbf);
+	  u = 0x80010000 + ((u & 0x0f) << 6) + (*p++ & 0x3f);
+        }
+      else
+	u += 0x80000000;
+    }
+  else
+    p++;
+  *uu = u;
+  return p;
+}
 
-#define SKIP_TAGGED_CHAR(p) do {				\
-  if (*p >= 0x80 && *p < 0xc0)					\
-    {								\
-      uns u = *p++;						\
-      if (u >= 0xa0 && u < 0xb0 && *p >= 0x80 && *p < 0xc0)	\
-	p++;							\
-    }								\
-  else								\
-    UTF8_SKIP(p);						\
-} while (0)
+static inline const byte *
+skip_tagged_char(const byte *p)
+{
+  if (*p >= 0x80 && *p < 0xc0)
+    {
+      uns u = *p++;
+      if (u >= 0xa0 && u < 0xb0 && *p >= 0x80 && *p < 0xc0)
+	p++;
+    }
+  else
+    UTF8_SKIP(p);
+  return p;
+}
+
+#define GET_TAGGED_CHAR(p,u) p=(byte*)get_tagged_char(p, &u)
+#define SKIP_TAGGED_CHAR(p) p=(byte*)skip_tagged_char(p)
 
 /* The same from a fastbuf */
 
