@@ -16,6 +16,7 @@
 #include "lib/fastbuf.h"
 #include "lib/chartype.h"
 #include "lib/hashfunc.h"
+#include "lib/getopt.h"
 
 #include <string.h>
 #include <fcntl.h>
@@ -54,7 +55,7 @@ struct pxtab_node {
 static inline uns
 pxtab_hash(HASH_KEY_DECL)
 {
-  return ((uns)(addr_int_t)parent) ^ hash_block(component, len);
+  return ((uns)(uintptr_t)parent) ^ hash_block(component, len);
 }
 
 static inline int
@@ -326,7 +327,7 @@ url_fingerprint(byte *url, struct fingerprint *fp)
 
 int main(int argc, char **argv)
 {
-  cf_read(cfdeffile);
+  cf_load(cf_def_file);
   url_key_init();
   for (int i=1; i<argc; i++)
     {
