@@ -11,7 +11,7 @@ use integer;
 use lib 'lib/perl5';
 use Sherlock::Query();
 
-my (@buckets, @urls, @bids, @cards, @URLs, @oids);
+my (@buckets, @urls, @bids, @cards, @patched, @URLs, @oids);
 my $idxdir = "index";
 my $host = "localhost";
 my $port = 8192;
@@ -49,13 +49,14 @@ Descriptors:
 	--url URL
 	--bucket BUCKETNR
 	--id ID		... stage1 object ID, intervals are also allowed
-	  --withlinks
+	  --withlinks	... documents that point TO this document
 	  --withsimilar
 	  --withmerges
 	  --withcards
   Cards:
 	--turl URL
 	--card CARDID	... stage2 card ID, all cards can be dumped by CARDID=0-
+	--patched CARDID... fake object created by patch-index for stage2
 	--oid OID	... stage2 object ID, intervals are also allowed
 	  --withbuckets
 ";
@@ -82,6 +83,7 @@ GetOptions(
 	"withmerges!" => \$withmerges,
 	"withcards!" => \$convert_id2oid,
 	"card=s@" => \@cards,
+	"patched=s@" => \@patched,
 	"oid=s@" => \@oids,
 	"withbuckets!" => \$convert_oid2id,
 ) || die $help;
@@ -91,6 +93,7 @@ die "Superfluous options (@ARGV)\n$help" if $#ARGV >= 0;
 @buckets = map { split(/,?\s+/, $_) } @buckets;
 @bids = map { split(/,?\s+/, $_) } @bids;
 @cards = map { split(/,?\s+/, $_) } @cards;
+@patched = map { split(/,?\s+/, $_) } @patched;
 @oids = map { split(/,?\s+/, $_) } @oids;
 @urls = map { split(/\s+/, $_) } @urls;		# XXX: not sure if this should be done, but my convenience tells me yes
 @URLs = map { split(/\s+/, $_) } @URLs;
@@ -99,6 +102,7 @@ if ($decimal)
 	@buckets = map { sprintf("%x", $_) } @buckets;
 	@bids = map { sprintf("%x", $_) } @bids;
 	@cards = map { sprintf("%x", $_) } @cards;
+	@patched = map { sprintf("%x", $_) } @patched;
 	@oids = map { sprintf("%x", $_) } @oids;
 }
 
@@ -143,8 +147,9 @@ if ($#oids >= 0)
 ### Sort the dumped files
 @buckets = make_unique(sort { hex $a <=> hex $b } @buckets);
 @cards = make_unique(sort { hex $a <=> hex $b } @cards);
+@patched = make_unique(sort { hex $a <=> hex $b } @patched);
 ### Prepare the dumping script
-die "Nothing to dump\n$help" if ($#buckets < 0 and $#cards < 0 and !$allbuckets);
+die "Nothing to dump\n$help" if ($#buckets < 0 and $#cards < 0 and $#patched < 0 and !$allbuckets);
 my $program = "(\n";
 if ($allbuckets)
 {
@@ -184,6 +189,13 @@ if ($#cards >= 0)
 	$" = " ";
 	$program .= "./bin/idxdump -f $idxdir/cards -c @cards\n";
 }
+if ($#patched >= 0)
+{
+	$program .= "echo \"### ffffffff 0 80000000\n#Patched objects:\n\"\n";
+	$" = ", "; print STDERR "patched: @patched\n";
+	$" = " ";
+	$program .= "./bin/idxdump -f $idxdir/objects -c @patched\n";
+}
 $program .= ") | ./bin/objdump $dumperparams";
 $program .= " | less" if !$to_stdout;
 $program .= "\n";
@@ -264,14 +276,17 @@ sub id2card
 sub get_links
 {
 	my ($bid) = @_;
-	my $graph_idx = `./bin/idxdump -f $idxdir/link-graph-index -G $bid`;
+	my $renum = `./bin/idxdump -f $idxdir/link-graph-goes -m $bid`;
+	$renum =~ /^0*$bid -> ([0-9a-f]*)$/ or return ($bid);
+	my $graph_idx = `./bin/idxdump -f $idxdir/link-graph-index -G $1`;
 	$graph_idx =~ /.* -> ([0-9a-f]*)/ or return ($bid);
+	if ($1 eq "ffffffffff") { return ($bid); }
 	my $target = `./bin/idxdump -f $idxdir/link-graph -g $1`;
 	$target =~ /Vertex ([0-9a-f]*) /
 	and hex $1 == hex $bid
 		or return ($bid);
 	return $bid, map {
-		if (/-> ([0-9a-f]*)($| \[[fr])/)
+		if (/<- ([0-9a-f]*)($| \[[fr])/)
 		{ $1 }
 		else { () }
 	} split /\n/, $target;
