@@ -16,6 +16,7 @@ if ! [ -f bin/daemon-helper -a -f cf/sherlock ] ; then
 	echo >&2 "gcontrol: Cannot find Sherlock runtime directory"
 	exit 1
 fi
+GCONTROL="bin/gcontrol$UCW_CF"
 
 DHO="--pid-file=lock/scheduler.pid --status-file=lock/scheduler.slot"
 NAME=scheduler
@@ -43,13 +44,13 @@ case "$1" in
 				exit 1
 			fi
 			;;
-	reload)		bin/gcontrol stop
-			bin/gcontrol start
+	reload)		$GCONTROL stop
+			$GCONTROL start
 			;;
-	restart)	bin/gcontrol stop
-			bin/gcontrol start
+	restart)	$GCONTROL stop
+			$GCONTROL start
 			;;
-	*)		echo >&2 "Usage: [SH_USER=<user>] [SH_HOME=<homedir>] gcontrol (start [<initial-slot>]|stop|force-stop|restart)"
+	*)		echo >&2 "Usage: [SH_USER=<user>] [SH_HOME=<homedir>] gcontrol (start [<initial-slot>]|stop|force-stop|restart|reload)"
 			exit 1
 			;;
 esac
