@@ -6,19 +6,20 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
 #include "sherlock/bucket.h"
 #include "sherlock/object.h"
-#include "lib/mempool.h"
 #include "sherlock/index.h"
 #include "sherlock/tagged-text.h"
-#include "lang/detect.h"
+#include "lib/getopt.h"
+#include "lib/fastbuf.h"
+#include "lib/mempool.h"
 #include "lib/unicode.h"
 #include "charset/charconv.h"
 #include "charset/fb-charconv.h"
+#include "lang/detect.h"
 #include "utils/dumpconfig.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -28,6 +29,7 @@ static int raw_dump;
 static int do_lang_detect;
 static int verbose;
 static uns bucket_type = BUCKET_TYPE_PLAIN;
+static struct lang_detect *lang_detect;
 
 /* Maximal input line length.  */
 #define	BUFSIZE	1024
@@ -104,9 +106,9 @@ objdump_line(uns type, byte *line)
 		int is_meta = (type == 'M');
 		char **type_names = is_meta ? mt_names : wt_names;
 		byte *source = line, *end = line + strlen(line);
-#ifdef CONFIG_LANG
+#if defined(CONFIG_LANG) && defined(CONFIG_LANG_DETECT)
 		if (type == 'X' && do_lang_detect)
-			lang_detect_add_string(source);
+			lang_detect_add_string(lang_detect, source);
 #endif
 		out_space = 1;
 		this_attr = type;
@@ -167,10 +169,10 @@ objdump_done(void)
 {
 	flush_outtext();
 	bprintf(output, "\n");
-#ifdef CONFIG_LANG
+#if defined(CONFIG_LANG) && defined(CONFIG_LANG_DETECT)
 	if (do_lang_detect)
 	{
-		struct lang_detect_results *res = lang_detect_compute();
+		struct lang_detect_results *res = lang_detect_compute(lang_detect);
 		if (res->lang1 < 0)
 			bprintf(output, "The language cannot be detected\n\n");
 		else
@@ -218,7 +220,7 @@ objdump_done(void)
 				bprintf(output, "\n");
 			}
 		}
-		lang_detect_start();
+		lang_detect_start(lang_detect);
 	}
 #endif
 }
@@ -293,7 +295,7 @@ read_and_dump(uns type)
 
 	struct odes *oh = obj_new(pool);
 	struct odes *ob = obj_new(pool);
-	if (buck2obj_parse(b2ob, type, len, r, oh, NULL, ob) < 0)
+	if (buck2obj_parse(b2ob, type, len, r, oh, NULL, ob, 1) < 0)
 		bprintf(output, ".Parse error: %m\n");
 	else {
 		put_attr_set_type(BUCKET_TYPE_PLAIN);
@@ -345,11 +347,12 @@ main(int argc, char **argv)
 		is_recoding = (term_charset_id != CONV_CHARSET_UTF8);
 		output = fb_wrap_charconv_out(output, CONV_CHARSET_UTF8, term_charset_id);
 	}
-#ifdef CONFIG_LANG
+#if defined(CONFIG_LANG) && defined(CONFIG_LANG_DETECT)
 	if (do_lang_detect)
 	{
-		lang_detect_build_automaton();
-		lang_detect_start();
+		lang_detect_load_tables();
+		lang_detect = lang_detect_alloc(NULL);
+		lang_detect_start(lang_detect);
 	}
 #endif
 	if (bucket_type == BUCKET_TYPE_PLAIN)
