@@ -191,7 +191,73 @@ case "$1" in
 			rmdir "$CF_SKeeper_SwapLock"
 			exit 1
 			;;
-	*)		echo >&2 "Usage: [SH_USER=<user>] [SH_HOME=<homedir>] scontrol (start|stop|restart|cron|test|swap [<index>])"
+	swap-blacklist)	shift
+			INDEX="${1:-index}"
+			BLACKLIST="${2:-blacklist}"
+			SUFFIX="${3:-.new}"
+			log "Swapping blacklist: $INDEX$SUFFIX/$BLACKLIST"
+			if [ ! -f "$INDEX$SUFFIX/$BLACKLIST" -o ! -f "$INDEX$SUFFIX/parameters" ] ; then
+				log "Nothing to swap"
+				exit 0
+			fi
+			echo -n "Acquiring swap lock..."
+			RETRY=$CF_SKeeper_SwapLockTimeout
+			while ! mkdir "$CF_SKeeper_SwapLock" 2>/dev/null ; do
+				if [ $RETRY = 0 ] ; then
+					echo " TIMED OUT."
+					exit 1
+				fi
+				RETRY=$(($RETRY-1))
+				echo -n "."
+				sleep 1
+			done
+			echo " done."
+			RUN=
+			if bin/daemon-helper --check $DHO ; then
+				RUN=1
+			fi
+			echo -n "Checking blacklist version..."
+			if [ "`bin/index-version $INDEX$SUFFIX`" != "`bin/index-version $INDEX`" ] ; then
+				echo " INVALID."
+				rmdir "$CF_SKeeper_SwapLock"
+				exit 2
+			fi
+			echo " done."
+			rm -rf "$INDEX/$BLACKLIST.old"
+			if [ -f "$INDEX/$BLACKLIST" ] ; then
+				mv "$INDEX/$BLACKLIST" "$INDEX/$BLACKLIST.old"
+			fi
+			log "Swapping $BLACKLIST"
+			mv "$INDEX$SUFFIX/$BLACKLIST" "$INDEX/$BLACKLIST"
+			if [ -z "$RUN" ] ; then
+				rm -rf "$INDEX$SUFFIX"
+				rmdir "$CF_SKeeper_SwapLock"
+				exit 0
+			fi
+			if bin/sherlockd --merge-only ; then
+				if $SCONTROL test ; then
+					rm -rf "$INDEX$SUFFIX"
+					rmdir "$CF_SKeeper_SwapLock"
+					exit 0
+				fi
+			fi
+			$SCONTROL stop
+			if [ -f "$INDEX/$BLACKLIST.old" ] ; then
+				log "Falling back to old $BLACKLIST"
+				mv "$INDEX/$BLACKLIST" "$INDEX$SUFFIX/$BLACKLIST.new"
+				mv "$INDEX/$BLACKLIST.old" "$INDEX/$BLACKLIST"
+			else
+				errlog "No old $BLACKLIST to fall back to"
+			fi
+			if $SCONTROL start ; then
+				log "Fallback succeeded"
+			else
+				errlog "Fallback failed"
+			fi
+			rmdir "$CF_SKeeper_SwapLock"
+			exit 1
+			;;
+	*)		echo >&2 "Usage: [SH_USER=<user>] [SH_HOME=<homedir>] scontrol (start|stop|restart|cron|test|swap|swap-blacklist [<index> | <index> <blacklist> <suffix>])"
 			exit 1
 			;;
 esac
