@@ -567,7 +567,11 @@ open_file(struct host_info *hi, struct file_info *fi)
   rw_flags = (hi->function == SHCP_SEND) ? O_WRONLY | O_CREAT | O_TRUNC : O_RDONLY;
 
   if(odirect)
+#ifdef O_DIRECT
     rw_flags |= O_DIRECT;
+#else
+    die("Direct mode is not supported on this platform");
+#endif
 
   if(hi->function == SHCP_SEND) {
     if(is_dir(hi->path))
@@ -662,7 +666,7 @@ read_file_header(struct host_info *hi, struct file_info *fi)
       sh_seek(hi->descriptor, 0, SEEK_SET);
       randomkey(keybin, MD5_SIZE);
       md5_to_hex(keybin, fi->key);
-      TRACE("Creating file headers - size: %Ld bytes, key: %s", fi->size, fi->key);
+      TRACE("Creating file headers - size: %lld bytes, key: %s", (long long) fi->size, fi->key);
       break;
 
     case SHCP_SERVER:
@@ -672,7 +676,7 @@ read_file_header(struct host_info *hi, struct file_info *fi)
       sread(hi, fi->path, length);
       sread(hi, &fi->size, 8);
       sread(hi, fi->key, MD5_HEX_SIZE);
-      TRACE("Getting file headers - size: %Ld bytes, key: %s", fi->size, fi->key);
+      TRACE("Getting file headers - size: %lld bytes, key: %s", (long long) fi->size, fi->key);
       break;
   }
 }
@@ -1206,10 +1210,10 @@ main(int argc, char **argv)
       limit_init();
 
     if(*hi_file_recv_list)
-      LOG(L_INFO, "Sending file %s (%Ld bytes)", (*hi_file_recv_list)->file, file_current->size);
+      LOG(L_INFO, "Sending file %s (%lld bytes)", (*hi_file_recv_list)->file, (long long) file_current->size);
     if(trace)
       for(p = hi_file_send_list; *p != NULL; p++)
-        LOG(L_INFO, "Receiving file %s (%Ld bytes)", (*p)->file, file_current->size);
+        LOG(L_INFO, "Receiving file %s (%lld bytes)", (*p)->file, (long long) file_current->size);
 
     size = file_current->size;
     while(size > 0) {
@@ -1249,11 +1253,11 @@ main(int argc, char **argv)
 
     if(trace || host_info_list_count(hi_send_list) == host_info_list_count(hi_file_send_list)) {
       if(hi_file_recv_list[0] && hi_file_send_list[0])
-        LOG(L_INFO, "Transceived %Ld bytes in %.1fs (%.1f MB/s)", file_current->size, tdiff, speed);
+        LOG(L_INFO, "Transceived %lld bytes in %.1fs (%.1f MB/s)", (long long) file_current->size, tdiff, speed);
       else if(hi_file_recv_list[0])
-        LOG(L_INFO, "Sent %Ld bytes in %.1fs (%.1f MB/s)", file_current->size, tdiff, speed);
+        LOG(L_INFO, "Sent %lld bytes in %.1fs (%.1f MB/s)", (long long) file_current->size, tdiff, speed);
       else if(hi_file_send_list[0])
-        LOG(L_INFO, "Received %Ld bytes in %.1fs (%.1f MB/s)", file_current->size, tdiff, speed);
+        LOG(L_INFO, "Received %lld bytes in %.1fs (%.1f MB/s)", (long long) file_current->size, tdiff, speed);
     }
   }
 
