@@ -46,7 +46,7 @@ main(int argc, char **argv)
 	      err = 1;
 	    }
 	  else
-	    p += sprintf(p, "%Ld", (long long) len);
+	    p += sprintf(p, "%lld", (long long) len);
 	  close(fd);
 	}
     }
