@@ -2,6 +2,8 @@
 # Perl library module for Watson monitoring system
 #
 # (c) 2003-2004 Tomas Valla <tom@ucw.cz>
+# (c) 2006 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
+# (c) 2007 Pavel Charvat <pchar@ucw.cz>
 #
 
 package Sherlock::Watsonlib;
@@ -15,6 +17,7 @@ analyze_init analyze_finish
 compute_stat logprint info warning stat_usage stat_options
 analyze_options analyze_usage
 $stat_prefix $stat_picname $stat_begintime $stat_endtime $stat_expanddays
+read_config read_config_value read_config_array server_config_value $config_file %config
 );
 
 use strict;
@@ -32,8 +35,67 @@ our $verbose = 0;
 
 our $quantum = 5*60;
 
-our $header_size=70;
+our $header_size = 70;
 
+our $error_time = 0;
+
+########################################################
+# Read config
+
+our $config_file = "cf/watson";
+our %config = ();
+
+sub read_config {
+	-f $config_file or die "Cannot open config file $config_file";
+	my $cmd = ". $config_file >/dev/null 2>&1";
+	foreach my $var (@_) {
+		# A little complex import of bash variables from the configuration script ;-)
+		# ... should work for any string values
+		if ($var =~ /^[a-zA-Z_][a-zA-Z0-9_]*$/) { # A single varible
+			$cmd.=' && echo -n "\$config{\"'.$var.'\"}='."'".'" && ' .
+			'(echo -n "${'.$var.'}" | sed "s'."/'/'.\\\"'\\\".'/".'g") && ' .
+			'echo "'."'".';"';
+		}
+		elsif ($var =~ /^([a-zA-Z_][a-zA-Z_0-9]*)\*$/) { # All variables with a given prefix
+			$cmd.=' && (for VAR in ${!'.$1.'*} ; do ' . 
+			'echo -n "\$config{\"$VAR\"}='."'".'" && ' .
+			'(eval echo -n "\${$VAR}" | sed "s'."/'/'.\\\"'\\\".'/".'g") && ' .
+			'echo "'."'".';"; done)';
+		}
+		else { die "Invalid read_config parameter"; }
+	}
+	eval `$cmd`;
+}
+
+sub read_config_value {
+	my ($var, $default) = @_;
+	defined $config{"$var"} or return $default;
+	if ($config{"$var"} eq '') {
+		defined $default or die "Missing required value $var in config file $config_file";
+		return $default;
+	}
+	return $config{"$var"};
+}
+
+sub read_config_array {
+	my ($var) = @_;
+	return split(/\s+/, read_config_value($var, ""));
+}
+
+sub server_config_value {
+	my ($server, $var_prefix, $var_default, $default) = @_;
+	my $v = $default;
+	if (defined($config{$var_default ? $var_default : $var_prefix})) {
+		$v = $config{$var_default ? $var_default : $var_prefix};
+	}
+	my $x = $var_prefix;
+	foreach my $y (split(/_/, $server)) {
+		$x = $x."_".$y;
+		if (defined($config{$x})) { $v = $config{$x}; }
+        }
+	return $v;
+}
+			    
 ########################################################
 # Misc. funtions
 
@@ -84,7 +146,7 @@ Options:
 		For special purposes only.
 --[no]expand	If noexpand specified, the date won't be appended to <prefix>,
 		computation will hold only to the file <prefix>. Default is expand.\
---[no]tmpdel	If notmpdel specified, temporary files will not be deleted and 
+--[no]tmpdel	If notmpdel specified, temporary files will not be deleted and
 		their names will be printed instead.
 "
 }
@@ -111,18 +173,19 @@ sub compute_stat {
 	$begin-=$begin % $quantum;
 	$end+= $quantum - ($end % $quantum);
 
+	$error_time = $begin;
+
 	foreach my $file ($stat_expanddays ? expand_days($begin,$end) : "") {
 
 		next unless -e "$prefix$file";
-		
+
 		my $f = new IO::File "$prefix$file" or die "Can't open $prefix$file";
 		my %row = ();
 		my $l;
 		my $file_type=1;
 		my $last_time;
 		my $count =0;
-  		
-		
+
 		# skip beginning of the file
 		while($l=<$f>) {
 			#$count++;
@@ -146,7 +209,7 @@ sub compute_stat {
 				}
 			}
 		}
-  		
+
 		# process lines with apropriate timestamp
 		do {
 			#print "chewing $prefix$file [$count] $l";
@@ -197,7 +260,7 @@ sub expand_days {
 	my @p = localtime($end);
 	my $k = sprintf("%04d%02d%02d",$p[5]+1900,$p[4]+1,$p[3]);
 	my $day;
-	
+
 	do {
 		my @a = localtime($b);
 		$day = sprintf("%04d%02d%02d",$a[5]+1900,$a[4]+1,$a[3]);
@@ -310,7 +373,7 @@ sub header_update {
 	# there may be some troubles with the \n length, assume it is 1
 	$s.="*"x($header_size-1-length($s));
 
-	print $f $s;   
+	print $f $s;
 	seek($f,$curr,0) or die "Can't seek";
 }
 
@@ -353,6 +416,7 @@ sub new {
 "set grid
 set data style lines
 set bmargin 3
+set border 0
 set xdata time
 set timefmt \"%Y:%j:%H:%M\"
 set terminal png
@@ -362,11 +426,12 @@ set output \"$picname\"
 	defined $pictitle and $self->{INIT_CMD} .= "set title \"$pictitle\"\n";
 
 	$self->{PIC_NAME} = $picname;
-	
+
 	$self->{PLOT_FILE} = ();
 	$self->{PLOT_HANDLE} = ();
 	$self->{PLOT_CNT} = 0;
 	$self->{PLOT_USING} = ();
+	$self->{PLOT_TITLE} = ();
 	$self->{PLOT_WITH} = ();
 	$self->{PLOT_COMMAND} = "plot";
 
@@ -390,26 +455,25 @@ sub draw_picture {
 	my $self = shift;
 	my $c;
 	my $size = 0;
-	my $is_data = 0;
 
 	my $f = $self->{CMD_HANDLE};
 	print $f $self->{INIT_CMD};
 
 	for (my $i=1; $i<=$self->{PLOT_CNT}; $i++) {
-		if ($self->{PLOT_HANDLE}{$i}->tell != 0) { $is_data = 1 }
+		if ($self->{PLOT_HANDLE}{$i}->tell == 0) {
+			my $fh = $self->{PLOT_HANDLE}{$i};
+			my $tm = strftime("%Y:%j:%H:%M", localtime($error_time));
+			print $fh "$tm 0 0 0 0 0 0 0 0 0\n";
+		}
 
 		if (defined $c) {
 			$c .= ", ";
 		} else {
 			$c = $self->{PLOT_COMMAND}." ";
 		}
-		$c .= "\"".$self->{PLOT_FILE}{$i}."\" using ".$self->{PLOT_USING}{$i}
+		$c .= "\"".$self->{PLOT_FILE}{$i}."\" using ".$self->{PLOT_USING}{$i}." title \"".$self->{PLOT_TITLE}{$i}."\""
 			.(defined $self->{PLOT_WITH}{$i} ? "with ".$self->{PLOT_WITH}{$i} : "");
 	}
-	if (!$is_data) {
-		my $fh = $self->{PLOT_HANDLE}{1};
-		print $fh "1620:312:12:00 0 0 0 0 0 0 0 0 0\n";
-	}
 	for (my $i=1; $i<=$self->{PLOT_CNT}; $i++) {
 		$self->{PLOT_HANDLE}{$i}->close;
 	}
@@ -443,7 +507,8 @@ sub new_plot {
 	}
 	$self->{PLOT_FILE}{$i} = $nm;
 	$self->{PLOT_HANDLE}{$i} = $f;
-	$self->{PLOT_USING}{$i} = (defined $datafields ? $datafields :"1:2")." title \"$title\"";
+	$self->{PLOT_USING}{$i} = (defined $datafields ? $datafields :"1:2");
+	$self->{PLOT_TITLE}{$i} = "$title";
 
 	return $i;
 }
