@@ -3,6 +3,8 @@
 # Precomputing data from indexer logs
 #
 # (c) 2003-2005 Tomas Valla <tom@ucw.cz>
+# (c) 2006 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
+# (c) 2007 Pavel Charvat <pchar@ucw.cz>
 #
 
 use lib 'lib/perl5';
@@ -13,6 +15,12 @@ use warnings;
 analyze_options() or die
 "Reads Sherlock indexer logfile and generates interfile\n".analyze_usage();
 
+# Recompute whole interfile, because we don't remember which index we are processing
+$force = 1;
+
+my $last_index = '';
+my $sending = 0;
+
 chew_indexerlog($ARGV[0],$ARGV[1]);
 
 sub chew_indexerlog {
@@ -48,6 +56,7 @@ sub chew_indexerlog {
 		lang_fr
 		lang_es
 		lang_it
+		lang_ru
 		indexing_start
 		indexing_end
 		idxcopy_start
@@ -72,6 +81,9 @@ sub chew_indexerlog {
 		size_refs
 		size_string
 		size_total );
+		# chewer_card_*
+		# size_total_*
+		# size_lexicon_*
 
 	my $hdr_string="#@\n";
 
@@ -91,7 +103,7 @@ sub chew_indexerlog {
 		}
 
 		defined $start_time or $start_time = $time - ($time % $quantum);
-		
+
 		if ($time>$start_time+$quantum) {
 			for my $k (sort @keys) {
 				defined $d{$k} and print $out "$start_time\t$k\t$d{$k}\n";
@@ -102,31 +114,28 @@ sub chew_indexerlog {
 			%d=();
 		}
 
-		if (/\[indexer] Building index/) {
+		if (/\[indexer] Processing subindex (\S+)/) {
+			$last_index = $1;
+		}
+		elsif (/\[indexer] Building index/) {
 			$d{'indexing_start'} = $time;
 		}
 		elsif (/\[indexer] Index built successfully/) {
 			$d{'indexing_end'} = $time;
 		}
-		elsif (/\[send-index] Sending index version/) {
-			$d{'idxcopy_start'} = $time;
-		}
-		elsif (/\[send-index] Index transferred/) {
-			$d{'idxcopy_end'} = $time;
-		}
 		elsif (/\[scanner] Scanned (\d+) objects, created (\d+) cards/)	{ # old-style
 			$d{'scan_obj'} = $1;
 			$d{'scan_card'} = $2;
 		}
-		elsif (/\[scanner] Scanned (\d+) objects \((\d+) ok, (\d+) err, (\d+) robots, (\d+) new\)/) {
+		elsif (/\[scanner] Scanned (\d+) objects \((\d+) ok, (\d+) err, (\d+) robots, (\d+) (new|skeletons)\)/) {
 			$d{'scan_obj'} = $1;
 			$d{'scan_doc'} = $2 + $3 + $4;
 		}
 		elsif (/\[scanner] Created (\d+) cards /) {
 			$d{'scan_card'} = $1;
 		}
-		elsif (/\[chewer] Generated (\d+) cards/) {
-			$d{'chewer_card'} = $1;
+		elsif (/\[chewer] ([^:]+): Generated (\d+) cards/) {
+			$d{"chewer_card_$1"} = $2;
 		}
 		elsif (/\[merger] Merged (\d+) cards: (\d+) non-trivial classes \(max \d+\), (\d+) duplicates, (\d+) penalized/) {
 			$d{'merge_card'} = $1;
@@ -146,10 +155,10 @@ sub chew_indexerlog {
 			if (/queue=(\d+)/) { $d{'size_queue'} = $1; }
 			if (/card-attrs=(\d+)/) { $d{'size_attrs'} = $1; }
 			if (/cards=(\d+)/) { $d{'size_cards'} = $1; }
-			if (/lexicon=(\d+)/) { $d{'size_lexicon'} = $1; }
+			if (/lexicon=(\d+)/) { $d{"size_lexicon_$last_index"} = $1; }
 			if (/references=(\d+)/) { $d{'size_refs'} = $1; }
 			if (/string-map=(\d+)/) { $d{'size_string'} = $1; }
-			if (/total index size is (\d+)/) { $d{'size_total'} = $1; }
+			if (/total index size is (\d+)/) { $d{"size_total_$last_index"} = $1; }
 		}
 		elsif (/\[ireport] Filetypes in: unknown=(\d+) html=(\d+) pdf=(\d+) text=(\d+) msword=(\d+) excel=(\d+) RFU6=0 RFU7=0 jpeg=(\d+) png=(\d+) gif=(\d+)/) {
 			$d{'ft_in_unknown'} = $1;
@@ -173,18 +182,28 @@ sub chew_indexerlog {
 			$d{'ft_out_png'} = $8;
 			$d{'ft_out_gif'} = $9;
 		}
-		elsif (/\[ireport] Languages: \?\?=(\d+) en=(\d+) cs=(\d+) sk=(\d+) pl=(\d+) hu=(\d+) de=(\d+) nl=(\d+) fr=(\d+) es=(\d+) it=(\d+)/) {
-			$d{'lang_unknown'} = $1;
-			$d{'lang_en'} = $2;
-			$d{'lang_cs'} = $3;
-			$d{'lang_sk'} = $4;
-			$d{'lang_pl'} = $5;
-			$d{'lang_hu'} = $6;
-			$d{'lang_de'} = $7;
-			$d{'lang_nl'} = $8;
-			$d{'lang_fr'} = $9;
-			$d{'lang_es'} = $10;
-			$d{'lang_it'} = $11;
+		elsif (/\[ireport] Languages: (.+)$/) {
+			my %lang = map { split /=/, $_, 2 } split(/\s+/, $1);
+			$d{'lang_unknown'} = $lang{'??'};
+			$d{'lang_en'} = $lang{'en'};
+			$d{'lang_cs'} = $lang{'cs'};
+			$d{'lang_sk'} = $lang{'sk'};
+			$d{'lang_pl'} = $lang{'pl'};
+			$d{'lang_hu'} = $lang{'hu'};
+			$d{'lang_de'} = $lang{'de'};
+			$d{'lang_nl'} = $lang{'nl'};
+			$d{'lang_fr'} = $lang{'fr'};
+			$d{'lang_es'} = $lang{'es'};
+			$d{'lang_it'} = $lang{'it'};
+			$d{'lang_ru'} = $lang{'ru'};
+		}
+
+		if (!$sending && /\[send-indices]/) {
+			$d{'idxcopy_start'} = $time;
+			$sending = 1;
+		}
+		elsif ($sending && /\[scheduler] All indices send/) {
+			$d{'idxcopy_end'} = $time;
 		}
 	}
 	for my $k (sort @keys) {
